﻿window.MedwebCloud = window.MedwebCloud || {};
// Setup the ExtJS Loader
Ext.Loader.setConfig({
    enabled: true,
    paths: {
        'Ext.ux': '/scripts/app/ux',
    }
});

var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
    autoLoad: true,
    listeners: {
        loaded: function () {
            //Apply the host info to bundle data before using it
            var format = function (str, col) {
                col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                    if (m == "{{") {
                        return "{";
                    } if (m == "}}") {
                        return "}";
                    }
                    return col[n] || '{' + n + '}';
                });
            };
            var hostSubstitution = {}
            Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                this["Host_" + key] = value;
            }, hostSubstitution);

            bundle.each(function (record, idx, count) {
                var newVal = record.get('value');
                record.set('value', format(newVal, hostSubstitution));
            });
            MedwebCloud.getMsg = function () {
                var bdl = bundle;
                return bdl.getMsg.apply(bdl, arguments);
            };

            Ext.onReady(function () {
                Ext.define('MedwebCloud.loginInfo', {
                    extend: 'Ext.data.Model',
                    fields: [
                        { name: 'email' },
                        { name: 'password' }
                    ]
                });
                Ext.define('MedwebCloud.fielderror', {
                    extend: 'Ext.data.Model',
                    fields: ['id', 'msg']
                });

                var w, performLogin;

                var initToken = Ext.History.getToken();

                createAccount = function () {
                    var form = w.down('#loginForm');
                    if (!form.isValid()) {
                        return;
                    }

                    w.mask('Creating account...');
                    w.down('#login').disable();

                    var subdomain = window.location.host.split('.')[0];
                    console.log("Using subdomain: " + subdomain);

                    Ext.Ajax.request({
                        url: MedwebCloud.api('/v1/createaccount'),
                        method: 'POST',
                        jsonData: {
                            Username: w.down('#username').getValue(),
                            Password: w.down('#password').getValue(),
                            ConfirmPassword: w.down('#confirmPassword').getValue(),
                            DisplayName: w.down('#displayName').getValue(),
                            Subdomain: subdomain
                        },
                        success: function (response, options) {
                            var success = w.down("#success");
                            var successMsg = MedwebCloud.getMsg('medweb.create.success.message');
                            var successTitle = MedwebCloud.getMsg('medweb.create.success.title');
                            var successCtaText = MedwebCloud.getMsg('medweb.create.success.callToAction');
                            w.unmask();
                            form.down('#confirmPassword').hide();
                            form.down('#displayName').hide();
                            form.down('#error').hide();
                            form.down('#password').hide();
                            form.down('#username').hide();
                            w.down('#login').hide();
                            success.update(successMsg)
                            success.show();
                            w.setTitle(successTitle);
                            w.down('#already-exists').update(successCtaText);
                            //Ext.Msg.alert('Success', 'Your account has been successfully created. Please check your email to claim your account and log in!', function () {
                            //    window.location = "/Login.aspx";
                            //});
                        },
                        failure: function (response, options) {
                            var responseObj = {};
                            try {    // This will fail if not valid json response
                                responseObj = Ext.decode(response.responseText);
                            } catch (e) { }
                            var error = responseObj || response.responseText,
                                el = w.down('#error');

                            el.update(error);
                            el.show();

                            w.unmask();
                        },
                        callback: function () {
                            w.down('#login').enable();
                        }
                    });
                };

                handleEnter = function (field, e) {
                    if (e.getKey() == e.ENTER) {
                        createAccount();
                    }
                };
                var windowTitle = MedwebCloud.getMsg('medweb.create.window.title');
                var labelDisplayName = MedwebCloud.getMsg('medweb.user.displayname.label');
                var labelEmail = MedwebCloud.getMsg('medweb.user.email.label');
                var labelPassword = MedwebCloud.getMsg('medweb.user.password.label');
                var labelPasswordConfirm = MedwebCloud.getMsg('medweb.user.password.confirm.label');

                var labelPositiveButton = MedwebCloud.getMsg('medweb.create.button.positive.label');
                var labelExistingAcct = MedwebCloud.getMsg('medweb.common.account.existing.question');
                w = Ext.create('Ext.window.Window', {
                    title: windowTitle,
                    closable: false,
                    resizable: false,
                    draggable: false,
                    layout: 'fit',
                    width: 350,
                    border: false,
                    items: [{
                        xtype: 'form',
                        itemId: 'loginForm',
                        defaults: {
                            margin: 10,
                            anchor: '100%'
                        },
                        fieldDefaults: {
                            labelWidth: 120
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />' + (window.MWError ? '<br/><br/>' + window.MWError : ''),
                        }, {
                            xtype: 'textfield',
                            itemId: 'username',
                            fieldLabel: labelEmail,
                            allowBlank: false,
                            vtype: 'email',
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'displayName',
                            fieldLabel: labelDisplayName,
                            hidden: window.MWError ? true : false,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'password',
                            inputType: 'password',
                            fieldLabel: labelPassword,
                            //width: 285,
                            hidden: window.MWError ? true : false,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'confirmPassword',
                            inputType: 'password',
                            fieldLabel: labelPasswordConfirm,
                            hidden: window.MWError ? true : false,
                            //width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'error',
                            border: false,
                            bodyCls: 'error'
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'success',
                            border: false,
                            bodyCls: 'success'
                        }],
                        // configure how to read the XML data
                        reader: Ext.create('Ext.data.reader.Json', {
                            model: 'MedwebCloud.loginInfo',
                            record: 'data',
                            successProperty: 'success'
                        })
                    }],
                    buttons: [{
                        xtype: 'box',
                        autoEl: { tag: 'a', href: '/Login.aspx', html: labelExistingAcct, cls: 'login-link' },
                        id: 'already-exists'
                    }, '->', {
                        text: labelPositiveButton,
                        itemId: 'login',
                        handler: createAccount
                    }]
                }).show();

                // make sure it stays centered
                Ext.EventManager.onWindowResize(function () {
                    w.center();
                    //dlg is not defined
                    //if (dlg && dlg.isVisible()) {
                    //    dlg.center();
                    //}
                });
                MedwebCloud.ready = true;
            });
        }
    }
}, MedwebCloud.bundle));