﻿window.MedwebCloud = window.MedwebCloud || {};

var bundle = Ext.create('Ext.ux.i18n.Bundle', Ext.apply({
    autoLoad: true,
    listeners: {
        loaded: function () {
            //Apply the host info to bundle data before using it
            var format = function (str, col) {
                col = typeof col === 'object' ? col : Array.prototype.slice.call(arguments, 1);
                return str.replace(/\{\{|\}\}|\{([\w_]+)\}/g, function substitute(m, n) {
                    if (m == "{{") {
                        return "{";
                    } if (m == "}}") {
                        return "}";
                    }
                    return col[n] || '{' + n + '}';
                });
            };
            var hostSubstitution = {}
            Ext.Object.each(MedwebCloud.UIConfig.HostContext || {}, function (key, value, all) {
                this["Host_" + key] = value;
            }, hostSubstitution);

            bundle.each(function (record, idx, count) {
                var newVal = record.get('value');
                record.set('value', format(newVal, hostSubstitution));
            });
            MedwebCloud.getMsg = function () {
                var bdl = bundle;
                return bdl.getMsg.apply(bdl, arguments);
            };

            Ext.onReady(function () {
                Ext.define('MedwebCloud.loginInfo', {
                    extend: 'Ext.data.Model',
                    fields: [
                        { name: 'email' },
                        { name: 'password' }
                    ]
                });
                Ext.define('MedwebCloud.fielderror', {
                    extend: 'Ext.data.Model',
                    fields: ['id', 'msg']
                });

                var w, performLogin;

                var initToken = Ext.History.getToken();

                var performLogin = function () {
                    var form = w.down('#loginForm');

                    if (!form.isValid()) {
                        return;
                    }

                    var performMask = MedwebCloud.getMsg('medweb.login.perform.mask.message');
                    w.mask(performMask);
                    w.down('#login').disable();

                    Ext.Ajax.request({
                        url: MedwebCloud.apiPath + '/v1/authenticate',
                        withCredentials: true,
                        method: 'POST',
                        jsonData: {
                            Username: w.down('#username').getValue(),
                            Password: w.down('#password').getValue()
                        },
                        success: function (response, options) {
                            //form.submit();
                            //window.location.href = '/Default.aspx' + ((initToken != null) ? '#' + initToken : '');
                            Ext.get('username').dom.value = w.down('#username').getValue();
                            Ext.get('password').dom.value = w.down('#password').getValue();
                            Ext.get('submit').dom.click();
                        },
                        failure: function (response, options) {
                            var responseObj = {};
                            try {    // This will fail if not valid json response
                                responseObj = Ext.decode(response.responseText);
                            } catch (e) { }
                            var error = responseObj || response.responseText,
                                el = w.down('#error');

                            el.update(error);
                            el.show();

                            w.unmask();
                        },
                        callback: function () {
                            w.down('#login').enable();
                        }
                    });
                },

                resetPassword = function (dialogInstance) {
                    window.location.href = '/ForgotPassword.aspx';
                },

                handleEnter = function (field, e) {
                    if (e.getKey() == e.ENTER) {
                        performLogin();
                    }
                };

                var windowTitle = MedwebCloud.getMsg('medweb.login.window.title');

                var labelEmail = MedwebCloud.getMsg('medweb.user.email.label');
                var labelPassword = MedwebCloud.getMsg('medweb.user.password.label');

                var labelCreateAcct = MedwebCloud.getMsg('medweb.login.button.create.label');
                var labelResetPass = MedwebCloud.getMsg('medweb.login.button.reset.label');
                var labelPositiveButton = MedwebCloud.getMsg('medweb.login.button.positive.label');

                w = Ext.create('Ext.window.Window', {
                    title: windowTitle,
                    closable: false,
                    resizable: false,
                    draggable: false,
                    layout: 'fit',
                    border: false,
                    items: [{
                        xtype: 'form',
                        itemId: 'loginForm',
                        defaults: {
                            margin: 10
                        },
                        fieldDefaults: {
                            labelWidth: 80
                        },
                        items: [{
                            xtype: 'panel',
                            border: false,
                            layout: 'fit',
                            style: 'text-align:center',
                            html: '<img src="/images/logo.png" width="235px" height="78px" border="0" alt="Login logo" />'
                        }, {
                            xtype: 'textfield',
                            itemId: 'username',
                            fieldLabel: labelEmail,
                            value: MedwebCloud.getURLParameter('Username') || '',
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'textfield',
                            itemId: 'password',
                            inputType: 'password',
                            fieldLabel: labelPassword,
                            //labelStyle: 'width:50px;',
                            width: 285,
                            allowBlank: false,
                            listeners: {
                                specialkey: handleEnter
                            }
                        }, {
                            xtype: 'panel',
                            hidden: true,
                            itemId: 'error',
                            border: false,
                            bodyCls: 'error'
                        }],
                        // configure how to read the XML data
                        reader: Ext.create('Ext.data.reader.Json', {
                            model: 'MedwebCloud.loginInfo',
                            record: 'data',
                            successProperty: 'success'
                        }),
                        // configure how to read the XML errors
                        errorReader: Ext.create('Ext.data.reader.Json', {
                            model: 'MedwebCloud.fielderror',
                            record: 'errors',
                            successProperty: 'success'
                        })
                    }],
                    buttons: [{
                        xtype: 'box',
                        autoEl: { tag: 'a', href: '/Create.aspx', html: labelCreateAcct, cls: 'login-link' },
                        id: 'create-account'
                    }, '-', {
                        xtype: 'box',
                        autoEl: { tag: 'a', href: '/ForgotPassword.aspx', html: labelResetPass, cls: 'login-link' },
                        id: 'forgot-password'
                    }, '->', {
                        text: labelPositiveButton,
                        itemId: 'login',
                        handler: performLogin
                    }]
                }).show();

                // make sure it stays centered
                Ext.EventManager.onWindowResize(function () {
                    w.center();
                });

                var remember = window.setInterval(function () {
                    if (Ext.get('username').dom.value && Ext.get('password').dom.value) {
                        w.down('#username').setValue(Ext.get('username').dom.value);
                        w.down('#password').setValue(Ext.get('password').dom.value);

                        w.down('#username').focus();

                        window.clearInterval(remember);
                    }
                }, 50);
            });
        }
    }
}, MedwebCloud.bundle));