﻿Ext.define('MedwebCloud.Viewport', {
    extend: 'Ext.container.Viewport',
    defaultSection: 'collage',
    defaultCollageSection: 'devices',
    requires: [
        'Ext.layout.container.Border',
        'MedwebCloud.view.CollagePanel',
        'MedwebCloud.view.UserManagementPanel',
        'MedwebCloud.view.UsersGrid',
        'MedwebCloud.view.BuddyList',
        'MedwebCloud.view.MyMedwebPanel',
        'MedwebCloud.view.WaitingRoomPanel',
        'MedwebCloud.view.UserEditor',
        'MedwebCloud.view.AdminPanel'
    ],
    initHistory: function () {
        var me = this;

        var cookieHash = Ext.util.Cookies.get('history_token');
        Ext.util.Cookies.clear('history_token');

        Ext.History.init();
        Ext.History.on('change', function (token) {
            var parts, length, i;

            if (token) {
                me.applyHistoryToken(token);
            }
        });

        // from the hash in the url, then the hash in the cookie, then the default for the company
        this.applyHistoryToken(Ext.History.getToken() || cookieHash || this.defaultSection); // TODO: set the default token based on the web config, then based on the company config
    },
    applyHistoryToken: function (token) {
        if (!token) {
            return;
        }
        var components = token.split('/');

        if (components.length > 0) {
            this.goToSection({
                name: components[0],
                subsection: {
                    name: components[1],
                    ref: components[2]
                }
            });
        }
    },
    goToSection: function (config) {

        /*
        { 
            name: 'collage',
            subsection: {
                name: 'session',
                ref: '10'
            }
        }
        */
        var me = this, tokenParts = [];

        // debugger;
        //Check to see if anything is different:
        var newConfig = Ext.Object.merge({section:config.name}, config.subsection);
        var prevConfig = this.currentConfig || {};
        if (Ext.Object.equals(newConfig, prevConfig)) {
            return; //nothing new to do
        } else {
            this.currentConfig = newConfig;
        }


        if (this.currentSection && this.currentSection != config.name) {
            Ext.eventBus.fireEvent('section:' + this.currentSection + ':deactivated');
        }

        var target = me.down('#maintb').down('button[name=' + config.name + ']');
        if (!target) {
            console.warn('Invalid path: ', config);
            return;
        }

        var all = me.down('#maintb').down('button');
        for (var i = 0; i < all.length; i++) {
            all[i].toggle(false);
        }
            
        if (target.toggle) {
            target.toggle(true);
        }
        var item = me.down('#center').down('panel[name=' + config.name + ']');
        me.down('#center').getLayout().setActiveItem(item);
        item.doLayout();
        tokenParts.push(config.name);

        if (this.currentSection != config.name) {
            Ext.eventBus.fireEvent('section:' + config.name + ':activated');
        }
        this.currentSection = config.name;

        if (config.subsection && config.subsection.name) {
            if (item.setActiveSection) {
                item.setActiveSection(config.subsection);

                tokenParts.push(config.subsection.name);
                tokenParts.push(config.subsection.ref);
            }
        }
        
        var newToken = tokenParts.join('/');
        var oldToken = Ext.History.getToken();
        if (oldToken === null || oldToken.search(newToken) === -1) {
            Ext.History.add(newToken);
        }
    },
    initComponent: function () {
        var me = this;
        var buttonCollage = MedwebCloud.getMsg('medweb.viewport.button.collage.label');
        var buttonEvisit = MedwebCloud.getMsg('medweb.viewport.button.evisit.label');
        var buttonMyMedweb = MedwebCloud.getMsg('medweb.viewport.button.mymedweb');
        var menuSetting = MedwebCloud.getMsg('medweb.viewport.menu.setting.label');
        var windowSettingsTitle = MedwebCloud.getMsg('medweb.settings.window.title');
        var windowSettingsPositive = MedwebCloud.getMsg('medweb.settings.button.positive.label');
        var windowSettingsNegative = MedwebCloud.getMsg('medweb.settings.button.negative.label');

        var buttonAdmin = MedwebCloud.getMsg('medweb.viewport.button.admin');
        var menuManageUsers = MedwebCloud.getMsg('medweb.viewport.menu.manageusers.label');
        var windowManageUsersTitle = MedwebCloud.getMsg('medweb.manageusers.window.title');
        var buttonLogout = MedwebCloud.getMsg('medweb.viewport.button.logout');

        var regionBuddyListTitle = MedwebCloud.getMsg('medweb.viewport.buddylist.title');

        var modules = function (m) {
            return MedwebCloud.UIConfig.AvailableModules.indexOf(m.toLowerCase()) > -1;
        };
        /*var pressCollage = modules('collage');
        var pressInvite = !pressCollage && modules('evisit');
        var pressMyMedweb = !pressCollage && !pressInvite && modules('mymedweb');
        var pressGlass = !pressCollage && !pressInvite && !pressMyMedweb && modules('lookingglass');
        var pressAdmin = false;
        var currentSection = pressCollage ? 'collage' : (pressInvite ? 'evisit' : 'mymedweb');

        goToSection({ name: currentSection });*/

        var goToSection = this.goToSection.bind(this);
        

        Ext.apply(this, {
            layout: 'border',
            items: [{
                xtype: 'toolbar',
                region: 'north',
                itemId: 'maintb',
                border: false,
                items: [{
                    text: 'Logo',
                    xtype: 'box',
                    layout: 'fit',
                    style: 'text-align:center;margin:5px',
                    html: '<img src="/images/logo-small.png" width="151px" height="50px" border="0" alt="Login logo" />'
                }, {
                    xtype: 'button',
                    scale: 'large',
                    text: buttonCollage,
                    hidden: !modules('collage'),
                    icon: 'images/doctor.png',
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    name: 'collage',
                    handler: goToSection
                }, {
                    xtype: 'button',
                    scale: 'large',
                    text: buttonEvisit,
                    hidden: !modules('evisit'),
                    name: 'evisit',
                    itemId: 'evisit',
                    icon: 'images/clipboard2.png',
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    handler: goToSection
                }, {
                    xtype: 'button',
                    scale: 'large',
                    text: buttonMyMedweb,
                    hidden: !modules('mymedweb'),
                    name: 'mymedweb',
                    icon: 'images/medical_bag.png',
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    handler: goToSection
                }, {
                    xtype: 'button',
                    icon: 'images/safety_glasses.png',
                    text: 'Looking Glass',
                    scale: 'large',
                    hidden: !modules('lookingglass'),
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    name: 'lookingglass',
                    handler: goToSection
                }, {
                    xtype: 'button',
                    icon: 'images/thermometer2.png',
                    text: 'Vitals+',
                    scale: 'large',
                    hidden: !modules('vitalsplus'),
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    name: 'vitalsplus',
                    handler: goToSection
                }, '->', {
                    text: MedwebCloud.userProfile.DisplayName,
                    //scale: 'large',
                    //iconCls: 'presence active',
                    menu: {
                        items: [/*{
                            text: 'Available',
                            //icon: 'images/user.png',
                            iconCls: 'presence active',
                            handler: function () {
                                
                            }
                        }, {
                            text: 'Busy',
                            //icon: 'images/user.png',
                            iconCls: 'presence busy',
                            handler: function () {

                            }
                        },'-', */{
                            text: menuSetting,
                            //icon: 'images/user.png',
                            //iconCls: 'active',
                            icon: 'images/cog.png',
                            handler: function () {
                                var w = new Ext.Window({
                                    title: windowSettingsTitle,
                                    layout: 'fit',
                                    border: false,
                                    modal: true,
                                    //margin: 10,
                                    //height: 500,
                                    width: 500,
                                    items: [{
                                        bodyPadding: 10,
                                        xtype: 'MedwebCloud.usereditor',
                                        recordId: MedwebCloud.userProfile.Id
                                    }],
                                    buttons: [{
                                        text: windowSettingsPositive,
                                        handler: function () {
                                            var form = w.down('form');
                                            if (!form.isValid()) {
                                                return;
                                            }
                                            var settingsSaveMask = MedwebCloud.getMsg('medweb.settings.perform.mask.message');
                                            Ext.Msg.wait(settingsSaveMask);
                                            
                                            var record = form.getRecord();
                                            record.set(form.getForm().getFieldValues());
                                            record.save({
                                                success: function () {
                                                    Ext.Msg.hide();
                                                    w.close();
                                                },
                                                failure: function (user, response) {
                                                    var alertTitle = MedwebCloud.getMsg('medweb.settings.error.title');
                                                    var alertMessage = MedwebCloud.getMsg('medweb.settings.error.message', response.error.statusText);

                                                    Ext.Msg.alert(alertTitle, alertMessage);
                                                }
                                            });
                                        }
                                    }, {
                                        text: windowSettingsNegative,
                                        handler: function () {
                                            w.close();
                                        }
                                    }]
                                });
                                w.show();
                            }
                        }]
                    }
                }, {
                    xtype: 'button',
                    icon: '',
                    hidden:true,
                    text: 'TestAuthGet',
                    //hidden: !MedwebCloud.userProfile.IsAdmin,
                    handler: function () {

                        
                        //https://www.googleapis.com/calendar/v3/users/me/calendarList
                        Ext.Ajax.request({
                            url: MedwebCloud.api('/oauth/externalresource?query=' + encodeURIComponent('https://test.medweb.org/api/patients')),   //'https://git.medweb.net/api/v3/user')),
                            withCredentials: true,
                            method: 'GET',
                            headers: { 'Accept': 'application/json, */*' },
                            jsonData: {},
                            success: function (response) {
                                console.log("Calendar request succeeded.");
                                console.log(response);
                            },
                            failure: function (response) {
                                console.log("Calendar request failed.");
                                console.log(response);
                                if (response && response.responseText)
                                {
                                    try
                                    {
                                        var obj = JSON.parse(response.responseText);
                                        if (obj.status && obj.status === 'require_auth' && obj.url) {
                                            document.cookie = 'auth_redirect=' + window.location;// + '?action=currentAction';
                                            window.location = obj.url;
                                        }
                                    }
                                    catch (ex)
                                    {
                                        console.log("Failure: " + ex);
                                    }
                                }
                            }
                        });
                    }
                }, {
                    xtype: 'button',
                    icon: '',
                    hidden:true,
                    text: 'TestAuthPost',
                    //hidden: !MedwebCloud.userProfile.IsAdmin,
                    handler: function () {


                        //https://www.googleapis.com/calendar/v3/users/me/calendarList
                        Ext.Ajax.request({
                            url: MedwebCloud.api('/oauth/externalresource?query=' + encodeURIComponent('https://test.medweb.org/api/patient')),   //'https://git.medweb.net/api/v3/user')),
                            withCredentials: true,
                            method: 'POST',
                            headers: { 'Accept': 'application/json, */*' },
                            jsonData: {
                                firstName: 'test' + Math.random(),
                                lastName: 'PostOAuth',
                                dob: '19990203',
                                sex: 'male',
                            },
                            success: function (response) {
                                console.log("Post request succeeded.");
                                console.log(response);
                            },
                            failure: function (response) {
                                console.log("Post request failed.");
                                console.log(response);
                                if (response && response.responseText) {
                                    try {
                                        var obj = JSON.parse(response.responseText);
                                        if (obj.status && obj.status === 'require_auth' && obj.url) {
                                            document.cookie = 'auth_redirect=' + window.location;// + '?action=currentAction';
                                            window.location = obj.url;
                                        }
                                    }
                                    catch (ex) {
                                        console.log("Failure: " + ex);
                                    }
                                }
                            }
                        });
                    }
                }, {
                    xtype: 'button',
                    icon: 'images/config16.png',
                    text: buttonAdmin,
                    //scale: 'large',
                    name: 'admin',
                    hidden: !MedwebCloud.userProfile.IsAdmin,
                    enableToggle: true,
                    toggleGroup: 'maintb',
                    handler: goToSection
                    /*menu: {
                        items: [{
                            text: menuManageUsers,
                            icon: 'images/user.png',
                            handler: function () {
                                var w = new Ext.Window({
                                    title: windowManageUsersTitle,
                                    modal: true,
                                    width: 1000,
                                    height: 700,
                                    layout: 'fit',
                                    border: false,
                                    items: [{
                                        xtype: 'MedwebCloud.usermanagementpanel'
                                    }]
                                });

                                w.show();
                            }
                        }, {
                            text: 'Manage Licenses',
                            icon: 'images/license.png',
                            handler: function () {
                                var w = new Ext.Window({
                                    title: 'Licenses',
                                    modal: true,
                                    width: 1000,
                                    height: 700,
                                    layout: 'fit',
                                    border: false,
                                    items: [{
                                        xtype: 'medwebcloud.licensetab'
                                    }]
                                });

                                w.show();
                            }
                        }]
                    }*/
                }, {
                    text: buttonLogout,
                    //scale: 'large',
                    icon: 'images/exit16.png',
                    style: {
                        marginRight: '15px'
                    },
                    handler: MedwebCloud.app.logout
                }]
            }, {
                region: 'center',
                xtype: 'panel',
                itemId: 'main_app',
                layout: 'border',
                activeItem: 0,
                border: false,
                items: [{
                    region: 'east',
                    title: regionBuddyListTitle,
                    width: 250,
                    collapsed: true,
                    collapsible: true,
                    split: true,
                    stateful: true,
                    stateId: 'buddylist',
                    hidden: !modules('internalchat'),
                    xtype: 'MedwebCloud.buddylist',
                    client: MedwebCloud.websyncClient
                },{
                    region: 'center',
                    layout: 'card',
                    activeItem: 0,
                    border: false,
                    itemId: 'center',
                    id: 'main_center',
                    items: [{
                        border: false,
                        xtype: 'MedwebCloud.collagepanel',
                        name: 'collage',
                        defaultSection: this.defaultCollageSection,
                        client: MedwebCloud.websyncClient
                    }, {
                        border: false,
                        xtype: 'MedwebCloud.waitingroompanel',
                        name: 'evisit',
                        client: MedwebCloud.websyncClient,
                        listeners: {
                            chatstarted: function () {
                                me.down('#center').getLayout().setActiveItem(1);
                                me.down('#evisit').toggle(true);
                            }
                        }
                    }, {
                        border: false,
                        xtype: 'MedwebCloud.mymedwebpanel',
                        name: 'mymedweb',
                        client: MedwebCloud.websyncClient
                    }, {
                        border: false,
                        name: 'lookingglass',
                        xtype: 'medwebcloud.lookingglasstab',
                        client: MedwebCloud.websyncClient
                    }, {
                        border: false,
                        name: 'vitalsplus',
                        xtype: 'medwebcloud.vitalsplustab',
                        client: MedwebCloud.websyncClient
                    }, {
                        border: false,
                        name: 'admin',
                        xtype: 'medwebcloud.adminpanel',
                        client: MedwebCloud.websyncClient
                    }]
                }]
            }]
        });

        this.callParent(arguments);

        // determine the panel we actually want to see
        this.on('afterrender', this.initHistory.bind(this));

    },
    onResize: function () {
        Ext.eventBus.fireEvent('window:resize');
    },
    showMsg: function (msg) {
        var el = Ext.get('app-msg'),
            msgId = Ext.id();

        this.msgId = msgId;
        el.update(msg).show();

        Ext.defer(this.clearMsg, 3000, this, [msgId]);
    },

    clearMsg: function (msgId) {
        if (msgId === this.msgId) {
            Ext.get('app-msg').hide();
        }
    },

    showSettings: function () {

    }
});