﻿Ext.define('MedwebCloud.view.ChatPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.chatpanel',
    record: null,
    client: null,
    subchannel: 'livechat',
    initComponent: function () {
        var me = this;
        Ext.apply(this, {
            border: false,
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            items: [{
                xtype: 'panel',
                itemId: 'chatlog',
                autoScroll: true,

                flex: 2
            }, {
                layout: 'fit',
                flex: 1,
                border: false,
                items: [{
                    xtype: 'textarea',
                    emptyText: 'Type here and hit Enter to send.',
                    enterIsSpecial: true,
                    flex: 1,
                    listeners: {
                        specialkey: function (field, e) {
                            if (e.getKey() == e.ENTER) {
                                me.submitText()
                            }
                        }
                    }
                }]
            }]
        });
        this.callParent(arguments);
        this.join();
    },
    onDestroy: function () {
        this.leave();
        this.callParent(arguments);
    },
    getChannel: function () {
        return '/collage/' + this.record.get('Id') + '/' + this.subchannel;
    },
    join: function () {
        var client = this.client;
        var me = this;
        client.subscribe({
            channel: this.getChannel(),
            onFailure: function (e) {
                Ext.Msg.alert('Error', 'Could not subscribe to ' + e.getChannel() + '. ' + e.getException().message);
            },
            onReceive: function (e) {
                me.appendMessage(e.getData());
            }
        });
    },
    appendMessage: function (data) {
        var me = this;

        var log = me.down('#chatlog');
        // NOTE: if we swap from an hbox layout, this next line will break
        var target = log.getLayout().innerCt;

        var template = new Ext.XTemplate(
            '<div class="message">',
                '<span class="speaker">[{CreatedOn:date("g:i:s a")}] {User.DisplayName}: </span>',
                '<span class="text" id="chatlog_{Id}">{Text}</span>',
            '</div>'
        );
        template.append(target, data);

        var d = log.body.dom;
        if (d) {
            d.scrollTop = d.scrollHeight - d.offsetHeight + 2; // +2 accounts for border...bit hacky, but gets the job done for now
        }
    },
    leave: function () {
        this.client.unsubscribe({
            channel: this.getChannel()
        });
    },
    submitText: function () {
        var client = this.client;
        var item = this.down('.textarea');

        if (!item.getValue()) {
            return;
        }

        var data = {
            SessionId: this.record.get('Id'),
            UserId: MedwebCloud.userProfile.Id,
            Text: item.getValue()
        };
        item.disable();
        client.publish({
            channel: this.getChannel(),
            data: data,
            onSuccess: function () {
                item.setValue('');
                item.enable();
            },
            onFailure: function (e) {
                Ext.Msg.alert('Error', 'The message could not be sent.');
                item.enable();
            }
        });
    }
});