﻿Ext.define('Ext.data.reader.RestJson', {
    extend: 'Ext.data.reader.Json',
    alias: 'reader.restjson',
    getResponseData: function (response) {
        var data, error;

        try {
            data = Ext.decode(response.responseText);
            var dataset = this.readRecords(data);
            var range = response.getResponseHeader('Content-Range');
            if (range) {
                var rangeVals = range.split('=')[1];
                var total = rangeVals.split('/')[1];
                dataset.total = parseInt(total, 10);
            }
            return dataset;
        } catch (ex) {
            error = new Ext.data.ResultSet({
                total: 0,
                count: 0,
                records: [],
                success: false,
                message: ex.message
            });

            this.fireEvent('exception', this, response, error);

            Ext.Logger.warn('Unable to parse the JSON returned by the server');

            return error;
        }
    }
});

Ext.define('Ext.data.proxy.TheRest', {
    requires: ['Ext.util.MixedCollection', 'Ext.Ajax'],
    extend: 'Ext.data.proxy.Server',
    alias: 'proxy.therest',
    alternateClassName: ['Ext.data.TheRestProxy'],

    pageParam: false,   // TheRest client will use the proper HTTP Headers for paging ranges
    startParam: false,
    limitParam: false,  //'count',
    expandParam: false,
    queryParam: false,
    noCache: false,     // TheRest client will add its own cache buster query param

    //locale: null,
    rootUrl: null,

    // Added by Ben to provide default configuration values.
    constructor: function (config) {
        Ext.applyIf(config, {
            appendId: true, //
            pageParam: '', // Defaults to Page, Common doesn't use page so set to blank. (Note Common by Default needs this Blank. Make sure to override this)
            filterParam: 'filters', // Relates to ExtJs4Filter.cs in Common, Matches filterParam.
            directionParam: 'dir', // Matches ExtJs4Sort.cs
            cacheString: '_fm', // Default is _dc, figured while here it was a nice touch :D
            limitParam: 'limit', // Matches TheRestExtensions (NOTE: Common has it set to Count, Needs to be changed here if you use the default common).
            startParam: 'start', // Matches TheRestExtensions
            sortParam: 'sort', // Relates to ExtJS4Sort.cs In Common, Matches sortParam.
            expandParam: 'expand',
            noCache: true,
            timeout: 30000,
            reader: {
                type: 'restjson',
                root: ''
            },
            writer: {
                writeRecordId: true,
                nameProperty: 'name',
                writeAllFields: true
            },
            extraParams: {
                resultFormat: 'ext'
            }
        });

        // TheRest will use our provided 'rootUrl' for the api root, and this proxy will remove the 'rootUrl' from any request paths if present.
        this.rootUrl = config.rootUrl || MedwebCloud.apiPath;
        this.client = new fm.therest.client(this.rootUrl);

        this.client.setRequestTimeout(60000);

        // Call superclass constructor if needed.
        this.callParent(arguments);
    },

    // Added by Ben to add in language headers on our request
    //buildRequest: function (operation) {
    //    var request = this.callParent(arguments),
    //        app = ExpenseTracker.getApplication(),
    //        locale = app.defaultLocale;

    //    if (request.operation.locale) {
    //        locale = request.operation.locale;
    //    }
    //    if (locale) {
    //        request.headers = {
    //            'Accept-Language': locale,
    //            'Content-Language': locale
    //        };
    //    }

    //    return request;
    //},

    // Copied from Rest.js
    /**
    * @property {Object} actionMethods
    * Mapping of action name to HTTP request method. These default to RESTful conventions for the 'create', 'read',
    * 'update' and 'destroy' actions (which map to 'POST', 'GET', 'PUT' and 'DELETE' respectively). This object
    * should not be changed except globally via {@link Ext#override Ext.override} - the {@link #getMethod} function
    * can be overridden instead.
    */
    actionMethods: {
        create: 'POST',
        read: 'GET',
        update: 'PUT',
        destroy: 'DELETE'
    },

    // Copied from Rest.js
    /**
    * @cfg {Boolean} appendId
    * True to automatically append the ID of a Model instance when performing a request based on that single instance.
    * See Rest proxy intro docs for more details. Defaults to true.
    */
    //appendId: false, // I put this above.

    // Copied from Rest.js
    /**
    * @cfg {String} format
    * Optional data format to send to the server when making any request (e.g. 'json'). See the Rest proxy intro docs
    * for full details. Defaults to undefined.
    */

    // Copied from Rest.js
    /**
    * @cfg {Boolean} batchActions
    * True to batch actions of a particular type when synchronizing the store. Defaults to false.
    */
    batchActions: false,

    // Copied from Rest.js
    /**
    * Specialized version of buildUrl that incorporates the {@link #appendId} and {@link #format} options into the
    * generated url. Override this to provide further customizations, but remember to call the superclass buildUrl so
    * that additional parameters like the cache buster string are appended.
    * @param {Object} request
    */
    buildUrl: function (request) {
        var me = this,
            operation = request.operation,
            records = operation.records || [],
            record = records[0],
            format = me.format,
            url = me.getUrl(request),
            id = record ? record.getId() : operation.id;
        /* Edited: I modified this function so you can pass the appendId through the config object of a proxy operation.
                   this is because Extjs 4.2 will not append the id during a Model.Load without it, but having appendId to true
                   will add a 0 to POST events. */
        if (me.appendId && ((id > 0) || (request.params.id)) || request.params.appendId && (id != null) && (id > 0)) {
            if (!url.match(/\/$/)) {
                url += '/';
            }
            if (id > 0) {
                url += id;
            } else {
                url += request.params.id;
            }
        }

        if (format) {
            if (!url.match(/\.$/)) {
                url += '.';
            }

            url += format;
        }

        request.params = request.params || {};

        for (var prop in request.params) {
            if (request.params.hasOwnProperty(prop)) {
                if (url.indexOf('?') > -1) {
                    url += '&';
                } else {
                    url += '?';
                }
                url += prop + '=' + encodeURIComponent(request.params[prop]);
            }
        }

        request.url = url;

        return me.callParent(arguments);
    },

    /**
    * @cfg {Boolean} binary
    * True to request binary data from the server.  This feature requires
    * the use of a binary reader such as {@link Ext.data.amf.Reader AMF Reader}
    */
    binary: false,

    /**
    * @cfg {Object} headers
    * Any headers to add to the Ajax request. Defaults to undefined.
    */
    headers: {
        'X-Medweb-Company': window.location.host.split('.')[0],
    },

    getParams: function (operation, scope) {
        var me = this,
            params = {},
            isDef = Ext.isDefined,
            groupers = operation.groupers,
            sorters = operation.sorters,
            filters = operation.filters,
            page = operation.page,
            start = operation.start,
            limit = operation.limit,
            expand = operation.expand,
            query = operation.query,
            simpleSortMode = me.simpleSortMode,
            simpleGroupMode = me.simpleGroupMode,
            pageParam = me.pageParam,
            startParam = me.startParam,
            limitParam = me.limitParam,
            expandParam = me.expandParam,
            queryParam = me.queryParam,
            groupParam = me.groupParam,
            groupDirectionParam = me.groupDirectionParam,
            sortParam = me.sortParam,
            filterParam = me.filterParam,
            directionParam = me.directionParam,
            hasGroups, index;

        if (pageParam && isDef(page)) {
            params[pageParam] = page;
        }

        if (startParam && isDef(start)) {
            params[startParam] = start;
        }

        if (limitParam && isDef(limit)) {
            params[limitParam] = limit;
        }

        if (expandParam && (isDef(expand) || scope.expand)) {
            params[expandParam] = expand || scope.expand;
        }

        if (queryParam && isDef(query)) {
            params[queryParam] = query;
        }

        hasGroups = groupParam && groupers && groupers.length > 0;
        if (hasGroups) {
            // Grouper is a subclass of sorter, so we can just use the sorter method
            if (simpleGroupMode) {
                params[groupParam] = groupers[0].property;
                params[groupDirectionParam] = groupers[0].direction || 'ASC';
            } else {
                params[groupParam] = me.encodeSorters(groupers);
            }
        }

        if (sortParam && sorters && sorters.length > 0) {
            if (simpleSortMode) {
                index = 0;
                // Group will be included in sorters, so grab the next one
                if (sorters.length > 1 && hasGroups) {
                    index = 1;
                }
                params[sortParam] = sorters[index].property;
                params[directionParam] = sorters[index].direction;
            } else {
                params[sortParam] = me.encodeSorters(sorters);
            }
        }
        if (filterParam && filters && filters.length > 0) {
            params[filterParam] = me.encodeFilters(filters);
        }
        return params;
    },
    doRequest: function (operation, callback, scope) {
        var proxy = this,
            writer = this.getWriter(),
            request = this.buildRequest(operation, scope);

        if (operation.allowWrite()) {
            request = writer.write(request);
        }

        /*
        Ext.apply(request, {
        binary        : this.binary,
        headers       : this.headers,
        timeout       : this.timeout,
        scope         : this,
        callback      : this.createRequestCallback(request, operation, callback, scope),
        method        : this.getMethod(request),
        disableCaching: false // explicitly set it to false, ServerProxy handles caching
        });

        Ext.Ajax.request(request);
        */

        // Remove 'rootUrl' from the generated URL

        if (request.url.indexOf(proxy.rootUrl) === 0) {
            request.path = request.url.substring(proxy.rootUrl.length);
            //request.url = proxy.rootUrl;
            //
            request.url = '/rest.ashx';
        }

        //request.path = request.path + '&resultFormat=ext';
        // make sure params are a url encoded string
        if (Ext.isObject(request.params)) {
            //
            // The proxy sets a parameter to carry the entity ID based upon the Operation's id
            // That partameter name defaults to "id".
            // TreeStore however uses a nodeParam configuration to specify the entity id, which is assigned to the idParam.
            //    proxy.idParam = this.nodeParam;
            //
            // But let's not delete the idParam for TreeStores... check for operation.node which is only present for TreeStore operations.
            //
            // TODO: Check if AppendId = True before performing this. (AppendId = False and This is Removing it from Params.
            if (request.params[proxy.idParam] && !operation.node) {
                // We don't need the record Id as a query string param as we are including it in the request path.
                delete request.params[proxy.idParam];
            }

            //debugger;
            request.encodedParams = Ext.Object.toQueryString(request.params);

            /*if (request.url.indexOf('rest.ashx') > -1) {
                request.path = request.url.slice(9);
            }

            request.path = Ext.urlAppend(request.path, request.encodedParams);*/
        }

        // TheRest - request path
        var sendArgs = {
            path: request.path || request.url,
            headers: Ext.apply({}, request.headers || {}, this.headers),
        };

        // TheRest - request method
        switch (request.action) {
            case 'read':
                sendArgs.method = fm.httpMethod.Get;
                break;
            case 'create':
                sendArgs.method = fm.httpMethod.Post;
                break;
            case 'update':
                sendArgs.method = fm.httpMethod.Put;
                break;
            case 'delete':
            case 'destroy':
                sendArgs.method = fm.httpMethod.Delete;
                break;
        }

        // TheRest - request language
        //var locale = operation.locale;
        //if (!locale) {
        //    locale = WebAdmin.getApplication().defaultLocale;
        //}
        //if (locale) {
        //    sendArgs.language = {
        //        code: locale
        //    };
        //}

        // TheRest - request range
        var range = null;
        if (request.params) {
            var startDefined = typeof request.params.start != 'undefined';
            var countDefined = typeof request.params.count != 'undefined';
            if (startDefined || countDefined) {
                sendArgs.range = {};
                if (startDefined) {
                    sendArgs.range.start = request.params.start;
                }
                if (countDefined) {
                    sendArgs.range.length = request.params.count;
                }
            }
            if (request.params.expand) {
                sendArgs.expand = request.params.expand;
            }

            if (request.params.query) {
                sendArgs.query = request.params.query;
            }
        }

        // TheRest - request data, jsonData
        if (request.data) {
            sendArgs.data = request.data;
        }
        if (request.jsonData) {
            sendArgs.data = request.jsonData;
        }

        // callbacks
        var me = this;
        sendArgs.onSuccess = function (e) {
            var responseHeaders = e.getResponseHeaders();
            me.processResponse(true, operation, request, {
                request: request,
                //requestId: ???
                responseText: e.getResponseDataJson(),
                responseXML: null,
                status: 200,
                statusText: 'OK',
                e: e,
                getResponseHeader: function (headerName) {
                    return responseHeaders.get(headerName);
                },
                getAllResponseHeaders: function () {
                    return responseHeaders.toHash();
                }
            }, callback, scope);
        };
        sendArgs.onFailure = function (e) {
            var responseStatusCode = e.getResponseStatusCode(),
            //responseHeaders = e.getResponseHeaders(),
                responseException = e.getException(),
                responseMsg = (responseException && responseException.message) ? responseException.message : MedwebCloud.getMsg('medweb.restproxy.unknown.error.message'),
                responseJson = JSON.stringify({
                    success: false,
                    message: responseMsg
                });

            console.log(responseJson);
            me.processResponse(false, operation, request, {
                request: request,
                //requestId: ???
                responseText: responseJson,
                responseXML: null,
                status: responseStatusCode,
                statusText: responseException.message
                /*getResponseHeader: function (headerName) {
                return responseHeaders.get(headerName);
                },
                getAllResponseHeaders: function () {
                return responseHeaders.toHash();
                }*/
            }, callback, scope);
        };

        this.client.send(sendArgs);

        return request;
    },

    /**
    * Returns the HTTP method name for a given request. By default this returns based on a lookup on
    * {@link #actionMethods}.
    * @param {Ext.data.Request} request The request object
    * @return {String} The HTTP method to use (should be one of 'GET', 'POST', 'PUT' or 'DELETE')
    */
    getMethod: function (request) {
        return this.actionMethods[request.action];
    },

    buildRequest: function (operation, scope) {
        var me = this,
            // Clone params right now so that they can be mutated at any point further down the call stack
            params = operation.params = Ext.apply({}, operation.params, me.extraParams),
            request;

        //copy any sorters, filters etc into the params so they can be sent over the wire
        Ext.applyIf(params, me.getParams(operation, scope));

        // Set up the entity id parameter according to the configured name.
        // This defaults to "id". But TreeStore has a "nodeParam" configuration which
        // specifies the id parameter name of the node being loaded.
        if (operation.id !== undefined && params[me.idParam] === undefined) {
            params[me.idParam] = operation.id;
        }

        request = new Ext.data.Request({
            params: params,
            action: operation.action,
            records: operation.records,
            operation: operation,
            url: operation.url,

            // this is needed by JsonSimlet in order to properly construct responses for
            // requests from this proxy
            proxy: me
        });
        request.url = me.buildUrl(request);

        /*
         * Save the request on the Operation. Operations don't usually care about Request and Response data, but in the
         * ServerProxy and any of its subclasses we add both request and response as they may be useful for further processing
         */
        operation.request = request;

        return request;
    },

    /**
    * @private
    * TODO: This is currently identical to the JsonPProxy version except for the return function's signature. There is a lot
    * of code duplication inside the returned function so we need to find a way to DRY this up.
    * @param {Ext.data.Request} request The Request object
    * @param {Ext.data.Operation} operation The Operation being executed
    * @param {Function} callback The callback function to be called when the request completes. This is usually the callback
    * passed to doRequest
    * @param {Object} scope The scope in which to execute the callback function
    * @return {Function} The callback function
    */
    createRequestCallback: function (request, operation, callback, scope) {
        var me = this;

        return function (options, success, response) {
            //debugger;
            me.processResponse(success, operation, request, response, callback, scope);
        };
    },

    /**
     * Encodes the array of {@link Ext.util.Filter} objects into a string to be sent in the request url. By default,
     * this simply JSON-encodes the filter data
     * @param {Ext.util.Filter[]} filters The array of {@link Ext.util.Filter Filter} objects
     * @return {String} The encoded filters
     */
    encodeFilters: function (filters) {
        var min = [],
            length = filters.length,
            i = 0;

        for (; i < length; i++) {
            min[i] = {
                property: filters[i].property,
                value: filters[i].value
            };
            if (filters[i].comparison !== undefined && filters[i].comparison !== null) { min[i].comparison = filters[i].comparison; }
            if (filters[i].type !== undefined && filters[i].type !== null) { min[i].type = filters[i].type; }
        }
        return this.applyEncoding(min);
    },
}, function () { });