var silence = window.google ? google.maps.event.removeListener : function () { };

GoogleMapsObject = Class.create({
    initialize: function (target) {
        this._target = target;

        Object.keys(target).each(function (key) {
            var method = target[key];

            if (Object.isFunction(method))
                if (!(key in this))
                    this[key] = method.bind(target);
        }.bind(this))
    },

    trigger: function (signal) {
        return google.maps.event.
          trigger(this._target, signal);
    },

    observe: function (signal, method) {
        return google.maps.event.
          addListener(
          this._target, signal, method);
    },
});

var InfoPanel = Class.create(GoogleMapsObject, {
    initialize: function ($super, cart, options) {
        if (window.google) {
            $super(new google.maps.InfoWindow(options));
        } else {
            $super({});
        }
    },

    on_domready: function (event) {
        var element = this.content;

        var height = this.height
          - InfoPanel.VADJUST
          + InfoPanel.VPADDING;
        var width = this.width
          + InfoPanel.HPADDING;

        var element = element.up().up();

        // some browsers don't hang on
        // to the element's contents
        var scrap = new Element('div');
        scrap.update(this.content);

        element.update(null);

        this.panel = new Ext.Panel({
            height: height, width: width,
            title: this._title,
            renderTo: element,
            tools: [{
                id: 'close', handler:
                  this.close.bind(this)
            }],
            listeners: {
                'render':
                  this.on_render.bind(this)
            }
        });
    },

    on_render: function (event) {
        this.rendered = true;
        this.setContent(this.content);
    },

    open: function (cart, position) {
        if (this.rendered) {
            this._target.open(cart, position);

            this.panel.setTitle(this._title);
            var element = this.content;

            var height = element.getHeight();
            this.setHeight(
              height + InfoPanel.VPADDING);
            var width = element.getWidth();
            this.setWidth(
              width + InfoPanel.HPADDING);
        }

        var adjust = this.rendered ?
          0 : InfoPanel.VADJUST;
        this.content.style.height =
          this.content.getHeight() -
          adjust -
          InfoPanel.VPADDING / 2 + 'px';

        var pointer = new Element('img', {
            'class': 'x-info-pointer',
            src: InfoPanel.POINTER
        });
        pointer.style.position = 'relative';
        pointer.style.left = (
          this.content.getWidth() -
          InfoPanel.PWIDTH) / 2 + 'px';
        pointer.style.bottom =
          -InfoPanel.HPADDING / 2 +
          InfoPanel.POFFSET + 'px';
        this.content.insert({ bottom: pointer });

        if (!this.rendered) {
            var listener = this.observe('domready',
            function (event) {
                silence(listener);
                this.on_domready(event);
            }.bind(this));
            this._target.open(cart, position);
        }
    },

    setContent: function (content) {
        this.height = content.getHeight();
        this.width = content.getWidth();
        this.content = content;
        this._target.setContent(content);
    },

    setTitle: function (title) {
        return this._title = title;
    },

    close: function (event) {
        this.rendered = false;
        return this._target.close(event);
    },

    setWidth: function (width) {
        return this.panel.setWidth(width);
    },

    setHeight: function (height) {
        return this.panel.setHeight(height);
    }
});

InfoPanel.VPADDING = 36;
InfoPanel.VADJUST = 26;
InfoPanel.HPADDING = 36;
InfoPanel.PWIDTH = 93;
InfoPanel.POFFSET = -1;
InfoPanel.POINTER =
  '/images/pointer.png';