﻿
Ext.define('MedwebCloud.view.ChatPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.chatpanel',
    record: null,
    client: null,
    subchannel: 'livechat',
    initComponent: function () {
        var me = this;
        var plchldrMessage = MedwebCloud.getMsg('medweb.view.chatpanel.message.placeholder');
        var clearChatLogs = MedwebCloud.getMsg('medweb.view.chatpanel.message.clearchatlogs');
        Ext.apply(this, {
            border: false,
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            bbar: {
                items: ['->',{
                    text: clearChatLogs,
                    handler: function () {
                        var log = me.down('#chatlog');
                        log.getLayout().innerCt.update('');

                        MedwebCloud.model.SessionMessage.destroy({
                            filters: [{
                                'property': 'SessionId', 'value': me.record.get('Id')
                            }]
                        });
                    }
                }]
            },
            items: [{
                xtype: 'panel',
                itemId: 'chatlog',
                autoScroll: true,

                flex: 2
            }, {
                layout: 'fit',
                flex: 1,
                border: false,
                items: [{
                    xtype: 'textarea',
                    emptyText: plchldrMessage,
                    enterIsSpecial: true,
                    flex: 1,
                    listeners: {
                        specialkey: function (field, e) {
                            if (e.getKey() == e.ENTER) {
                                me.submitText()
                            }
                        }
                    }
                }]
            }]
        });
        this.callParent(arguments);
        this.join();
    },
    onDestroy: function () {
        this.leave();
        this.callParent(arguments);
    },
    getChannel: function () {
        return '/collage/' + this.record.get('Id') + '/' + this.subchannel;
    },
    join: function () {
        var client = this.client;
        var me = this;
        client.subscribe({
            channel: this.getChannel(),
            onSuccess: function (e) {
                var messages = e.getExtensionValue('chat.history');
                if (messages) {
                    for (var i = 0; i < messages.length; i++) {
                        me.appendMessage(messages[i]);
                    }
                }
                MedwebCloud.openSessions += 1;
            },
            onFailure: function (e) {
                e.setRetry(true);
                //Ext.Msg.alert('Error', 'Could not subscribe to ' + e.getChannel() + '. ' + e.getException().message);
            },
            onReceive: function (e) {
                me.appendMessage(e.getData());
            }
        });
    },
    appendMessage: function (data) {
        var me = this;

        var log = me.down('#chatlog');
        // NOTE: if we swap from an hbox layout, this next line will break
        var target = log.getLayout().innerCt;

        if (data.Text.replace(/\s/gi, '').length == 0) {
            return;
        }

        data.TextHtml = data.Text.replace(/<br\s*\/?>/gi, "\n").replace(/(\r?\n)+/gi, "<br />");
        //console.log(data);

        // TODO, the date format in this template should come from i18n property.
        var template = new Ext.XTemplate(
            '<div class="message">',
                '<span class="speaker">[{CreatedOn:date("g:i:s a")}] {User.DisplayName}: </span>',
                '<span class="text" id="chatlog_{Id}">{TextHtml}</span>',
            '</div>'
        );
        if (Ext.get('chatlog_' + data.Id)) {
            return;
        }

        template.append(target, data);

        var d = log.body.dom;
        if (d) {
            d.scrollTop = d.scrollHeight - d.offsetHeight + 2; // +2 accounts for border...bit hacky, but gets the job done for now
        }
    },
    leave: function () {
        MedwebCloud.openSessions -= 1;

        this.client.unsubscribe({
            channel: this.getChannel()
        });
    },
    submitText: function () {
        var client = this.client;
        var item = this.down('.textarea');

        if (!item.getValue()) {
            return;
        }

        var data = {
            SessionId: this.record.get('Id'),
            UserId: MedwebCloud.userProfile.Id,
            Text: item.getValue()
        };
        item.disable();
        client.publish({
            channel: this.getChannel(),
            data: data,
            onSuccess: function () {
                item.setValue('');
                item.enable();
            },
            onFailure: function (e) {
                var title = MedwebCloud.getMsg('medweb.view.chatpanel.action.send.error.title');
                var msg = MedwebCloud.getMsg('medweb.view.chatpanel.action.send.error.message');
                Ext.Msg.alert(title, msg);
                item.enable();
            }
        });
    }
});