﻿Ext.define('MedwebCloud.view.CollagePanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.collagepanel',
    requires: ['MedwebCloud.view.SessionPanel', 'MedwebCloud.view.ScheduleSessionPanel'],
    client: null,
    defaultSection: '',
    initComponent: function () {
        var me = this;
        var msg = MedwebCloud.getMsg('medweb.view.collagepanel.sessioncontainer.message');
        var btn = MedwebCloud.getMsg('medweb.view.collagepanel.sessioncontainer.button.label');
        Ext.apply(me, {
            layout: 'fit',
            itemId: 'session-container',
            items: [{
                layout: {
                    type: 'vbox',
                    align: 'center',
                    pack: 'center'
                },
                items: [{
                    border: false,
                    html: msg,
                    padding: 15,
                }, {
                    xtype: 'button',
                    text: btn,
                    scale: 'large',
                    icon: 'images/doctor.png',
                    handler: function () {
                        me.showSessionPicker();
                    }
                }]
            }]
        });

        me.callParent(arguments);
    },
    setActiveSection: function(config){
        // name and ref property
        if (config.ref) {
            var me = this;
            me.joinSession(config.ref, function () { }, function () {
                me.showSessionPicker();
            });
        }
    },
    joinSession: function (id, success, failure) {
        var me = this;
        Ext.Msg.wait(MedwebCloud.getMsg('medweb.common.loading'));
        var Session = Ext.ModelManager.getModel('MedwebCloud.model.Session');
        Session.load(id, {
            failure: function () {
                Ext.Msg.hide();
                failure();
            },
            success: function (session) {
                me.selectSession(session, function () {
                    success();
                });
            }
        });
    },
    showSessionScheduler: function () {
        var me = this;
        var w;
        var failure = function () {
            Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.collagepanel.action.schedule.error.title'), MedwebCloud.getMsg('medweb.view.collagepanel.action.schedule.error.message'));
        };
        var succes = function () {
            w.close();
        };
        var btnText = MedwebCloud.getMsg('medweb.view.collagepanel.action.schedule.button.positive.label');
        w = new Ext.Window({
            center: true,
            height: 600,
            width: 600,
            border: false,
            modal: true,
            layout: 'fit',
            closable: true,
            title: 'Schedule a Session',
            items: [{
                xtype: 'MedwebCloud.schedulesessionpanel',
                itemId: 'schedule'
            }
            ],
            buttons: [{
                text: btnText,
                itemId: 'scheduleButton',
                disabled: true,
                handler: function () {
                    var sp = w.getComponent('schedule');
                    if (sp != null) {
                        sp.createSession();
                    }
                    w.close();
                }
            }]
        });
        w.show();
    },

    showSessionPicker: function () {
        //Ext.ModelManager.getModel('MedwebCloud.v1.model.User').load(1, { success: function () { } })
        var me = this;
        var w;

        var failure = function () {
            Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.collagepanel.action.join.error.title'), MedwebCloud.getMsg('medweb.view.collagepanel.action.join.error.message'));
        };
        var success = function (session) {
            w.close();
        };

        var title = MedwebCloud.getMsg('medweb.view.collagepanel.action.join.window.title');
        var btnSched = MedwebCloud.getMsg('medweb.view.collagepanel.action.join.button.schedule.label');
        var btnNew = MedwebCloud.getMsg('medweb.view.collagepanel.action.join.button.create.label');
        var btnJoin = MedwebCloud.getMsg('medweb.view.collagepanel.action.join.button.join.label');

        // ok, load up the session list as well
        w = new Ext.Window({
            center: true,
            height: 600,
            width: 600,
            border: false,
            modal: true,
            layout: 'fit',
            closable: true,
            title: title,
            items: [{
                xtype: 'MedwebCloud.sessionsgrid',
                listeners: {
                    cellclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                        w.selectedSessionId = record.get('Id');
                        w.down('#join').enable();
                    },
                    celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                        me.joinSession(record.get('Id'), success, failure);
                    }
                }
            }],
            buttons: [{
                text: btnSched,
                itemId: 'schedule',
                handler: function () {
                    me.showSessionScheduler();
                    w.close();
                }
            }, {
                text: btnNew,
                itemId: 'new',
                handler: function () {
                    var session = Ext.create('MedwebCloud.model.Session', {});
                    session.save({
                        failure: function () {
                            Ext.Msg.alert('Error', 'The session could not be created.');
                        },
                        success: function (s) {
                            me.selectSession(s, function () {
                                w.close();
                            });
                        }
                    });
                }
            }, {
                text: btnJoin,
                itemId: 'join',
                disabled: true,
                handler: function () {
                    me.joinSession(w.selectedSessionId, success, failure);
                }
            }]
        });
        w.show();
    },

    selectSession: function (session, callback) {
        var me = this;

        this.insert(0, {
            border: false,
            xtype: 'MedwebCloud.sessionpanel',
            defaultSection: this.defaultSection,
            client: this.client,
            listeners: {
                'leave': function () {
                    me.remove(me.items.getAt(0), true);
                    //me.showSessionPicker();
                }
            },
            record: session
        });
        me.doLayout();
        callback();
    }
});