﻿Ext.define('MedwebCloud.view.AddCompanyForm2', {
    xtype: 'medwebcloud.addcompanyform2',
    extend: 'Ext.form.Panel',
    initComponent: function () {
        var utils = MedwebCloud.util.ViewUtils;
        var me = this;
        me.recordId = me.record.get("Id");

        me.hosts = Ext.create('MedwebCloud.store.Hosts');
        me.hosts.load();

        var sections = utils.getModuleStore();
        function moduleFilter(valid) {
            return {
                filterFn: function (item) {
                    return valid.indexOf(item.get("value")) > -1;
                }
            };
        }

        sections.addFilter(moduleFilter(utils.getAllModules()));

        var hostCombo = Ext.create('Ext.form.ComboBox', me.getField('HostId', Ext.apply(utils.getFormDefaults(), {
            xtype: 'combo',
            queryMode: 'remote',
            store: me.hosts,
            displayField: 'Name',
            valueField: 'Id',
            autoSelect: true,
            forceSelection: true,
            editable: false,
            allowBlank: true,
            listeners: {
                beforequery: function (qe) {
                    qe.combo.lastQuery = null;
                },
                change: function (combo, newVal, oldVal, eOpts) {
                    if (!!newVal) {
                        var host = me.hosts.getById(newVal);
                        var availableModules = host.get("ModuleData");
                        sections.clearFilter();
                        sections.filter(moduleFilter(availableModules));

                        var incCollage = availableModules.indexOf('collage') > -1;
                        var incEVisit = availableModules.indexOf("evisit") > -1;

                        var defSection = me.getComponent('DefaultSection');
                        if (availableModules.indexOf(defSection.getValue()) < 0) {
                            defSection.setValue('');
                        }

                        var defCollage = me.getComponent("DefaultCollageSection");
                        defCollage.setDisabled(!incCollage);
                        defCollage.setVisible(incCollage);

                        var wait = me.getComponent("WaitingRoom");
                        wait.setDisabled(!incEVisit);
                        wait.setVisible(incEVisit);

                        var tferMod = wait.getComponent("TransferMode");
                        tferMod.setDisabled(!incCollage || !incEVisit);
                        tferMod.setVisible(incCollage && incEVisit);
                    }
                }
            }
        })));

        var sectionData = [];
        for (var i = 0; i < MedwebCloud.UIConfig.CollageSections.length; i++) {
            var s = MedwebCloud.UIConfig.CollageSections[i];
            sectionData.push(s);
        }
        var collageSections = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: sectionData
        });

        var transferModesData = [];
        var m = MedwebCloud.UIConfig.TransferModes.sort();
        for (var i = 0; i < m.length; i++) {
            var m = MedwebCloud.UIConfig.TransferModes[i];
            transferModesData.push({ Name: m, DisplayName: m.charAt(0).toUpperCase() + m.substring(1) });
        }
        var transferModes = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: transferModesData
        });

        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();

        Ext.apply(me, {
            layout: utils.getFormLayout(),
            defaults: utils.getFormDefaults(),
            items: [{
                fieldLabel: 'Name',
                name: 'Name',
                xtype: 'textfield',
                allowBlank: false
            }, {
                fieldLabel: 'Code Name',
                name: 'CodeName',
                itemId: 'CodeName',
                xtype: 'textfield',
                disabled: this.recordId > 0 ? true : false,
                allowBlank: true
            }, {
                fieldLabel: 'Subdomain',
                name: 'Subdomain',
                itemId: 'Subdomain',
                xtype: 'textfield',
                allowBlank: false,
            }, {
                fieldLabel: 'Twilio Phone Number',
                name: 'TwilioPhoneNumber',
                itemId: 'TwilioPhoneNumber',
                xtype: 'textfield',
            },
            hostCombo,
            {
                fieldLabel: 'Default Section',
                name: 'DefaultSection',
                itemId: 'DefaultSection',
                xtype: 'combo',
                queryMode: 'local',
                store: sections,
                displayField: 'name',
                valueField: 'value',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                fieldLabel: 'Default Collage Section',
                name: 'DefaultCollageSection',
                itemId: 'DefaultCollageSection',
                xtype: 'combo',
                queryMode: 'local',
                store: collageSections,
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                xtype: 'fieldset',
                itemId: 'WaitingRoom',
                title: 'Waiting Room',
                collapsible: true,
                defaults: {
                    anchor: '100%',
                    labelWidth: 150
                },
                items: [{
                    fieldLabel: 'Logo URL',
                    name: 'LogoUrl',
                    xtype: 'textfield',
                    allowBlank: true,
                    editable: true,
                }, {
                    fieldLabel: 'Custom HTML',
                    name: 'ChatWaitSection',
                    itemId: 'ChatWaitSection',
                    xtype: 'htmleditor',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Allow Video Start',
                    name: 'ChatRemoteCanStartVideo',
                    itemId: 'ChatRemoteCanStartVideo',
                    xtype: 'checkbox',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Transfer Mode',
                    name: 'TransferMode',
                    itemId: 'TransferMode',
                    xtype: 'combo',
                    queryMode: 'local',
                    store: transferModes,
                    displayField: 'DisplayName',
                    valueField: 'Name',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true
                }],
            }]
        });
        this.callParent(arguments);
    },
    getField: function (fieldName, opts) {
        return MedwebCloud.util.ViewUtils.getField(fieldName, opts, 'medweb.view.admin.company.', '.label');
    },
    getLabel: function (fieldName) {
        return MedwebCloud.getMsg('medweb.view.admin.company.' + fieldName.toLowerCase() + '.label');
    }
});