﻿Ext.define('MedwebCloud.view.CompanyForm', {
    xtype: 'medwebcloud.companyform',
    extend: 'MedwebCloud.view.ItemForm',
    initComponent: function () {
        var me = this;
        var utils = MedwebCloud.util.ViewUtils;

        me.hosts = Ext.create('MedwebCloud.store.Hosts');
        me.hosts.load();

        var hostCombo = Ext.create('Ext.form.ComboBox', me.getField('HostId', {
            xtype: 'combo',
            queryMode: 'remote',
            store: me.hosts,
            displayField: 'Name',
            valueField: 'Id',
            autoSelect: true,
            forceSelection: true,
            editable: false,
            allowBlank: true,
            listeners: {
                beforequery: function(qe){
                    qe.combo.lastQuery = null;
                },
                change: function (combo, newval, oldval, eOpts) {
                    var host = me.hosts.getById(newVal);
                    console.log(host);
                }
            }
        }));







        var sections = utils.sectionStore;

        var sectionData = [];
        for (var i = 0; i < MedwebCloud.UIConfig.CollageSections.length; i++) {
            var s = MedwebCloud.UIConfig.CollageSections[i];
            sectionData.push(s);
        }
        var collageSections = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: sectionData
        });

        var transferModesData = [];
        var m = MedwebCloud.UIConfig.TransferModes.sort();
        for (var i = 0; i < m.length; i++) {
            var m = MedwebCloud.UIConfig.TransferModes[i];
            transferModesData.push({ Name: m, DisplayName: m.charAt(0).toUpperCase() + m.substring(1) });
        }
        var transferModes = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: transferModesData
        });

        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();

        

        Ext.apply(me, {
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            defaults: {
                anchor: '100%',
                labelWidth: 150
            },
            items: [{
                fieldLabel: 'Name',
                name: 'Name',
                xtype: 'textfield',
                allowBlank: false

            }, {
                fieldLabel: 'Code Name',
                name: 'CodeName',
                itemId: 'CodeName',
                xtype: 'textfield',
                disabled: this.recordId > 0 ? true : false,
                allowBlank: true
            }, {
                fieldLabel: 'Subdomain',
                name: 'Subdomain',
                itemId: 'Subdomain',
                xtype: 'textfield',
                allowBlank: false,
            }, {
                fieldLabel: 'Twilio Phone Number',
                name: 'TwilioPhoneNumber',
                itemId: 'TwilioPhoneNumber',
                xtype: 'textfield',
            }, hostCombo
            , {
                fieldLabel: 'Default Section',
                name: 'DefaultSection',
                itemId: 'DefaultSection',
                xtype: 'combo',
                queryMode: 'local',
                store: sections,
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                fieldLabel: 'Default Collage Section',
                name: 'DefaultCollageSection',
                itemId: 'DefaultCollageSection',
                xtype: 'combo',
                queryMode: 'local',
                store: collageSections,
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }, {
                xtype: 'fieldset',
                title: 'Waiting Room',
                //collapsed: true,
                collapsible: true,
                defaults: {
                    anchor: '100%',
                    labelWidth: 150
                },
                items : [{
                    fieldLabel: 'Logo URL',
                    name: 'LogoUrl',
                    xtype: 'textfield',
                    allowBlank: true,
                    editable: true,
                }, {
                    fieldLabel: 'Custom HTML',
                    name: 'ChatWaitSection',
                    itemId: 'ChatWaitSection',
                    xtype: 'htmleditor',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Allow Video Start',
                    name: 'ChatRemoteCanStartVideo',
                    itemId: 'ChatRemoteCanStartVideo',
                    xtype: 'checkbox',
                    editable: true,
                    allowBlank: true
                }, {
                    fieldLabel: 'Transfer Mode',
                    name: 'TransferMode',
                    itemId: 'TransferMode',
                    xtype: 'combo',
                    queryMode: 'local',
                    store: transferModes,
                    displayField: 'DisplayName',
                    valueField: 'Name',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true
                }],
            }, {
                xtype: 'tabpanel',
                flex: 1, // full-height
                items: [ {
                    title: 'Users',
                    itemName: 'User',
                    itemsName: 'Users',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.userform2',
                    expands: ['Department'],
                    storeType: 'MedwebCloud.store.Users',
                    layout: 'fit',
                    
                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.recordId
                    }],

                    columns: [{
                        dataIndex: 'DisplayName',
                        text: 'Name',
                        flex: 2,
                    }, {
                        dataIndex: 'Username',
                        text: 'Email',
                        flex: 2,
                    }  , {
                        dataIndex: 'IsDoctor',
                        text: 'Is Doctor',
                        flex: 0,
                        renderer: function (isDoc) {
                            var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                                tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src +'" />';
                            return tag;
                        }
                    }, {
                        dataIndex: 'DepartmentId',
                        text: 'Department',
                        flex: 1,
                        renderer: function (dept, meta, record) {
                            dept = dept || 0;
                            try{
                                var dpt = me.departments.getById(dept);
                                return dpt.get("Name");
                            }
                            catch(ex){
                                console.log(ex);
                                return "";
                            }
                        }
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.recordId);
                    },
                },  {
                    title: 'Affiliates',
                    itemName: 'Affiliate',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.affiliationform2',
                    storeType: 'MedwebCloud.store.Affiliations',
                    editEnabled: false,

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.recordId
                    }],

                    columns: [{
                        dataIndex: 'UserDisplayName',
                        text: 'User',
                        flex: 3,
                    }, {
                        dataIndex: 'UserName',
                        text: 'Username',
                        flex: 3,
                    }, {
                        dataIndex: 'IsDoctor',
                        text: 'Is Doctor',
                        flex: 0,
                        renderer: function (isDoc, meta, record) {
                            console.log(record, record.raw);
                            var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                                tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src + '" />';
                            return tag;
                        }
                    }, {
                        dataIndex: 'DepartmentName',
                        text: 'Department',
                        flex: 3,
                    }, {
                        dataIndex: 'UserCompanyName',
                        text: 'From',
                        flex: 1,
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.recordId);
                    },
                }, {
                    title: 'eVisit Links',
                    itemName: 'Link',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.evisitlinkform2',
                    storeType: 'MedwebCloud.store.EVisitLinks',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.recordId
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 1,
                    }, {
                        dataIndex: 'Url',
                        text: 'Url',
                        flex: 2,
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.recordId);
                    },
                }, {
                    title: 'Email Domains',
                    itemName: 'Domain',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.companydomainform2',
                    storeType: 'MedwebCloud.store.CompanyDomains',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.recordId
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 1,
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.recordId);
                    },
                }, {
                    title: 'Cameras',
                    itemName: 'Camera',
                    xtype: 'medwebcloud.windoweditor',
                    formType: 'medwebcloud.remotecameraform2',
                    storeType: 'MedwebCloud.store.RemoteCameras',

                    filters: [{
                        property: 'CompanyId',
                        operator: '=',
                        value: me.recordId
                    }],

                    columns: [{
                        dataIndex: 'Name',
                        text: 'Name',
                        flex: 2,
                    }, {
                        dataIndex: 'Url',
                        text: 'URL',
                        flex: 2,
                    }],

                    beforeAdd: function (item) {
                        item.set("CompanyId", me.recordId);
                    },
                }]
            }]
        });
        this.callParent(arguments);
    },
    getField: function (fieldName, opts) {
        return MedwebCloud.util.ViewUtils.getField(fieldName, opts, 'medweb.view.admin.company.', '.label');
    },
    getLabel: function (fieldName) {
        return MedwebCloud.getMsg('medweb.view.admin.company.' + fieldName.toLowerCase() + '.label');
    }
});