﻿Ext.define('MedwebCloud.view.CompanyFormAdmin2', {
    xtype: 'medwebcloud.companyformadmin2',
    extend: 'Ext.form.Panel',
    initComponent: function () {
        var utils = MedwebCloud.util.ViewUtils;
        var me = this;
        me.recordId = me.record.get("Id");
        var sections = utils.getModuleStore();
        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();

        var incCollage = true;
        var incEVisit = true;
        var incGlass = true;

        function beforeAddItem(item) {
            item.set("CompanyId", me.record.get("Id"));
        }

        var host = me.record.raw.Host;
        if (!!host) {
            function moduleFilter(valid) {
                return {
                    filterFn: function (item) {
                        return valid.indexOf(item.get("value")) > -1;
                    }
                };
            }
            var modules = host.ModuleData || "";
            modules = modules.split(',');
            sections.addFilter(moduleFilter(modules));
            incCollage = modules.indexOf('collage') > -1;
            incEVisit = modules.indexOf("evisit") > -1;
            incGlass = modules.indexOf("lookingglass") > -1;
        }

        var items = [];
        items.push(me.getField('Name', {
            xtype: 'textfield',
            allowBlank: false
        }));
        items.push(me.getField('CodeName', {
            xtype: 'textfield',
            readonly:true
        }));
        items.push(me.getField('Subdomain', {
            xtype: 'textfield',
            allowBlank: false
        }));
        items.push(me.getField('TwilioPhoneNumber', {
            xtype: 'textfield',
        }));
        items.push(me.getField('DefaultSection', {
            xtype: 'combo',
            queryMode: 'local',
            store: sections,
            displayField: 'name',
            valueField: 'value',
            autoSelect: true,
            forceSelection: true,
            editable: false,
            allowBlank: true
        }));
        if (incCollage) {
            items.push(me.getField('DefaultCollageSection', {
                xtype: 'combo',
                queryMode: 'local',
                store: utils.getCollageSectionStore(),
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                editable: false,
                allowBlank: true
            }));
        }
        if(incEVisit) {
            var waitingRoom = {
                xtype: 'fieldset',
                itemId: 'WaitingRoom',
                title: me.getLabel('WaitingRoom'),
                collapsible: true,
                defaults:utils.getFormDefaults()
            };
            var wrItems = [me.getField("LogoUrl", {
                    xtype: 'textfield',
                    allowBlank: true,
                    editable: true
                }),
                me.getField("ChatWaitSection",{
                    xtype: 'htmleditor',
                    allowBlank: true,
                    editable: true
                }),
                me.getField("ChatRemoteCanStartVideo",{
                    xtype: 'checkbox',
                    editable: true,
                    allowBlank: true
                })
            ];
            if (incCollage) {
                wrItems.push(me.getField("TransferMode",{
                    xtype: 'combo',
                    queryMode: 'local',
                    store: utils.getTransferModeStore(),
                    displayField: 'DisplayName',
                    valueField: 'Name',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true
                }));
            }
            waitingRoom.items = wrItems;
            items.push(waitingRoom);
        }

        //Create TabPanel;

        var tabPanelItems = [{
            title: me.getLabel('UserTab'),
            itemName: 'User',
            xtype: 'medwebcloud.windoweditor',
            formType: 'medwebcloud.userform2',
            expands: ['Department'],
            storeType: 'MedwebCloud.store.Users',
            layout: 'fit',

            filters: [{
                property: 'CompanyId',
                operator: '=',
                value: me.record.get("Id")
            }],
            columns: [{
                dataIndex: 'DisplayName',
                text:  me.getLabel('UserTab.DisplayName'),
                flex: 2,
            }, {
                dataIndex: 'Username',
                text: me.getLabel('UserTab.Username'),
                flex: 2,
            }, {
                dataIndex: 'IsDoctor',
                text: me.getLabel('UserTab.IsDoctor'),
                flex: 0,
                renderer: function (isDoc) {
                    var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                        tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src + '" />';
                    return tag;
                }
            }, {
                dataIndex: 'DepartmentId',
                text: me.getLabel('UserTab.DepartmentId'),
                flex: 1,
                renderer: function (dept, meta, record) {
                    dept = dept || 0;
                    try {
                        var dpt = me.departments.getById(dept);
                        return dpt.get("Name");
                    }
                    catch (ex) {
                        console.log(ex);
                        return "";
                    }
                }
            }],

            beforeAdd: beforeAddItem,
        },{
            title: me.getLabel('AffiliateTab'),
            itemName: 'Affiliate',
            xtype: 'medwebcloud.windoweditor',
            formType: 'medwebcloud.affiliationform2',
            storeType: 'MedwebCloud.store.Affiliations',
            layout: 'fit',
            editEnabled: false,

            filters: [{
                property: 'CompanyId',
                operator: '=',
                value: me.record.get("Id")
            }],

            columns: [{
                dataIndex: 'UserDisplayName',
                text:  me.getLabel('AffiliateTab.UserDisplayName'),
                flex: 3,
            }, {
                dataIndex: 'UserName',
                text: me.getLabel('AffiliateTab.UserName'),
                flex: 3,
            }, {
                dataIndex: 'IsDoctor',
                text:me.getLabel('AffiliateTab.IsDoctor'),
                flex: 0,
                renderer: function (isDoc, meta, record) {
                   // console.log(record, record.raw);
                    var src = isDoc ? "/images/tick.png" : "data:image/gif;base64,R0lGODlhAQABAAAAACwAAAAAAQABAAA=",
                        tag = '<img style="display:block;height:16px;width:16px;margin:0px auto" src="' + src + '" />';
                    return tag;
                }
            }, {
                dataIndex: 'DepartmentName',
                text: me.getLabel('AffiliateTab.DepartmentName'),
                flex: 3,
            }, {
                dataIndex: 'UserCompanyName',
                text:me.getLabel('AffiliateTab.UserCompanyName'),
                flex: 1,
            }],

            beforeAdd: beforeAddItem,
        }, {
            title: me.getLabel('DomainTab'),
            itemName: 'Domain',
            xtype: 'medwebcloud.windoweditor',
            formType: 'medwebcloud.companydomainform2',
            storeType: 'MedwebCloud.store.CompanyDomains',
            layout: 'fit',

            filters: [{
                property: 'CompanyId',
                operator: '=',
                value: me.record.get("Id")
            }],

            columns: [{
                dataIndex: 'Name',
                text:  me.getLabel('DomainTab.Name'),
                flex: 1,
            }],

            beforeAdd: function (item) {
                item.set("CompanyId", me.record.get("Id"));
            },
        }];

        if (incEVisit) {
            tabPanelItems.push({
                title: me.getLabel('LinkTab'),
                itemName: 'Link',
                xtype: 'medwebcloud.windoweditor',
                formType: 'medwebcloud.evisitlinkform2',
                storeType: 'MedwebCloud.store.EVisitLinks',
                layout: 'fit',

                filters: [{
                    property: 'CompanyId',
                    operator: '=',
                    value: me.record.get("Id")
                }],

                columns: [{
                    dataIndex: 'Name',
                    text: me.getLabel('LinkTab.Name'),
                    flex: 1,
                }, {
                    dataIndex: 'Url',
                    text: me.getLabel('LinkTab.Url'),
                    flex: 2,
                }],
                beforeAdd: beforeAddItem,
            });
        }
        if(incCollage) {
            tabPanelItems.push({
                title: me.getLabel('CameraTab'),
                itemName: 'Camera',
                xtype: 'medwebcloud.windoweditor',
                formType: 'medwebcloud.remotecameraform2',
                storeType: 'MedwebCloud.store.RemoteCameras',
                layout: 'fit',

                filters: [{
                    property: 'CompanyId',
                    operator: '=',
                    value: me.record.get("Id")
                }],

                columns: [{
                    dataIndex: 'Name',
                    text: me.getLabel('CameraTab.Name'),
                    flex: 2,
                }, {
                    dataIndex: 'Url',
                    text: me.getLabel('CameraTab.Url'),
                    flex: 2,
                }],

                beforeAdd: beforeAddItem,
            });
        }
        if (incGlass) {
            tabPanelItems.push({
                title: MedwebCloud.getMsg('medweb.view.lookingglass.glasses'),
                itemName: 'Glasses',
                xtype: 'medwebcloud.windoweditor',
                addEnabled: MedwebCloud.userProfile.IsSuperAdmin,
                removeEnabled: MedwebCloud.userProfile.IsSuperAdmin,
                editForm: 'medwebcloud.lookingglassform2',
                addForm: 'medwebcloud.addlookingglassform',
                addButton: {
                    text: MedwebCloud.getMsg('medweb.view.lookingglass.import_glasses'),
                    iconCls: 'importGlassesIcon',
                },
                addTitle: MedwebCloud.getMsg('medweb.view.lookingglass.import_glasses'),
                storeType: 'MedwebCloud.store.Devices',
                layout: 'fit',
                customArgs: {
                    'CompanyId': me.record.get('Id'),
                },

                filters: [{
                    property: 'CompanyId',
                    operator: '=',
                    value: me.record.get("Id")
                }, {
                    property: 'DeviceType',
                    operator: '=',
                    value: MedwebCloud.enums.DeviceType.glasses
                }],

                columns: [{
                    dataIndex: 'Id',

                    text: me.getLabel('GlassesTab.Id'),
                    flex: 1,
                }, {
                    dataIndex: 'Name',
                    text: me.getLabel('GlassesTab.Name'),
                    flex: 9,
                }],

                beforeAdd: beforeAddItem,
            });
        }

        tabPanelItems.push({
            title: 'Servers',
            itemName: 'Server',
            xtype: 'medwebcloud.windoweditor',
            formType: 'medwebcloud.supportserverform2',
            storeType: 'MedwebCloud.store.SupportServers',
            layout: 'fit',

            filters: [{
                property: 'CompanyId',
                operator: '=',
                value: me.record.get("Id")
            }],

            columns: [{
                dataIndex: 'Name',
                text: 'Name',
                flex: 2,
            }, {
                dataIndex: 'HostUrl',
                text: 'Url',
                flex: 5,
            }, {
                dataIndex: 'ServerType',
                text: 'Type',
                renderer: function (type, meta, record) {
                    switch (type)
                    {
                        case 2:
                            return 'Medweb EMR';

                        default:
                            return '--';
                    }
                },
                flex: 1,
            }],

            beforeAdd: function (item) {
                item.set("CompanyId", me.record.get("Id"));
            },
        });

        items.push({
            xtype: 'tabpanel',
            flex: 1, // full-height
            items: tabPanelItems
        });

        Ext.apply(me, {
            layout: utils.getFormLayout(),
            defaults: utils.getFormDefaults(),
            items: items
        });
        this.callParent(arguments);
    },
    getField: function (fieldName, opts) {
        return MedwebCloud.util.ViewUtils.getField(fieldName, opts, 'medweb.view.admin.company.', '.label');
    },
    getLabel: function (fieldName) {
        return MedwebCloud.getMsg('medweb.view.admin.company.' + fieldName.toLowerCase() + '.label');
    }
});