﻿Ext.define('MedwebCloud.view.ContactPanel', {
    extend: 'Ext.Window',
    xtype: 'MedwebCloud.contactpanel',
    contactRecord: null,
    loadMask: false,
    initComponent: function () {
        var me = this;

        var winTitle = MedwebCloud.getMsg('medweb.contact.update.window.title');
        var okTitle = MedwebCloud.getMsg('medweb.contact.update.button.positive.label');
        var cancelTitle = MedwebCloud.getMsg('medweb.common.button.negative.label');
        var labelDisplayName = MedwebCloud.getMsg('medweb.user.displayname.label');
        var labelEmail = MedwebCloud.getMsg('medweb.user.email.label');

        var labelGivenName = "First Name"
        var labelFamilyName = "Last Name"
        var labelMiddleName = "Middle Name"
        var labelSuffix = "Suffix"

        if (!me.contactRecord) {
            return;
        }

        var details = Ext.create('Ext.data.Store', {
            autoDestroy: true,
            model: 'MedwebCloud.model.ContactDetail',
            data: me.contactRecord.get('ContactDetails') || []
        });
        var detailTypes = Ext.create('Ext.data.Store', {
            autoDestroy: true,
            fields: ['value', 'name'],
            data: [
                { "value": "home", "name": "Home" },
                { "value": "work", "name": "Work" }
            ]
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToMoveEditor: 1,
            autoCancel: false
        });

        // create the grid and specify what field you want
        // to use for the editor at each column.
        var grid = Ext.create('Ext.grid.Panel', {
            itemId: 'details',
            title: 'Email Addresses',
            flex: 1,
            store: details,
            columns: [{
                header: 'Label',
                dataIndex: 'Level1Name',
                flex: 1,
                editor: {
                    xtype: 'combobox',
                    store: detailTypes,
                    queryMode: 'local',
                    displayField: 'name',
                    valueField: 'value',
                    allowBlank: false
                }
            }, {
                header: 'Email',
                dataIndex: 'Value',
                width: 160,
                editor: {
                    allowBlank: false,
                    vtype: 'email'
                }
            }],
            tbar: [{
                text: 'Add',
                iconCls: 'addIcon',
                handler: function () {
                    rowEditing.cancelEdit();

                    // Create a model instance
                    var r = Ext.create('MedwebCloud.model.ContactDetail', {
                        ContactId: me.contactRecord.get('Id'),
                        TypeName: 'email',
                        Level1Name: '',
                        Value: ''
                    });

                    details.insert(0, r);
                    rowEditing.startEdit(0, 0);
                }
            }, {
                itemId: 'removeEmployee',
                text: 'Remove',
                iconCls: 'deleteIcon',
                handler: function () {
                    var sm = grid.getSelectionModel();
                    var rc = sm.getSelection();
                    rowEditing.cancelEdit();
                    Ext.Array.each(rc, function (record, index, all) {
                        record.set('IsDeleted', true);
                    });
                    details.filter(function (r) {
                        return !r.get("IsDeleted");
                    });
                    if (details.getCount() > 0) {
                        sm.select(0);
                    }
                },
                disabled: true
            }],
            plugins: [rowEditing],
            listeners: {
                'selectionchange': function (view, records) {
                    grid.down('#removeEmployee').setDisabled(!records.length);
                }
            }
        });

        Ext.apply(me, {
            modal: true,
            title: winTitle,
            layout: 'fit',
            height: 500,
            width: 400,
            items: [{
                border: false,
                xtype: 'form',
                layout: {
                    type: 'vbox',
                    align: 'stretch'
                },
                defaults: {
                    anchor: '100%',
                    padding: '0 15 5 15'
                },
                items: [
                    {
                        xtype: 'textfield',
                        fieldLabel: labelDisplayName,
                        itemId: 'displayName',
                        name: 'DisplayName',
                        value: me.contactRecord.get('DisplayName'),
                        allowBlank: true,
                        minLength: 2,
                        padding: '15 15 5 15'
                    },
                    { xtype: 'textfield', fieldLabel: labelGivenName, name: 'GivenName', value: me.contactRecord.get('GivenName'), allowBlank: true, minLength: 2, padding: '5 15 5 15' },
                    { xtype: 'textfield', fieldLabel: labelFamilyName, name: 'FamilyName', value: me.contactRecord.get('FamilyName'), allowBlank: true, minLength: 2, padding: '5 15 5 15' },
                    { xtype: 'textfield', fieldLabel: labelMiddleName, name: 'MiddleName', value: me.contactRecord.get('MiddleName'), allowBlank: true, padding: '5 15 5 15' },
                    { xtype: 'textfield', fieldLabel: labelSuffix, name: 'Suffix', value: me.contactRecord.get('Suffix'), allowBlank: true, minLength: 2, padding: '5 15 5 15' },
                    grid
                ]
            }],
            buttons: [{
                text: okTitle,
                handler: function () {
                    var form = me.down('.form');
                    if (form.isValid()) {
                        rowEditing.cancelEdit();
                        var store = me.down('.grid').getStore();
                        var values = form.getValues();
                        var record = me.contactRecord;
                        record.set('DisplayName', values.DisplayName);
                        record.set('GivenName', values.GivenName);
                        record.set('FamilyName', values.FamilyName);
                        record.set('MiddleName', values.MiddleName);
                        record.set('Suffix', values.Suffix);
                        var detailArray = store.getModifiedRecords();

                        record.set('ContactDetails', detailArray);

                        record.data.ContactDetails = Ext.Array.pluck(detailArray, 'data');

                        record.save();

                        ////It seems something can't handle the embedded Contact details so we're going to do it ourself
                        //var theRest = me.contactRecord.proxy.client;

                        me.close();
                    }
                }
            }, {
                text: cancelTitle,
                handler: function () {
                    me.close();
                }
            }]
        });
        me.callParent(arguments);
    }
});