﻿Ext.define('MedwebCloud.view.HostForm', {
    xtype: 'medwebcloud.hostform',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;
        var utils = MedwebCloud.util.ViewUtils;

        Ext.apply(me, {
            layout: utils.getFormLayout(),
            defaults: utils.getFormDefaults(),
            items: [
                me.getField('Name', {
                    xtype: 'textfield',
                    allowBlank: false
                }),
                me.getField('CodeName', {
                    xtype: 'textfield',
                    allowBlank: true,
                    disabled: this.recordId > 0 ? true : false,
                }),
                me.getField('Domain', {
                    xtype: 'textfield',
                    allowBlank: false
                }),
                me.getField('ThemeName', {
                    xtype: 'combo',
                    queryMode: 'local',
                    store: utils.themeStore,
                    displayField: 'name',
                    valueField: 'value',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true
                }),
                me.getField('ModuleData', {
                    xtype: 'combo',
                    multiSelect: true,
                    delimiter: ',',
                    queryMode: 'local',
                    store: utils.getModuleStore(),
                    displayField: 'name',
                    valueField: 'value',
                    autoSelect: true,
                    forceSelection: true,
                    editable: false,
                    allowBlank: true,
                    listeners: {
                        change: function (combo, newVal, oldVal, eOpts) {
                            var modules = me.getComponent("ModuleNames");
                            modules.getComponent("CollageName").setVisible(newVal.indexOf('collage') > -1);
                            modules.getComponent("EVisitName").setVisible(newVal.indexOf('evisit') > -1);
                            modules.getComponent("InternalChatName").setVisible(newVal.indexOf('internalchat') > -1);
                            modules.getComponent("LookingGlassName").setVisible(newVal.indexOf('lookingglass') > -1);
                            modules.getComponent("MyMedwebName").setVisible(newVal.indexOf('mymedweb') > -1);
                            modules.getComponent("VitalsPlusName").setVisible(newVal.indexOf('vitalsplus') > -1);
                        }
                    }
                }),
                me.getField('LogoUrl', {
                    xtype: 'textfield',
                    allowBlank: true
                }),
                me.getField('FaviconUrl', {
                    xtype: 'textfield',
                    allowBlank: true
                }),
                {
                    xtype: 'fieldset',
                    itemId: 'ModuleNames',
                    title: me.getLabel('ModuleNames'),
                    collapsible: true,
                    defaults: utils.getFormDefaults(),
                    items: [
                        me.getField('CollageName', {
                            xtype: 'textfield',
                            allowBlank: true
                        }),
                        me.getField('EVisitName', {
                            xtype: 'textfield',
                            allowBlank: true
                        }),
                        me.getField('LookingGlassName', {
                            xtype: 'textfield',
                            allowBlank: true
                        }),
                        me.getField('VitalsPlusName', {
                            xtype: 'textfield',
                            allowBlank: true
                        }),
                        me.getField('InternalChatName', {
                            xtype: 'textfield',
                            allowBlank: true
                        }),
                        me.getField('MyMedwebName', {
                            xtype: 'textfield',
                            allowBlank: true
                        })
                    ]
                }
            ]
        });
        this.callParent(arguments);
    },
    getField :function (fieldName, opts){
        return MedwebCloud.util.ViewUtils.getField(fieldName, opts, 'medweb.view.admin.host.', '.label');
    },
    getLabel: function(fieldName) {
        return MedwebCloud.getMsg('medweb.view.admin.host.'+fieldName.toLowerCase() + '.label');
    }
});