﻿Ext.define('MedwebCloud.view.ImagesPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.imagespanel',
    record: null,
    client: null,
    zoom: true,
    draw: true,
    zoomLevel: 1,
    subchannel: 'Image',
    initComponent: function () {
        var me = this;
        me.imageIdsToPanels = {};
        var btnNext = MedwebCloud.getMsg('medweb.view.imagespanel.button.next.label');
        var btnPrev = MedwebCloud.getMsg('medweb.view.imagespanel.button.previous.label');
        var btnRemove = MedwebCloud.getMsg('medweb.view.imagespanel.button.previous.label');
        var noImages = MedwebCloud.getMsg('medweb.view.imagespanel.count.none');

        var btnZoomIn = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomin.label');
        var btnZoomOut = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomout.label');
        var btnZoomReset = MedwebCloud.getMsg('medweb.view.videopanel.button.zoomreset.label');

        var focusButtons = [{
            text: 'Focus',
            iconCls: 'focus',
            itemId: 'focus_btn',
            handler: function () {
                try {
                    var p = me.getLayout().getActiveItem();
                    var id = $(p.getEl().dom).data('imgId');
                    me.fireEvent('focusRequested', { "ImageId": id });
                } catch (ex) {
                    console.log("Couldn't handle it!", ex);
                    console.log(me.getLayout().getActiveItem());
                    console.log($(me.getLayout().getActiveItem().getEl().dom));
                }
            }
        }, {
            text: 'Unfocus',
            iconCls: 'unfocus',
            itemId: 'unfocus_btn',
            handler: function () {
                try {
                    var p = me.getLayout().getActiveItem();
                    var id = $(p.getEl().dom).data('imgId');
                    me.fireEvent('unfocusRequested', { "ImageId": id });
                } catch (ex) {
                    console.log("Couldn't handle it!", ex);
                    console.log(me.getLayout().getActiveItem());
                    console.log($(me.getLayout().getActiveItem().getEl().dom));
                }
            }
        }];

        var zoomButtons = this.zoom ? [{
            text: btnZoomOut,
            iconCls: 'zoom-out',
            itemId: 'zoom-out-btn',
            handler: function () {
                me.zoomBy(-0.1);
            }
        }, {
            text: btnZoomReset,
            iconCls: 'zoom-reset',
            itemId: 'zoom-reset-btn',
            handler: function () {
                me.zoomLevel = 1;
                me.zoomTo(-1);
                me.client.publish({
                    channel: me.getChannel("Image"),
                    data: {
                        ImgId: $(me.getLayout().getActiveItem().getEl().dom).data('imgId'),
                        Command: "PAN",
                        Data: '{"X":0,"Y":0}',
                    }
                });
            }
        }, {
            text: btnZoomIn,
            iconCls: 'zoom-in',
            itemId: 'zoom-in-btn',
            handler: function () {
                me.zoomBy(0.1);
            }
        }] : [];

        var drawButtons = this.draw ? [{
            xtype: 'button',
            itemId: 'erase-btn',
            text: 'Clear',
            iconCls: 'contain-image trashIcon',

            handler: function () {
                var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                var jqCanvas = jqElem.find('canvas').first();

                jqCanvas.sketch('clear');

                me.client.publish({
                    channel: me.getChannel("Image"),
                    data: {
                        ImgId: $(me.getLayout().getActiveItem().getEl().dom).data('imgId'),
                        Command: "CLEAR",
                        Data: null,
                    }
                })
            },
        }, {
            xtype: 'button',
            itemId: 'draw-btn',
            text: 'Draw',
            iconCls: 'contain-image pencilIcon',
            stateEvents: ['press'],
            stateId: 'statefulButton',
            enableToggle: true,

            handler: function () {
                var togglingDrawOn = this.pressed;
                var cmd = togglingDrawOn ? "disable" : "enable";

                var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                var jqHandle = jqElem.find('.shared_image_handle').first();
                var jqCanvas = jqHandle.siblings('canvas').first();
                var jqImage = jqHandle.siblings('img').first();

                if (togglingDrawOn) {
                    jqHandle.css('z-index', 20);
                    jqCanvas.css('z-index', 30);
                    me.query('#zoom-in-btn,#zoom-out-btn,#zoom-reset-btn').forEach(function (itm) {
                        itm.setDisabled(true);
                    });
                }
                else {
                    jqHandle.css('z-index', 30);
                    jqCanvas.css('z-index', 20);
                    me.query('#zoom-in-btn,#zoom-out-btn,#zoom-reset-btn').forEach(function (itm) {
                        itm.setDisabled(false);
                    });
                }
                jqElem.panzoom(cmd);
            },
        }] : [];

        var topButtons = focusButtons
                            .concat(['->'])
                            .concat(zoomButtons)
                            .concat(['->'])
                            .concat(drawButtons);

        Ext.apply(this, {
            tbar: topButtons,
            bbar: {
                items: [{
                    xtype: 'button',
                    text: btnPrev,
                    iconCls: 'x-btn-icon x-tbar-page-prev',
                    handler: function () {
                        if (me.items.length === 0) {
                            return;
                        }
                        if (!me.getLayout().getPrev()) {
                            me.getLayout().setActiveItem(me.items.length - 1);
                        } else {
                            me.getLayout().prev();
                        }
                        me.updateStatus();
                    }
                }, '-', {
                    xtype: 'tbtext',
                    text: noImages,
                    itemId: 'status'
                }, '-', {
                    xtype: 'button',
                    text: btnNext,
                    iconCls: 'x-btn-icon x-tbar-page-next',
                    handler: function () {
                        if (me.items.length === 0) {
                            return;
                        }
                        if (!me.getLayout().getNext()) {
                            me.getLayout().setActiveItem(0);
                        } else {
                            me.getLayout().next();
                        }
                        me.updateStatus();
                    }
                }, '->', {
                    xtype: 'button',
                    text: 'Remove',
                    iconCls: 'deleteIcon',
                    handler: function () {
                        console.log("Removing Image...");
                        var packet = {
                            ImgId: $(me.getLayout().getActiveItem().getEl().dom).data('imgId'),
                            Command: "UNSHARE",
                            Data: null,
                        };
                        me.handleImagePacket(packet);
                        me.client.publish({
                            channel: me.getChannel("Image"),
                            data: packet
                        })
                    },
                }]
            },
            layout: {
                type: 'card'
            }
        });

        this.callParent(arguments);
        this.subscribe();
    },

    getChannel: function (subchannel) {
        if (!subchannel) {
            subchannel = this.subchannel;
        }
        return '/collage/' + this.record.get('Id') + '/' + subchannel;
    },
    subscribe: function () {
        var me = this;
        this.client.subscribe({
            channel: this.getChannel(),
            onSuccess: function (args) {
                var imgList = args.getExtensionValueJson('images');
                if (imgList && imgList.length) {
                    try {
                        var packetArray = JSON.parse(imgList);
                        console.log("Loading images... ", packetArray);
                        for (var i = 0; i < packetArray.length; i++) {
                            me.handleImagePacket(packetArray[i]);
                        }
                    }
                    catch (e) {
                        console.log("Error parsing image history: ", e)
                    }
                }
            },
            onReceive: function (args) {
                if (args.getWasSentByMe()) {
                    return;
                }

                try {
                    var imgPkt = args.getData();
                    if (imgPkt.Command) {
                        me.handleImagePacket(imgPkt);
                    }
                    else if (imgPkt.binary64) {
                        me.addPanel(imgPkt.binary64);
                    }
                }
                catch (e) {
                    console.log("Error parsing image. ", e)
                }
            }
        });
    },
    handleImagePacket: function (imgPkt) {
        var me = this;
        console.log("Received image packet", imgPkt);

        switch (imgPkt.Command) {
            case "SHARE":
                if (me.imageIdsToPanels[imgPkt.ImgId] === me.undefined) {
                    me.addPanel(imgPkt.Data, false, null, imgPkt.ImgId);
                }
                break;

            case "UNSHARE":
                if (me.imageIdsToPanels[imgPkt.ImgId]) {
                    if (me.imageIdsToPanels[imgPkt.ImgId] === me.getLayout().getActiveItem()) {
                        Ext.Msg.toast({ title: "Image Removed", body: "The image you were viewing has been removed." });

                        if (me.getLayout().getPrev()) {
                            me.getLayout().prev();
                        } else if (me.getLayout().getNext()) {
                            me.getLayout().next();
                        }
                    }

                    me.imageIdsToPanels[imgPkt.ImgId].close();
                    delete me.imageIdsToPanels[imgPkt.ImgId];
                    me.updateStatus();
                }
                break;

            case "ZOOM":
                if (imgPkt.ImgId === me.activeImageId) {
                    me.zoomTo(+imgPkt.Data, true);
                }
                break;

            case "PAN":
                if (imgPkt.ImgId === me.activeImageId) {
                    var data = JSON.parse(imgPkt.Data);
                    var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                    var jqHandle = jqElem.find('.shared_image_handle').first();

                    jqHandle.panzoom('pan', data.X, data.Y);
                    me.resizeCanvas();
                }
                break;

            case "DRAW":
                var data = JSON.parse(imgPkt.Data);
                var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                var jqHandle = jqElem.find('.shared_image_handle').first();
                var jqCanvas = jqHandle.siblings('canvas').first();

                jqCanvas.sketch('addLines', data);
                break;

            case "CLEAR":
                var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                var jqHandle = jqElem.find('.shared_image_handle').first();
                var jqCanvas = jqHandle.siblings('canvas').first();
                jqCanvas.sketch('clear');
                break;

            default:
                break;
        }
    },
    publish: function (binary64, fileName, id) {
        var me = this,
            onFailure = function () {
                Ext.Msg.alert('Publish failed.');
            };

        this.client.publish({
            channel: me.getChannel("Image"),
            data: {
                ImgId: id,
                Command: "SHARE",
                Data: binary64,
            }
        })
        /*
        MedwebCloud.StreamCommands.sendUserCommand(me.getChannel("System"), false, "1_" + fileName, false, function userCommandSuccess() {
            MedwebCloud.StreamCommands.sendStreamCreate(me.getChannel("System"), fm.guid.newGuid(), "File", fileName, function createStreaSuccess() {
                me.client.publish({
                    channel: me.getChannel(),
                    data: {
                        binary64: binary64
                    },
                    onFailure: onFailure
                });
            }, onFailure);
        }, onFailure);*/
    },
    resizeCanvas: function () {
        var me = this;
        //console.log("Resizing...");

        var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
        var jqHandle = jqElem.find('.shared_image_handle').first();
        var jqCanvas = jqHandle.siblings('canvas').first();
        var jqImage = jqHandle.siblings('img').first();

        var zoomMult = jqHandle.css('transform').replace(/[^0-9\-.,]/g, '').split(',')[0] || 1;
        var actualWidth = jqHandle.width() * zoomMult;
        var actualHeight = jqHandle.height() * zoomMult;

        var rect = jqHandle[0].getBoundingClientRect();
        actualWidth = rect.width;
        actualHeight = rect.height;
        console.log('Redrawing: ', zoomMult, actualWidth, actualHeight);

        jqCanvas.attr('width', actualWidth)
                .attr('height', actualHeight);

        jqImage.attr('width', actualWidth)
                .attr('height', actualHeight);

        var handleOffset = jqHandle.offset();
        jqCanvas.offset(handleOffset);
        jqImage.offset(handleOffset);
        console.log("Offset change: ", JSON.stringify(handleOffset));

        jqCanvas.sketch('redraw');
    },
    zoomBy: function (dZoom) {
        this.zoomLevel += dZoom;

        this.zoomTo(this.zoomLevel);
    },
    zoomTo: function (size, dontNotify) {
        var me = this;

        var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
        var jqHandle = jqElem.find('.shared_image_handle').first();
        var doReset = false;
        if (size === -1) {
            me.zoomLevel = 1;
            doReset = true;
        } else {
            me.zoomLevel = Math.max(0.5, Math.min(2, size));
        }

        jqElem.data('zoomLevel', me.zoomLevel);
        if (doReset) {
                jqHandle.panzoom("reset", {
                    animate: false,
                    contain: false,
                    silent: false
                });
               
        } else {
            jqHandle.panzoom("zoom", me.zoomLevel, { silent: true });
            //Don't know why, but apparently zooming doesn't trigger panzoomend?
            me.resizeCanvas();
        }

        if (!dontNotify) {
            this.client.publish({
                channel: this.getChannel("Image"),
                data: {
                    ImgId: jqElem.data('imgId'),
                    Command: "ZOOM",
                    Data: "" + me.zoomLevel + "",
                }
            });
        }
    },
    addPanel: function (base64, doPublish, fileName, id) {
        var me = this;

        me.fireEvent('imageReceived', me.items.length + 1);

        if (!id) {
            id = fm.guid.newGuid();
        }

        var prefix = 'data:image/jpeg;base64,',
            src;
        if (base64.indexOf('data:image') === 0) {
            //extract the prefix
            src = base64;
        } else {
            src = prefix + base64;
        }

        var p = new Ext.panel.Panel({
            border: false,
            layout: 'fit',
            closable: false,
            title: fileName,
            listeners: {
                close: function () {
                    if (this === me.getLayout().getActiveItem()) {
                        me.getLayout().setActiveItem(0);
                    }

                    window.setTimeout(function () {
                        me.updateStatus();
                    }, 1);
                },
                afterrender: function () {
                    console.log("After Rendering");
                    var jqHandle = $(this.el.dom).find('.shared_image_handle').first();
                    jqHandle.panzoom();
                    jqHandle.panzoom().on('panzoomchange', function (e, panzoom, matrix, changed) {
                        console.log("Image panzoomchange: ", e, panzoom, matrix);
                        me.resizeCanvas();
                    });
                    jqHandle.panzoom().on('panzoomend', function (e, panzoom, matrix, changed) {
                        if (changed) {
                            console.log("Image panzoomed: ", e, panzoom, matrix);
                            me.resizeCanvas();
                            // deal with drags or touch moves
                            me.client.publish({
                                channel: me.getChannel("Image"),
                                data: {
                                    ImgId: $(me.getLayout().getActiveItem().getEl().dom).data('imgId'),
                                    Command: "PAN",
                                    Data: '{"X":' + matrix[4] + ',"Y":' + matrix[5] + '}',
                                }
                            });
                        }
                    });
                },
                activate: function (panel) {
                    console.log("Activating image " + id);

                    me.activeImageId = id;

                    //Due to ExtJS timing weirdness, the panel body isn't actually rendered until activate
                    // which means it doesn't have a width or height until then, so we need to do *our* "rendering"
                    // in activate instead.
                    //Using the data for the tab as a flag for whether we should do this or not to prevent
                    // rerender when switching between images.
                    var jqElem = $(me.getLayout().getActiveItem().getEl().dom);
                    var localZoom = jqElem.data('zoomLevel');
                    if (!localZoom) localZoom = 1;
                    me.zoomLevel = localZoom;

                    //if (jqElem.data('firstpassdone'))
                    {
                        console.log("Fixing display...");

                        jqElem.data('undisplayed', false);

                        var jqHandle = jqElem.find('.shared_image_handle').first();
                        var jqCanvas = jqHandle.siblings('canvas').first();
                        var jqImg = jqHandle.siblings('img').first();

                        $('<img/>')
                            .attr('src', jqImg.attr('src'))
                            .load(function () {
                                jqHandle.css('width', this.naturalWidth);
                                jqHandle.css('height', this.naturalHeight);
                                jqHandle.css('left', 0.5 * (jqHandle.parent().width() - jqHandle.width()));  //Note this must come *after* the width recalculation
                                jqHandle.css('top', 0.5 * (jqHandle.parent().height() - jqHandle.height()));  //Note this must come *after* the height recalculation

                                $('.loader_msg').css('display', 'none');
                                me.resizeCanvas();
                                if (jqImg.width() > 0) {
                                    jqImg.css('visibility', '');
                                }
                            });

                        //jqHandle.css('width', jqImg.width()); //The image is effectively "contain"ed, but the div defaults to 100% width; fix it here so it resizes correctly in resizeCanvas
                        //jqHandle.css('left', 0.5 * (jqHandle.parent().width() - jqHandle.width()));  //Note this must come *after* the width recalculation

                        var pts = [];
                        jqCanvas.sketch({ defaultColor: "#ff0", markerTimeout: false })
                            .on('start', function () {
                                pts = [];
                            })
                            .on('stop', function () {
                                console.log('Publishing Draw: ', {
                                    channel: me.getChannel("Image"),
                                    data: {
                                        ImgId: id,
                                        Command: "DRAW",
                                        Data: JSON.stringify(pts),
                                    }
                                });

                                try {
                                    me.client.publish({
                                        channel: me.getChannel("Image"),
                                        data: {
                                            ImgId: id,
                                            Command: "DRAW",
                                            Data: JSON.stringify(pts),
                                        }
                                    })
                                }
                                catch (e) {
                                    console.log("EXCEPT: " + e);
                                }
                            })
                            .on('point', function (e, pt) {
                                try {
                                    pts.push({ 'x': pt.X, 'y': pt.Y });
                                }
                                catch (e) {
                                    console.log("EXCEPT: " + e);
                                }
                            });
                        jqHandle.css('z-index', 30);
                        jqCanvas.css('z-index', 20);
                        jqImg.css('z-index', 10);

                        //me.resizeCanvas();
                        //jqImg.css('height', 'auto'); //Switch this to now be auto-calc'ed
                    }

                    me.down('#draw-btn').toggle(false);
                    me.query('#zoom-in-btn,#zoom-out-btn,#zoom-reset-btn').forEach(function (itm) {
                        itm.setDisabled(false);
                    });
                    Ext.EventManager.onWindowResize(me.resizeCanvas, me);
                },
                show: function () {
                    console.log("Showing");
                },
                beforedeactivate: function () {
                    Ext.EventManager.removeResizeListener(me.resizeCanvas, me);
                },
            },
            html: '<div class="loader_msg" style="position: absolute; top: 50%; left: 50%; transform: translateX(-50%) translateY(-50%);">Loading image...</div><div class="shared_image_handle" style="position:absolute"></div><canvas style="position: absolute;"></canvas><img style="position:absolute;display:block;visibility:hidden;" src="' + src + '" />',
            itemId: 'imgC_' + id,
        });

        console.log("Pre-add");
        me.add(p);
        $(p.getEl().dom).data('imgId', id);
        console.log("Added");
        me.getLayout().setActiveItem(p);
        if (me.items.items.length === 1) {
            p.fireEvent('activate');
        }
        console.log("All Set");
        var jqElem = $(p.getEl().dom);
        jqElem.data('undisplayed', true);
        me.imageIdsToPanels[id] = p;

        me.updateStatus();

        if (doPublish) {
            window.setTimeout(function () {
                var img = new Image();
                var canvas = document.createElement('canvas');
                var ctx = canvas.getContext("2d");
                img.onload = function () {
                    var w = img.width;
                    var h = img.height;
                    canvas.width = w;
                    canvas.height = h;
                    ctx.drawImage(img, 0, 0); //draw image

                    var ratio = w / h;
                    var max = 1280, width = 0, height = 0;
                    if (ratio > 1) {
                        width = max;
                        height = max / ratio;
                    } else {
                        width = max * ratio;
                        height = max;
                    }

                    //resample_hermite(canvas, w, h, width, height);
                    me.publish(canvas.toDataURL("image/jpeg", 0.20).replace(prefix, ''), fileName, id);
                }
                img.src = src;
            }, 1);
        }

        return p;
    },
    showImage: function (imgId) {
        var me = this;
        console.log("Attempting to show image ", imgId);

        if (me.imageIdsToPanels[imgId] !== me.undefined) {
            me.getLayout().setActiveItem(me.imageIdsToPanels[imgId]);
        } else {
            console.log("Image " + imgId + " not found to show");
        }
    },
    updateStatus: function () {
        var me = this;

        var active = me.getLayout().getActiveItem();
        var index = me.items.indexOf(active);
        var text = MedwebCloud.getMsg('medweb.view.imagespanel.count.message', index + 1, me.items.length);
        var title = MedwebCloud.getMsg('medweb.view.imagespanel.count.title', me.items.length);

        me.down('#status').setText(text);
        //me.setTitle(title);
    },
    unsubscribe: function () {
        this.client.unsubscribe({
            channel: this.getChannel()
        });
    },
    onDestroy: function () {
        this.unsubscribe();
        this.callParent(arguments);
    }
});