﻿Ext.define('MedwebCloud.view.AddLookingGlassForm', {
    xtype: 'medwebcloud.addlookingglassform',
    extend: 'Ext.form.Panel',

    //Uses forceComplete to skip the default add handling

    initSelects: function (parentForm) {
        if (parentForm.glassesLoaded && parentForm.companiesLoaded && parentForm.cSelectRendered && parentForm.gSelectRendered) {
            parentForm.down('#companySelect').setDisabled(false);
            parentForm.down('#glassesSelect').setDisabled(false);

            parentForm.down('#companySelect').setValue(parentForm.companiesStore.getAt('0').get('Id'));
        }
    },
    initComponent: function () {
        var me = this;

        me.saveButton.text = MedwebCloud.getMsg('medweb.view.lookingglass.import_glasses');
        me.saveButton.handler = function () {
            var values = {};
            var items = (!!me.getFormItems) ? me.getFormItems() : me.items;

            items.each(function (f) {
                if (f.getName && f.getValue) { values[f.getName()] = f.getValue(); }
            });

            if (!values['glassesSelect']) {
                console.log("Illegal value for glassesSelect: ", values['glassesSelect']);
                return;
            }

            //fieldName, value, [startIndex], [anyMatch], [caseSensitive], [exactMatch]
            var claimed = me.glassesStore.findRecord('Id', values['glassesSelect'], 0, false, false, true);

            if (!claimed) {
                console.log("Could not find found glasses.  Logical impossibility.  Skynet failing.");
                Ext.Msg.alert("InvalidState Error", "Claim failed, please refresh your browser and try again.");
            }

            claimed.set('CompanyId', me.customArgs['CompanyId']);
            me.getEl().mask(window.MedwebCloud.getMsg('medweb.view.shared.editors.messages.saving'));
            me.glassesStore.sync({
                success: function () {
                    me.up().forceComplete(true);
                },
                failure: function (batch) {
                    console.log("Sync failed.", batch);
                    me.up().forceComplete(false);
                },
                callback: function () {
                    me.getEl().unmask();
                    me.up('.window').close();
                }
            });
        };

        me.companiesStore = Ext.create('MedwebCloud.store.Companies', {
            pageSize: 2147483647,
            filters: [{
                property: 'Id',
                comparison: 'ne',
                value: me.customArgs['CompanyId']
            }],
        });
        me.companiesStore.load(function () { me.companiesLoaded = true; me.initSelects(me); });

        me.glassesStore = Ext.create('MedwebCloud.store.Devices', {
            filters: [{
                property: 'DeviceType',
                operator: '=',
                value: MedwebCloud.enums.DeviceType.glasses
            }, {
                property: 'CompanyId',
                comparison: 'ne',
                value: me.customArgs['CompanyId']
            }],
            pageSize: 2147483647,
            remoteFilter: true,
        });
        me.glassesStore.load(function () {
            me.glassesLoaded = true;
            me.initSelects(me);
        });

        var companiesSelectProps = {
            xtype: 'combo',
            itemId: 'companySelect',
            store: me.companiesStore,
            displayField: 'Name',
            valueField: 'Id',
            forceSelection: true,
            multiSelect: false,
            triggerAction: 'all',
            fieldLabel: MedwebCloud.getMsg('medweb.view.lookingglass.glasses_owned_by'),
            selectOnFocus: false,
            labelStyle: 'display: inline-block; width: 150px',
            disabled: true,
            queryMode: 'local',
            lastQuery: '',
            listeners: {
                change: function (combo, newVal, oldVal, eOpts) {
                    me.glassesStore.clearFilter(true);
                    me.glassesStore.filterBy(function (rec, id) {
                        return rec.get('CompanyId') === newVal;
                    });

                    if (me.glassesStore.getCount() > 0)
                    {
                        me.down('#glassesSelect').setDisabled(false);
                        me.down('#glassesSelect').setValue(me.glassesStore.getAt('0').get('Id'));
                        if (me.saveButton) {
                            me.saveButton.setDisabled(false);
                        }
                    }
                    else
                    {
                        me.down('#glassesSelect').inputEl.dom.value = MedwebCloud.getMsg('medweb.view.lookingglass.no_glasses_available');
                        me.down('#glassesSelect').setDisabled(true);
                        if (me.saveButton) {
                            me.saveButton.setDisabled(true);
                        }
                    }
                },
                afterRender: function () {
                    me.cSelectRendered = true;
                    me.initSelects(me);
                }
            }
        };

        var glassesSelectProps = {
            xtype: 'combo',
            itemId: 'glassesSelect',
            name: 'glassesSelect',
            store: me.glassesStore,
            valueField: 'Id',
            // template for the content inside dropdown list
            tpl: Ext.create('Ext.XTemplate',
                '<tpl for=".">',
                    '<div class="x-boundlist-item">{Name} #{Id}</div>',
                '</tpl>'
            ),
            // template for the content inside text field
            displayTpl: Ext.create('Ext.XTemplate',
                '<tpl for=".">',
                    '{Name} #{Id}',
                '</tpl>'
            ),
            forceSelection: true,
            multiSelect: false,
            triggerAction: 'all',
            fieldLabel: MedwebCloud.getMsg('medweb.view.lookingglass.glasses'),
            selectOnFocus: false,
            disabled: true,
            labelStyle: 'display: inline-block; width: 150px',
            queryMode: 'local',
            lastQuery: '',
            listeners: {
                afterRender: function () {
                    me.gSelectRendered = true;
                    me.initSelects(me);
                },
            },
        };

        Ext.apply(me, {
            defaults: {
                anchor: '100%'
            },
            items: [
                companiesSelectProps,
                glassesSelectProps
            ],
        });
        this.callParent(arguments);
    }
});