﻿Ext.define('MedwebCloud.view.LookingGlassTab', {
    xtype: 'medwebcloud.lookingglasstab',
    extend: 'Ext.Panel',
    client: null,
    initComponent: function () {
        var me = this;

        var gridStore = null;
        me.showingOffline = true;

        if (true)//!MedwebCloud.glassList)
        {
            function checkOnlineGlasses() {
                MedwebCloud.websyncClient.join({
                    channel: '/lookingglass/devicelist',
                    userId: 'list',
                    userNickname: 'list',
                    onSuccess: function (e) {
                        console.log("Joined devicelist channel: ", e);
                        var users = e.getUsers();
                        parseOnlineGlassList(users);
                    },
                    onFailure: function (e) {
                        console.log("Error loading devicelist: ", e);
                        alert("Could not retrieve online glasses.  Please reload the page and try again.  If the problem persists, please contact your network administrator.");
                    },
                    onUserJoin: function (e) {
                        console.log("Glass joined", e);
                        var newGlassId = e.getJoinedUser().getUserId();
                        addOnlineGlass(newGlassId);
                    },
                    onUserLeave: function (e) {
                        console.log("Glass left", e);
                        var leftGlassId = e.getLeftUser().getUserId();
                        removeOnlineGlass(leftGlassId);
                    }
                });
            }

            function parseOnlineGlassList(userList)
            {
                for (var i = 0; i < userList.length; i++) {
                    MedwebCloud.glassList[userList[i].getUserId()] = true;
                }

                me.sortGrid();
                me.down('#devicegrid').reconfigure(MedwebCloud.glassStore);
            }

            function addOnlineGlass(glassId)
            {
                MedwebCloud.glassList[glassId] = true;
                me.sortGrid();
                me.down('#devicegrid').reconfigure(MedwebCloud.glassStore);
            }

            function removeOnlineGlass(glassId) {
                MedwebCloud.glassList[glassId] = false;
                me.sortGrid();
                me.down('#devicegrid').reconfigure(MedwebCloud.glassStore);
            }

            MedwebCloud.glassStore = Ext.create('MedwebCloud.store.Devices', {
                autoLoad: false,
                remoteSort: false,
                filters: [{
                    property: 'DeviceType',
                    value: MedwebCloud.enums.DeviceType.glasses,
                }, {
                    property: 'CompanyId',
                    value: MedwebCloud.userProfile.CompanyId,
                }],
                expand: 'Location',
            });

            MedwebCloud.glassStore.load(function (r, o, s) {
                MedwebCloud.glassList = {};
                checkOnlineGlasses();
                //console.log(MedwebCloud.glassStore);
            });
        }

        Ext.apply(this, {
            itemId: 'lookingglasstab',
            xtype: 'panel',
            layout: 'card',
            items: [{
                xtype: 'panel',
                layout: 'border',
                itemId: 'map-container',
                items: [{
                    collapsible: false,
                    split: true,
                    region: 'west',
                    width: 350,
                    xtype: 'panel',
                    layout: 'fit',
                    items: [{
                        xtype: 'medwebcloud.devicesgrid',
                        itemId: 'devicegrid',
                        overflowX: 'hidden',
                        overflowY: 'auto',
                        store: MedwebCloud.glassStore,

                        /*
                        tbar: [{
                            xtype: 'button',
                            text: 'Debug',
                            handler: function () {
                                me.debug = true;
                                me.sortGrid();
                            }
                        }],
                        */

                        //selModel: {
                        //    mode: 'MULTI',
                        //},
                        viewConfig: {
                            getRowClass: function (rec, idx) {
                                if (!MedwebCloud.glassList[rec.get('Id')] && !me.debug) {
                                    return 'disabled-row';
                                }
                            }
                        },
                        emptyText: '<div class="empty-grid">No glasses online.</div>',
                        listeners: {
                            sortchange: function (grid, column, direction) {
                                me.currentSort = {
                                    property: column.dataIndex,
                                    direction: direction,
                                };

                                me.sortGrid();
                            },
                            itemdblclick: function (grid, record, item, index) {
                                if (MedwebCloud.glassList[record.get('Id')]) {
                                    me.down('#lookingglasspanel').addGlasses(record);
                                }
                            }
                        }
                    }],
                }, {
                    region: 'center',
                    itemId: 'lookingglasspanel',
                    client: this.client,
                    layout: 'fit',
                    xtype: 'medwebcloud.lookingglasspanel'
                }],
                listeners: {
                    activate: function () {
                        me.down('#map-container').down('#lookingglasspanel').doLayout();
                    },
                }
            }, {
                xtype: 'panel',
                layout: 'fit',
                itemId: 'setup',
                items: [{
                    xtype: 'medwebcloud.lookingglasssetupwizard',
                }]
            }],
        });

        this.callParent(arguments);
    },
    sortGrid: function () {
        var me = this;

        if (!me.currentSort)
        {
            me.currentSort = {
                property: 'Id',
                direction: 'ASC',
            };
        }

        MedwebCloud.glassStore.sort([{
            sorterFn: function (g1, g2) {
                var g1online = MedwebCloud.glassList[g1.get('Id')];
                var g2online = MedwebCloud.glassList[g2.get('Id')];

                if (g1online && !g2online) return 1;
                if (!g1online && g2online) return -1;
                return 0;
            },
            direction: 'DESC'
        },
        me.currentSort,
        ]);
    },
    setActiveSection: function (config) {
        // config has name and ref property: #lookingglass/{name}/{ref}
        var me = this;

        if (config.name && config.name === 'configuration')
        {
            if (config.ref) {
                //Pass glass ID?
                me.startGlassCfg(config.ref);
            } /*else {
                me.startGlassCfg();
            }*/
        }
    },
    startGlassCfg: function (selectedGlasses) {
        var btn = this;

        function loadGlassConfig(glassChannel) {
            try {
                var subCfgArgs = new fm.websync.subscribeArgs([glassChannel]);
                subCfgArgs.setOnReceive(function (receiveArgs) {
                    if (receiveArgs.getWasSentByMe()) { return; }

                    var dta = JSON.parse(receiveArgs.getDataJson());
                    if (!dta.glass_setup) { this.handling = false; return; }

                    MedwebCloud.configWaiting = false;
                    //console.log("Glass Configuration received: ", dta.glass_setup);

                    var strSetup = JSON.stringify(dta.glass_setup);
                    var oldConfig = JSON.parse(strSetup);
                    var newConfig = JSON.parse(strSetup);

                    var pageList = [{
                        title: 'General Settings',
                        page: Ext.create('MedwebCloud.view.GeneralConfigWizardPage', {
                            header: false,
                            valueSet: newConfig,
                            glass: glassChannel,
                        }),
                    }, {
                        title: 'Voice Commands',
                        page: Ext.create('MedwebCloud.view.VoiceConfigWizardPage', {
                            header: false,
                            valueSet: newConfig,
                            glass: glassChannel,
                        }),
                    }, {
                        title: 'Summary',
                        page: Ext.create('MedwebCloud.view.SummaryWizardPage', {
                            header: false,
                            oldValues: oldConfig,
                            newValues: newConfig,
                            glass: glassChannel,
                        }),
                    }];

                    var wizWindow = Ext.create('MedwebCloud.view.LookingGlassWizardWindow', {
                        pages: pageList,
                        state: newConfig,
                        listeners: {
                            'wizardComplete': function (newVals) {
                                //console.log('Wizard Complete!', JSON.stringify(newVals));
                                var obj = {
                                    'Description': 'glassConfig-v1',
                                    'Data': newVals,
                                };

                                MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(glassChannel, JSON.stringify(obj)));
                            }
                        }
                    });
                    MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([glassChannel]));

                    wizWindow.show();
                });
                subCfgArgs.setOnSuccess(function (sa) {
                    MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(glassChannel, '["req-data"]'));
                });
                MedwebCloud.websyncClient.subscribe(subCfgArgs);

                MedwebCloud.configWaiting = true;
                setTimeout(function () {
                    if (MedwebCloud.configWaiting) {
                        MedwebCloud.configWaiting = false;
                        MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([glassChannel]));
                        Ext.Msg.alert('Configuration Timeout', 'Looking Glass is not responding to the configuration request.  Please ensure the glasses are on and connected to a WiFi network.');
                    }
                }, 5*1000);
            } catch (ex) {
                console.log(ex);
                Ext.Msg.alert("Configuration Error", "An error occurred while trying to configure Looking Glass.  Please try again.");
            }
        }

        var dftConfig;

        if (typeof (selectedGlasses) === 'string') {
            var syncChannel = '/lookingglass/config/' + selectedGlasses;
            console.log("Sync ID received: Listening on [" + syncChannel + "]");

            var subArgs = new fm.websync.subscribeArgs([syncChannel]);
            subArgs.setOnSuccess(function (sa) {
                MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(syncChannel, JSON.stringify({
                    Description: 'syncChannel',
                    'Data': syncChannel,
                })));
            })
            subArgs.setOnClientSubscribe(function (clientSubArgs) {
                console.log("Someone joined sync ", clientSubArgs);
            });
            subArgs.setOnReceive(function (receiveArgs) {
                if (receiveArgs.getWasSentByMe()) { return; }

                var dta = JSON.parse(receiveArgs.getDataJson());

                if (!dta.Config) { throw("Wrong format"); }
                var tgtChannel = dta.Config;
                var glassId = dta.Device.UniqueId;

                console.log("Sync complete.  Channel [" + tgtChannel + "]");

                MedwebCloud.syncWaiting = false;

                loadGlassConfig(tgtChannel);

                MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([syncChannel]));
            });

            MedwebCloud.websyncClient.subscribe(subArgs);

            MedwebCloud.syncWaiting = true;
            setTimeout(function () {
                if (MedwebCloud.syncWaiting) {
                    MedwebCloud.syncWaiting = false;
                    MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([syncChannel]));
                    Ext.Msg.alert('Configuration Timeout', 'Looking Glass does not seem to be responding to the sync request.  Please ensure the glasses are on and connected to a WiFi network.');
                }
            }, 5 * 1000);
        } else if (typeof (selectedGlasses) === 'object') {
            loadGlassConfig(selectedGlasses[0].channel);
        } else {
            console.log("No glasses.");
        }
    },
});