﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.Collage', {
    xtype: 'medwebcloud.lookingglasssetupwizard.collage',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var pinnedSessions = Ext.create('MedwebCloud.store.Sessions', {
            pageSize: 2147483647,
            filters: [{
                property: 'CompanyId',
                comparison: 'eq',
                value: MedwebCloud.userProfile.CompanyId
            }, {
                property: 'Pinned',
                comparison: 'eq',
                value: 1
            }],
        });

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure Medweb LIVE! options:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'If you select a default session, the glasses will automatically connect to that session at startup.' +
                    '<br /><br />' +
                    'A session must be permanent or "pinned" to be selected as a default session.',
            }, {
                xtype: 'panel',
                layout: {
                    type: 'vbox',
                    pack: 'center',
                    align: 'center',
                },
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                items: [{
                    xtype: 'combo',
                    itemId: 'sessionSelect',
                    fieldLabel: 'Default Medweb LIVE! Session',
                    name: 'DefaultSessionId',
                    xtype: 'combo',
                    store: pinnedSessions,
                    displayField: 'Name',
                    valueField: 'Id',
                    forceSelection: true,
                    multiSelect: false,
                    triggerAction: 'all',
                    selectOnFocus: false,
                    disabled: true,
                    queryMode: 'local',
                    lastQuery: '',
                    width: 600,
                    labelWidth: 200
                }],
            }, {
                xtype: 'checkbox',
                itemId: 'can-join',
                name: 'CanJoinSessions',
                fieldLabel: 'Allowed to join LIVE sessions on request',
                hidden: true,
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    var wiz = this.up('panel').up();

                    this.getForm().getFields().each(function (itm) {
                        var data = wiz.getData(itm.getItemId());

                        if (data != null && data != undefined && itm.setValue) {
                            itm.setValue(data);
                        }
                    });

                    pinnedSessions.load(function () {
                        this.insert(0, Ext.create('MedwebCloud.model.Session', {
                            Id: 0,
                            Name: 'No Session Selected'
                        }));
                        if (this.getCount() > 0) {
                            form.down('#sessionSelect').on('enable', function (box, eOpts) {
                                try {
                                    var dta = wiz.getData('DefaultSessionId');
                                    dta = dta ? dta : 0;

                                    box.setValue(dta);
                                } catch (e) {
                                    box.setValue(0);
                                }
                            });
                            form.down('#sessionSelect').setDisabled(false);
                        } else {
                            form.down('#sessionSelect').inputEl.dom.value = 'No LIVE! Sessions Available';
                        }
                    });
                },
            },
        });

        this.callParent(arguments);
    },
    saveChanges: function () {
        var me = this;

        me.getRecord()['DefaultSessionId'] = me.down('#sessionSelect').getValue();
        me.getRecord()['DefaultSessionName'] = me.down('#sessionSelect').getRawValue();
    },
});