﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard', {
    xtype: 'medwebcloud.lookingglasssetupwizard',
    extend: 'Ext.Panel',

    defaultData: {},

    initComponent: function () {
        var me = this;
        /*
        me.defaultData = Ext.merge({}, me.defaultData, {
            'servers': {
                'websync': MedwebCloud.websyncPath,
                'icelink': MedwebCloud.serverAddressIceLink + ':' + MedwebCloud.serverPortIceLink
            },
        });*/

        Ext.apply(this, {
            itemId: 'glass-setup-container',
            header: {
                title: 'Looking Glass Setup',
                titleAlign: 'center',
            },
            layout: 'card',
            padding: '30 30 30 30',
            items: [{
                itemId: 'glass-selection',
                name: 'glass-selection',
                border: false,
                layout: {
                    type: 'vbox',
                    pack: 'center',
                    align: 'center',
                },
                padding: '20 20 20 20',
                items: [{
                    xtype: 'displayfield',
                    fieldLabel: 'Welcome to Looking Glass',
                    labelAlign: 'top',
                    name: 'welcome-text',
                    value: '<p>Salutations, brave experimenter.  You are about to embark on a mysterious journey through largely untested code' +
                        ' and inconsistent design specs.  Are you ready?</p>'
                }, {
                    xtype: 'button',
                    text: 'I have a pair of glasses here that I would like to update and have the changes instantly applied.',
                    handler: function () {
                        me.startSyncSetup();
                    }
                }, {
                    xtype: 'button',
                    text: 'Start Setup',
                    handler: function () {
                        me.startConfig([
                            { title: "Network Setup", form: "Networks" },
                            { title: "Medweb LIVE!", form: "Collage" },
                            { title: "Looking Glass", form: "LookingGlass" },
                            { title: "Voice Commands", form: "Voice" },
                            { title: "Voice Commands - Camera", form: "VoiceCamera" },
                            { title: "Voice Commands - Images", form: "VoiceImages" },
                            { title: "Voice Commands - Devices", form: "VoiceInstruments" },
                            { title: "Voice Commands - Videos", form: "VoiceVideos" },
                            { title: "Generated QR", form: "QRGenerator" },
                        ]);
                    }
                }],
            }]
        });

        this.callParent(arguments);
    },
    startConfig: function (pageList) {
        var me = this;

        if (!me.wiz) {
            me.wiz = Ext.create('MedwebCloud.view.WizardForm', {
                pages: pageList,
                itemId: 'glass-wizard',
                defaultData: me.defaultData || {},
                finalizeData: function (data) {
                    /*if (data['network_setup'] && data['network_setup']['networks'] && data['network_setup']['networks'].length) {
                        data['network']['wifi'] = data['network_setup']['networks'][0];
                        delete data['network_setup'];
                    }
                    if (data['voice_commands']) {
                        data['voice'] = data['voice_commands'];
                        delete data['voice_commands'];
                    }

                    for (var prop in Object.keys(data))
                    {
                        console.log("Data" + prop + " : " + data[prop])
                        if (data.hasOwnProperty(prop) && (data[prop] == null || data[prop] == {}))
                        {
                            delete data[prop];
                        }
                    }*/

                    return data;
                },
            });

            me.wiz.on('dataChanged', function (changedData) {
                console.log('Wizard data changed', changedData);

                if (me.syncChannel) {
                    var pub = new fm.websync.publishArgs(me.syncChannel, JSON.stringify(changedData));
                    MedwebCloud.websyncClient.publish(pub);
                }
            });

            me.add(me.wiz);
            me.doLayout();
        }

        me.getLayout().setActiveItem(me.wiz.getItemId());
    },
    startSyncSetup: function () {
        var me = this;

        me.syncChannel = '/lookingglass/setup/' + Math.guid();

        var syncVals = {
            'websync': MedwebCloud.websyncPath,
            'icelink': MedwebCloud.serverAddressIceLink + ':' + MedwebCloud.serverPortIceLink,
            'sync': me.syncChannel,
        };

        var sub = new fm.websync.subscribeArgs([me.syncChannel]);
        sub.setOnReceive(function (r) {
            console.log(r.getDataJson());
            if (!r.getWasSentByMe()) {
                var dta = JSON.parse(r.getDataJson());
                if (dta['glasses'] && dta['glasses'] === 'OK') {
                    if (dta['glass-config']) {
                        var cfgChannel = dta['glass-config'];
                        MedwebCloud.websyncClient.unsubscribe(new fm.websync.unsubscribeArgs([me.syncChannel]));
                        var sub2 = new fm.websync.subscribeArgs([me.syncChannel]);
                        sub2.setOnSuccess(function (s2) {
                            MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(cfgChannel, JSON.stringify({ 'req-data': true })))
                        });
                        sub2.setOnReceive(function (r2) {
                            var d = JSON.parse(r2.getDataJson());
                            if (d['glasses'] && d['glasses'] === 'RCV-DATA') {
                                //Handshake
                                console.log("Handshake: Config OK");
                            }
                            else if (d['glass-init']) {
                                me.defaultData = d['glass-init'];
                                console.log("Default data changed: ", me.defaultData);

                                Ext.Msg.toast({ body: 'Glasses synced' });
                                var btn = Ext.getCmp('endSync');
                                console.log("Complete button", btn);
                                btn.disabled = false;
                                btn.text = 'Start Setup';
                                console.log("Button changed", btn);
                            }
                        });
                    }
                }
                else if (dta['glasses'] && dta['glasses'] === 'DATA') {
                    Ext.Msg.toast({ body: 'Configuration change synced' });
                }
            }
        });
        MedwebCloud.websyncClient.subscribe(sub);

        var windowTitle = MedwebCloud.getMsg('medweb.network.window.title');

        var labelNetwork = MedwebCloud.getMsg('medweb.network.network.label');
        var labelSecurity = MedwebCloud.getMsg('medweb.network.security.label');
        var labelPasscode = MedwebCloud.getMsg('medweb.network.password.label');

        w = Ext.create('Ext.window.Window', {
            title: 'Looking Glass Setup Sync',
            modal: true,
            layout: 'card',
            border: false,
            itemId: 'syncContainer',
            id: 'syncContainer',
            items: [{
                xtype: 'panel',
                itemId: 'reqNetwork',
                items: [{
                    xtype: 'displayfield',
                    value: 'Are the glasses currently connected to the local WiFi Network?'
                }, {
                    xtype: 'button',
                    text: 'Yes, they are connected.  I do not need to set up a network connection.',
                    handler: function () {
                        this.up('#syncContainer').getLayout().setActiveItem('syncQR');
                    }
                }, {
                    xtype: 'button',
                    text: 'No, they are not connected.  I need to set up a network connection.',
                    handler: function () {
                        this.up('#syncContainer').getLayout().setActiveItem('networkForm');
                    }
                }]
            }, {
                xtype: 'form',
                itemId: 'networkForm',
                id: 'networkForm',
                defaults: {
                    margin: 10
                },
                fieldDefaults: {
                    labelWidth: 80
                },
                items: [{
                    xtype: 'textfield',
                    itemId: 'network_id',
                    fieldLabel: labelNetwork,
                    width: 285,
                    allowBlank: false,
                }, {
                    xtype: 'combobox',
                    itemId: 'network_security',
                    fieldLabel: labelSecurity,
                    store: ['WPA', 'WEP', 'None'],
                    value: 'Please Select',
                    width: 285,
                    forceSelection: true,
                }, {
                    xtype: 'textfield',
                    itemId: 'network_passcode',
                    inputType: 'password',
                    fieldLabel: labelPasscode,
                    width: 285,
                    allowBlank: false,
                }, {
                    xtype: 'button',
                    text: 'Continue',
                    itemId: 'action',
                    handler: function () {
                        syncVals['networks'] = [{
                            network: this.up('form').down('#network_id').getValue(),
                            security: this.up('form').down('#network_security').getValue(),
                            passphrase: this.up('form').down('#network_passcode').getValue()
                        }];
                        this.up('#syncContainer').getLayout().setActiveItem('syncQR');
                    }
                }],
            }, {
                xtype: 'panel',
                itemId: 'syncQR',
                id: 'syncQR',
                items: [{
                    xtype: 'displayfield',
                    value: 'In order to begin the sync, enter "Config" mode on the glasses, then press the "Scan" button and scan the QR code below.'
                }, {
                    xtype: 'image',
                    id: 'qr',
                    width: 500,
                    height: 500,
                }, {
                    xtype: 'button',
                    itemId: 'endSync',
                    id: 'endSync',
                    text: 'Wait for connection...',
                    disabled: true,
                    handler: function () {
                        w.close();

                        me.startConfig([
                            { title: "Network Setup", form: "Networks" },
                            { title: "Medweb LIVE!", form: "Collage" },
                            { title: "Looking Glass", form: "LookingGlass" },
                            { title: "Voice Commands", form: "Voice" },
                            { title: "Generated QR", form: "QRGenerator" },
                        ]);
                    }
                }],
                listeners: {
                    activate: function () {
                        Ext.Ajax.request({
                            url: '/GenerateCode.aspx',
                            method: 'POST',
                            headers: {
                                'Content-Type': 'text/plain', //Mono error for app/json https://bugzilla.xamarin.com/show_bug.cgi?id=6989
                            },
                            jsonData: syncVals,
                            success: function (response, options) {
                                try {
                                    var src = JSON.parse(response.responseText)['data'];
                                    Ext.get('qr').dom.src = src;
                                }
                                catch (e) {
                                    console.log("Error success: ", e);
                                }
                            },
                            failure: function (response, options) {
                                //Error
                            },
                        });
                    }
                }
            }],
        }).show();
    }
});