﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.VoiceInstance', {
    xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
    extend: 'Ext.form.Panel',

    title: 'Voice Command',
    description: 'Command of Voice',

    phrases: [],

    initComponent: function () {
        var me = this;

        Ext.define("VoiceCommand", {
            extend: 'Ext.data.Model',
            fields: ['Command', 'Phrase', 'Sensitivity'],
        });

        me.phraseStore = Ext.create('Ext.data.ArrayStore', {
            model: 'VoiceCommand',
            data: me.phrases,
            listeners: {
            }
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 2,
            listeners: {
                edit: function () {
                    var srvArgs = new fm.websync.serviceArgs("/lookingglass/config/checkwords", JSON.stringify(me.getValues().map(function (r) { return r.Phrase; })));
                    var lm = new Ext.LoadMask(me.up('form').down('#syncBtn'), { msg: "..." });

                    srvArgs.setOnSuccess(function (ss) {
                        var response = JSON.parse(ss.getDataJson());

                        if (response && response.length) {
                            var errWords = '';

                            response.forEach(function (phrase) {
                                if (phrase.hasOwnProperty('Errors') && phrase.Errors.length === 0) {
                                    //;
                                    console.log("Voice phrases updated successfully.")
                                }
                                else if (phrase.hasOwnProperty('Errors')) {
                                    phrase.Errors.forEach(function (w) {
                                        errWords += '<li>' + w + '</li>';
                                    });
                                }
                            });

                            if (errWords.length) {
                                Ext.Msg.alert('Error in voice sync', 'The following words were not found in the command dictionary.  Please check their spelling or replace them with more common words:<br /><ul>' + errWords + '</ul>');
                                me.up('form').down('#syncBtn').setDisabled(true);
                            } else {
                                me.up('form').down('#syncBtn').setDisabled(false);
                            }
                        }
                    });

                    srvArgs.setOnComplete(function () {
                        lm.hide();
                    })

                    MedwebCloud.websyncClient.service(srvArgs);
                }
            }
        });

        Ext.apply(this, {
            header: {
                title: me.title,
            },
            layout: {
                type: 'vbox',
                pack: 'center',
                align: 'center',
            },
            width: '100%',
            defaults: {
                margin: '10 0 10 0',
            },
            items: [{
                xtype: 'panel',
                width: '90%',
                layout: {
                    type: 'hbox',
                    align: 'middle',
                },
                border: false,
                items: [{
                    xtype: 'displayfield',
                    name: 'vcmd-text',
                    value: me.description,
                    border: false,
                    flex: 0.85,
                }, {
                    xtype: 'panel',
                    flex: 0.15,
                    border: false,
                    layout: {
                        type: 'hbox',
                        pack: 'end',
                    },
                    margin: '0 10 0 10',
                    items: [{
                        xtype: 'button',
                        text: 'Sync Now',
                        scale: 'small',
                        disabled: !(me.glasses && me.glasses.length),
                        itemId: 'syncBtn',
                        iconCls: 'sync-glasses-flat-icon',
                        iconAlign: 'top',
                        handler: function () {
                            if (me.glasses && me.glasses.length) {
                                Ext.Msg.toast({ body: 'Sync started...' });

                                me.glasses.forEach(function (glass) {
                                    MedwebCloud.websyncClient.publish(new fm.websync.publishArgs(glass.channel, JSON.stringify({
                                        Description: 'voice-update',
                                        Data: {
                                            Key: me.getItemId().substr(4),
                                            Phrases: me.getValues()
                                        }
                                    })));
                                });
                                Ext.Msg.toast({ body: 'Sync Completed' });
                            }
                            else {
                                Ext.Msg.alert({ title: 'Error in voice phrase', body: 'No glasses were found to sync to.  Please select a single pair of glasses.' });
                            }
                        }
                    }],
                }],
            }, {
                xtype: 'grid',
                width: '90%',
                viewConfig: {
                    deferEmptyText: false,
                    emptyText: '<div class="empty-grid">No phrases defined.</div>',
                    stripeRows: true
                },
                selType: 'rowmodel',
                plugins: [rowEditing],
                tbar: [{
                    text: 'Add Phrase',
                    iconCls: 'phrase-add-icon contain-image',
                    handler: function () {
                        rowEditing.cancelEdit();

                        var r = Ext.create('VoiceCommand', {
                            'Phrase': '',
                            'Sensitivity': 20,
                        });

                        me.phraseStore.insert(0, r);
                        rowEditing.startEdit(0, 0);
                    }
                }, {
                    itemId: 'removePhrase',
                    text: 'Remove Phrase',
                    iconCls: 'phrase-remove-icon contain-image',
                    handler: function () {
                        var grid = this.up('grid');
                        var sm = grid.getSelectionModel();
                        rowEditing.cancelEdit();
                        me.phraseStore.remove(sm.getSelection());
                        if (me.phraseStore.getCount() > 0) {
                            sm.select(0);
                        }
                    },
                    disabled: true
                }],
                columns: [
                    { dataIndex: 'Phrase', header: 'Command', flex: 0.8, sortable: true, editor: { xtype: 'textfield', allowBlank: false } },
                    { dataIndex: 'Sensitivity', header: 'Sensitivity', flex: 0.2, editor: { xtype: 'numberfield', allowBlank: false, minValue: 0, maxValue: 100 } },
                ],
                minHeight: 100,
                store: me.phraseStore,
                listeners: {
                    'selectionchange': function (view, records) {
                        this.down('#removePhrase').setDisabled(!records.length);
                    }
                },
            }],
        });

        this.callParent(arguments);
    },
    getValues: function () {
        var me = this;

        var phraseArray = [];

        me.phraseStore.each(function (rec) {
            phraseArray.push({
                "Phrase": rec.get("Phrase"),
                "Sensitivity": rec.get("Sensitivity"),
            });
        });

        return phraseArray;
    },
    setValues: function (valArr) {
        var me = this;

        if (valArr && valArr.length) {
            me.phraseStore.loadData(valArr);
        }
        else {
            me.phraseStore.loadData([]);
        }
    },
});