﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.Voice', {
    xtype: 'medwebcloud.lookingglasssetupwizard.voice',
    extend: 'Ext.form.Panel',

    initComponent: function () {
        var me = this;

        Ext.define("VoiceCommand", {
            extend: 'Ext.data.Model',
            fields: ['Command', 'Sensitivity'],
        });

        var rowEditing = Ext.create('Ext.grid.plugin.RowEditing', {
            clicksToEdit: 1
        });

        var rec = me.getRecord();
        var glasses = me.context.glasses;

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Configure Voice Commands:',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'If enabled, voice command recognition may still be turned off and on temporarily <br />by using the Start and Stop Listening commands.' +
                    '<br /><br />' +
                    'You may also display a message whenever a voice command is recognized for additional feedback.',
            }, {
                xtype: 'panel',
                layout: {
                    type: 'vbox',
                    pack: 'center',
                    align: 'left',
                },
                padding: '10 10 10 50',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                items: [{
                    xtype: 'checkbox',
                    itemId: 'AreVoiceCommandsEnabled',
                    name: 'AreVoiceCommandsEnabled',
                    boxLabel: 'Enable Voice commands',
                    boxLabelCls: 'wiz-check-label',
                }],
            }, {
                xtype: 'panel',
                layout: {
                    type: 'vbox',
                    pack: 'left',
                    align: 'left',
                },
                padding: '10 10 10 50',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                items: [{
                    xtype: 'checkbox',
                    itemId: 'IsVoiceFeedbackEnabled',
                    name: 'IsVoiceFeedbackEnabled',
                    boxLabel: 'Show a message when a voice command is recognized',
                    boxLabelCls: 'wiz-check-label',
                }],
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Start Listening',
                itemId: 'cmd-start-listening',
                description: 'Begin listening for voice commands',
                glasses: glasses,
                phrases: [
                    ['wakeup', 70],
                    ['wake up', 70]
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Stop Listening',
                itemId: 'cmd-stop-listening',
                description: 'Stop listening for voice commands',
                glasses: glasses,
                phrases: [
                    ['go to sleep', 70],
                    ['goto sleep', 70]
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }, {
                xtype: 'medwebcloud.lookingglasssetupwizard.voiceinstance',
                title: 'Help',
                itemId: 'cmd-help',
                description: 'Show/Hide the voice command help screen',
                glasses: glasses,
                phrases: [
                    ['help', 70],
                ],
                listeners: {
                    'commandsChanged': function () {
                        me.raiseValuesChanged(this.getValues());
                    },
                },
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    var wiz = this.up('panel').up();
                    var voice = wiz.getData('VoiceCommands');

                    //Handle the actual form fields
                    this.getForm().getFields().each(function (itm) {
                        var data = wiz.getData(itm.getItemId());
                        if (data != null && data != undefined) {
                            itm.setValue(data);
                        }
                    });

                    //Handle the voice-instances
                    this.items.each(function (itm) {
                        var id = itm.getItemId();
                        if (id.substr(0, 4) === 'cmd-') {
                            if (voice && voice.length) {
                                var cmdId = id.substr(4);
                                var cmds = voice.filter(function (c) { return c.Key === cmdId; });
                                if (cmds && cmds.length && cmds[0].Phrases) {
                                    itm.setValues(cmds[0].Phrases);
                                    return;
                                }
                            }
                        }
                    });
                },
            }
        });

        this.callParent(arguments);
    },
    raiseValuesChanged: function () {
        var me = this;

        ///me.fireEvent('valuesChanged', me.getValues());
    },
    saveChanges: function () {
        var me = this;

        me.getRecord()['AreVoiceCommandsEnabled'] = me.down('#AreVoiceCommandsEnabled').getValue();
        me.getRecord()['IsVoiceFeedbackEnabled'] = me.down('#IsVoiceFeedbackEnabled').getRawValue();

        var items = (!!this.getFormItems) ? this.getFormItems() : this.items;
        items.each(function (itm) {
            if (itm.getValues) {
                me.saveVoiceCommand(itm.getItemId().substr(4));
            }
        });
    },
    saveVoiceCommand: function (cmd) {
        var me = this;
        if (!me.getRecord()['VoiceCommands']) me.getRecord()['VoiceCommands'] = [];

        var values = me.down('#cmd-' + cmd).getValues();

        var vc = me.getRecord()['VoiceCommands'].filter(function (vc) {
            return vc.Key === cmd;
        });
        if (!vc || !vc.length) {
            me.getRecord()['VoiceCommands'].push({
                Key: cmd,
                Phrases: values,
            });
        }
        else {
            vc[0].Phrases = values;
        }
    },
});