﻿Ext.define('MedwebCloud.view.LookingGlassSetupWizard.WizComplete', {
    xtype: 'medwebcloud.lookingglasssetupwizard.wizcomplete',
    extend: 'Ext.form.Panel',

    originalData: {},
    newData: {},

    initComponent: function () {
        var me = this;

        me.glassConfigs = Ext.create('MedwebCloud.store.GlassSetups', {
            pageSize: 2147483647,
            filters: [{
                property: 'CompanyId',
                comparison: 'eq',
                value: MedwebCloud.userProfile.CompanyId
            }, {
                property: 'Name',
                comparison: 'ne',
                value: 'Default'
            }],
        });

        Ext.apply(this, {
            defaults: {
                margin: '0 0 20 0',
            },
            items: [{
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-heading',
                width: '100%',
                html: 'Review your changes',
            }, {
                xtype: 'panel',
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                html: 'Compare your changes with the original configuration.' +
                    '<br /><br />' +
                    'You can go back and change the values at any time; this page will automatically update.',
            }, {
                xtype: 'panel',
                layout: {
                    type: 'hbox',
                    pack: 'center',
                    align: 'center',
                },
                title: false,
                border: false,
                cls: 'wiz-text',
                width: '100%',
                items: [{
                    xtype: 'textfield',
                    fieldLabel: 'Save Configuration As',
                    name: 'ConfigName',
                    itemId: 'ConfigName',
                    width: 500,
                    labelWidth: 200,
                }, {
                    xtype: 'button',
                    iconCls: 'saveIcon contain-image',
                    margin: '0 0 0 20',
                    handler: function () {
                        var wiz = me.up('#wizard-cards').up();

                        var saveName = this.up('form').down('#ConfigName').getRawValue();

                        wiz.allDataObj.Name = saveName;

                        if (wiz.allDataObj.Id === -1) {
                            delete wiz.allDataObj.Id;
                            var rec = Ext.create('MedwebCloud.model.GlassSetup', wiz.allDataObj);
                            rec['VoiceCommands'] = wiz.allDataObj['VoiceCommands'];
                            rec['SupportedNetworks'] = wiz.allDataObj['SupportedNetworks'];
                            rec.getProxy().setWriter(Ext.create('MedwebCloud.IncludeRelationshipJsonWriter'));
                            rec.save({
                                callback: function (rec, op, success) {
                                    if (success) {
                                        wiz.allDataObj.Id = rec.get('Id');
                                        Ext.Msg.toast({ body: 'Save complete' });
                                    } else {
                                        Ext.Msg.alert({ title: 'Save Error', body: 'An error occurred when attempting to save your changes.  Please try again or contact your network administrator if the problem persists.' });
                                        console.log("Error saving configuration.", arguments);
                                    }
                                }
                            });
                            //var rec = me.glassConfigs.add(rec)[0];
                            //me.glassConfigs.sync();
                        }
                        else {
                            var rec = Ext.create('MedwebCloud.model.GlassSetup', wiz.allDataObj);
                            rec['VoiceCommands'] = wiz.allDataObj['VoiceCommands'];
                            rec['SupportedNetworks'] = wiz.allDataObj['SupportedNetworks'];
                            rec.getProxy().setWriter(Ext.create('MedwebCloud.IncludeRelationshipJsonWriter'));
                            rec.save();
                        }
                    }
                }],
            }, {
                xtype: 'panel',
                layout: {
                    type: 'hbox',
                },
                border: false,
                cls: 'wizard-inner-form',
                items: [{
                    xtype: 'panel',
                    border: false,
                    itemId: 'oldDataContainer',
                    header: {
                        title: 'Original Settings',
                        titleAlign: 'center',
                        cls: 'wiz-inner-header',
                    },
                    defaults: {
                        width: '100%',
                        margin: '5 5 5 5',
                    },
                    layout: {
                        type: 'vbox',
                        align: 'center',
                    },
                    flex: 0.45,
                    cls: 'wiz-back',
                    items: [],
                }, {
                    xtype: 'panel',
                    border: false,
                    flex: 0.1,
                    cls: 'spacer',
                }, {
                    xtype: 'panel',
                    border: false,
                    itemId: 'newDataContainer',
                    header: {
                        title: 'New Settings',
                        titleAlign: 'center',
                        cls: 'wiz-inner-header',
                    },
                    defaults: {
                        width: '100%',
                        padding: '5 5 5 5',
                    },
                    layout: {
                        type: 'vbox',
                        align: 'center',
                    },
                    flex: 0.45,
                    cls: 'wiz-back',
                    items: [],
                }]
            }],
            listeners: {
                beforeactivate: function () {
                    var form = this;
                    if (me.activated) return;
                    console.log("Activating");
                    me.activated = true;

                    me.loadMask = new Ext.LoadMask(me.up('#wizard-cards'), {
                        msg: "Formatting changes...",
                    });
                    me.loadMask.show();

                    var wiz = me.up('#wizard-cards').up();
                    var name = wiz.allDataObj.Name;

                    if (name && name !== 'Default') {
                        form.down('#ConfigName').setValue(name);
                    } else {
                        form.down('#ConfigName').setValue('');
                    }

                    me.reloadData();

                    //Wait for afterLayout to hide the loadMask
                },
                deactivate: function () {
                    me.activated = false;
                    me.spinner = null;
                    me.loadMask.hide();
                },
                afterlayout: function () {
                    console.log('afterlayout');
                    if (!me.spinner) {
                        me.spinner = function () {
                            setTimeout(function () {
                                if (!me.activated) return;

                                if (me.toggled) {
                                    me.toggled = false;
                                    me.spinner();
                                    return;
                                }
                                else {
                                    me.loadMask.hide();
                                }
                            }, 100)
                        }
                        me.spinner();
                    }
                    else {
                        me.toggled = true;
                    }
                },
            }
        });

        this.callParent(arguments);
    },
    reloadData: function () {
        var me = this;

        me.newData = me.getChangedData();

        console.log("WizChangedData:", me.newData);

        var newDataContainer = me.down('#newDataContainer');
        var oldDataContainer = me.down('#oldDataContainer');

        newDataContainer.removeAll();
        oldDataContainer.removeAll();

        var origDataItems = [];
        var newDataItems = [];
        for (var prop in me.newData) {
            if (me.newData.hasOwnProperty(prop)) {
                var origFields = [];
                var newFields = [];

                for (innerProp in me.newData[prop]) {
                    if (typeof (me.newData[prop][innerProp]) === typeof ([]) && me.newData[prop][innerProp].length) {
                        var ps = [];
                        var cols = [];
                        for (var key in me.newData[prop][innerProp][0]) {
                            ps.push(key);
                            cols.push({
                                text: key,
                                dataIndex: key,
                            })
                        }

                        if (me.originalData[prop] && me.originalData[prop][innerProp] && typeof (me.originalData[prop][innerProp]) === typeof ([]) && me.originalData[prop][innerProp].length) {
                            var origPropStore = Ext.create('Ext.data.Store', {
                                fields: ps,
                                data: me.originalData[prop][innerProp],
                            });

                            origFields.push({
                                xtype: 'displayfield',
                                editable: false,
                                fieldLabel: innerProp,
                                labelAlign: 'left',
                                value: '',
                            });

                            origFields.push({
                                xtype: 'grid',
                                title: false,
                                border: false,
                                columns: cols,
                                padding: '5 5 5 5',
                                store: origPropStore,
                            });
                        } else {
                            origFields.push({
                                xtype: 'displayfield',
                                editable: false,
                                fieldLabel: innerProp,
                                labelAlign: 'top',
                                value: '<span class="wiz-disabled wiz-final-value">Unavailable</span>',
                            });
                        }

                        var newPropStore = Ext.create('Ext.data.Store', {
                            fields: ps,
                            data: me.newData[prop][innerProp],
                        });

                        newFields.push({
                            xtype: 'displayfield',
                            editable: false,
                            fieldLabel: innerProp,
                            labelAlign: 'left',
                            value: '',
                        });

                        newFields.push({
                            xtype: 'grid',
                            title: false,
                            border: false,
                            columns: cols,
                            padding: '5 5 5 5',
                            store: newPropStore,
                        });
                    }
                    else if (typeof (me.originalData[prop][innerProp]) === typeof ([]) && me.originalData[prop][innerProp].length) {
                        var ps = [];
                        var cols = [];
                        for (var key in me.originalData[prop][innerProp][0]) {
                            ps.push(key);
                            cols.push({
                                text: key,
                                dataIndex: key,
                            })
                        }

                        newFields.push({
                            xtype: 'displayfield',
                            editable: false,
                            fieldLabel: innerProp,
                            labelAlign: 'top',
                            value: '<span class="wiz-disabled wiz-final-value">Unavailable</span>',
                        });

                        var oldPropStore = Ext.create('Ext.data.Store', {
                            fields: ps,
                            data: me.originalData[prop][innerProp],
                        });

                        origFields.push({
                            xtype: 'displayfield',
                            editable: false,
                            fieldLabel: innerProp,
                            labelAlign: 'left',
                            value: '',
                        });

                        origFields.push({
                            xtype: 'grid',
                            title: false,
                            border: false,
                            columns: cols,
                            padding: '5 5 5 5',
                            store: oldPropStore,
                        });
                    }
                    else {
                        if (me.originalData[prop] && me.originalData[prop][innerProp]) {
                            origFields.push({
                                xtype: 'displayfield',
                                editable: false,
                                fieldLabel: innerProp,
                                labelAlign: 'top',
                                padding: '5 5 5 5',
                                value: me.originalData[prop][innerProp],
                            });
                        } else {
                            origFields.push({
                                xtype: 'displayfield',
                                editable: false,
                                fieldLabel: innerProp,
                                labelAlign: 'left',
                                value: '<span class="wiz-disabled wiz-final-value">Unavailable</span>',
                            });
                        }

                        newFields.push({
                            xtype: 'displayfield',
                            editable: false,
                            fieldLabel: innerProp,
                            labelAlign: 'top',
                            padding: '5 5 5 5',
                            value: me.newData[prop][innerProp],
                        });
                    }
                }

                oldDataContainer.add({
                    xtype: 'panel',
                    border: false,
                    header: {
                        title: prop,
                    },
                    items: origFields,
                });

                newDataContainer.add({
                    xtype: 'panel',
                    border: false,
                    header: {
                        title: prop,
                    },
                    items: newFields,
                });

                /*
                origDataItems.push({
                    xtype: 'panel',
                    border: false,
                    header: {
                        title: prop,
                    },
                    items: origFields,
                });

                newDataItems.push({
                    xtype: 'panel',
                    border: false,
                    header: {
                        title: prop,
                    },
                    title: prop,
                    items: newFields,
                });*/
            }
        }
    },
    saveChanges: function () {
        //;
    },
});