﻿Ext.define('MedwebCloud.view.GeneralConfigWizardPage', {
    xtype: 'medwebcloud.generalconfigwizardpage',
    extend: 'Ext.Panel',

    initComponent: function () {
        var me = this;

        var pinnedSessions = Ext.create('MedwebCloud.store.Sessions', {
            pageSize: 2147483647,
            filters: [{
                property: 'CompanyId',
                comparison: 'eq',
                value: MedwebCloud.userProfile.CompanyId
            }, {
                property: 'Pinned',
                comparison: 'eq',
                value: 1
            }],
        });

        Ext.apply(this, {
            xtype: 'form',
            layout: {
                type: 'vbox',
                pack: 'center',
                align: 'center',
            },
            items: [{
                xtype: 'fieldset',
                cls: 'wiz-fieldset',
                width: '90%',
                padding: 10,
                items: [{
                    xtype: 'panel',
                    layout: 'hbox',
                    width: '100%',
                    border: false,
                    items: [{
                        xtype: 'label',
                        forId: 'sessionSelect',
                        flex: 1,
                        text: 'Medweb LIVE! Session:',
                        margin: '7 0 0 0',
                    }, {
                        xtype: 'combo',
                        itemId: 'sessionSelect',
                        name: 'DefaultSessionId',
                        margin: '5 0 0 0',
                        listConfig: {
                            cls: 'wiz-drop',
                        },
                        id: 'sessionSelect',
                        store: pinnedSessions,
                        displayField: 'Name',
                        valueField: 'Id',
                        forceSelection: true,
                        multiSelect: false,
                        triggerAction: 'all',
                        selectOnFocus: false,
                        disabled: true,
                        queryMode: 'local',
                        lastQuery: '',
                        width: 300,
                    }],
                }, {
                    xtype: 'displayfield',
                    anchor: '0 0',
                    margin: '10 0 0 0',
                    cls: 'wiz-subtext',
                    value: 'You may select any pinned Medweb LIVE! session to automatically connect to with Looking Glass.  If a session is selected, whenever the glasses have network access, they will connect to that session.  This setting cannot be changed from within the glasses.',
                }],
            }, {
                xtype: 'fieldset',
                width: '90%',
                cls: 'wiz-fieldset',
                padding: 10,
                items: [{
                    xtype: 'medwebcloud.togglebutton',
                    onState: me.valueSet.IsLookingGlassEnabled,
                    labelText: 'Allow Remote Viewing:',
                    itemId: 'rv-toggle',
                }, {
                    xtype: 'displayfield',
                    anchor: '0 0',
                    margin: '10 0 0 0',
                    cls: 'wiz-subtext',
                    value: 'If Remote Viewing is enabled, these glasses can be observed from the Looking Glass tab without needing to be signed into a Medweb LIVE! session.  This will also report the glasses\' current location so that they can be tracked on the map.',
                }],
            }],
            listeners: {
                activate: function () {
                    var form = this;

                    pinnedSessions.load(function () {
                        this.insert(0, Ext.create('MedwebCloud.model.Session', {
                            Id: 0,
                            Name: 'Do not automatically connect to Medweb LIVE!'
                        }));

                        form.down('#sessionSelect').on('enable', function (box, eOpts) {
                            try {
                                var val = me.valueSet.DefaultSessionId;
                                if (me.pinnedSessions.findExact('Id', val) > -1) {
                                    box.setValue(me.valueSet.DefaultSessionId);
                                }
                                else {
                                    box.setValue(0);
                                }
                            } catch (e) {
                                box.setValue(0);
                            }
                        });
                        form.down('#sessionSelect').setDisabled(false);
                    });
                },
                beforedeactivate: function () {
                    me.valueSet.DefaultSessionId = me.down('#sessionSelect').getValue();
                    me.valueSet.DefaultSession = {};
                    me.valueSet.DefaultSession.Name = me.down('#sessionSelect').getRawValue();

                    me.valueSet.IsLookingGlassEnabled = me.down('#rv-toggle').getValue();
                },
            },
        });

        me.callParent(arguments);
    },
    getValues: function () {
        return { 'Config': null };
    }
});