﻿Ext.define('MedwebCloud.view.SummaryWizardPage', {
    xtype: 'medwebcloud.summarywizardpage',
    extend: 'Ext.Panel',

    initComponent: function () {
        var me = this;

        Ext.apply(this, {
            xtype: 'panel',
            layout: 'fit',
            items: [{
                xtype: 'panel',
                itemId: 'summaryContainer',
                layout: {
                    type: 'anchor',
                },
                overflowY: 'auto',
                width: '100%',
                padding: '0 10% 0 10%',
                border: false,
                items: me.checkDiff(),
            }],
            listeners: {
                activate: function () {
                    //console.log("Summary of changes: ", JSON.stringify(me.oldValues), JSON.stringify(me.newValues));

                    var c = me.down('#summaryContainer');
                    me.remove(c);
                    var itms = me.checkDiff();
                    //console.log(itms);
                    me.add({
                        xtype: 'panel',
                        layout: {
                            type: 'anchor',
                        },
                        overflowY: 'auto',
                        itemId: 'summaryContainer',
                        width: '100%',
                        padding: '0 10% 0 10%',
                        border: false,
                        items: itms,
                    });
                    me.doLayout();
                }
            }
        });

        me.callParent(arguments);
    },
    checkDiff: function () {
        var me = this;
        try {
            var items = [];
            if (me.oldValues.DefaultSessionId !== me.newValues.DefaultSessionId) {
                var oldSession = me.oldValues.DefaultSession;

                if (oldSession === null) {
                    oldSession = { 'Name': 'None' };
                }

                if (me.newValues.DefaultSessionId > 0 || me.oldValues.DefaultSessionId) {
                    items.push({
                        xtype: 'panel',
                        width: '50%',
                        border: false,
                        header: {
                            style: 'background: none !important; box-shadow: none !important;',
                            title: 'Medweb LIVE! Session',
                        },
                        items: [{
                            xtype: 'displayfield',
                            value: oldSession.Name + '   \u25BA   ' + me.newValues.DefaultSession.Name,
                            padding: '0 0 0 40',
                        }],
                    });
                }
            }

            if (me.oldValues.IsLookingGlassEnabled !== me.newValues.IsLookingGlassEnabled) {
                items.push({
                    xtype: 'panel',
                    width: '50%',
                    border: false,
                    header: {
                        style: 'background: none !important; box-shadow: none !important;',
                        title: 'Remote Viewing',
                    },
                    items: [{
                        xtype: 'displayfield',
                        value: (me.oldValues.IsLookingGlassEnabled ? 'Enabled' : 'Disabled') + '   \u25BA   ' + (me.newValues.IsLookingGlassEnabled ? 'Enabled' : 'Disabled'),
                        padding: '0 0 0 40',
                    }],
                });
            }

            if (me.oldValues.AreVoiceCommandsEnabled !== me.newValues.AreVoiceCommandsEnabled) {
                items.push({
                    xtype: 'panel',
                    width: '50%',
                    border: false,
                    header: {
                        style: 'background: none !important; box-shadow: none !important;',
                        title: 'Voice Commands',
                    },
                    items: [{
                        xtype: 'displayfield',
                        value: (me.oldValues.AreVoiceCommandsEnabled ? 'Enabled' : 'Disabled') + '   \u25BA   ' + (me.newValues.AreVoiceCommandsEnabled ? 'Enabled' : 'Disabled'),
                        padding: '0 0 0 40',
                    }],
                });
            }

            if (me.newValues.AreVoiceCommandsEnabled && (me.oldValues.IsVoiceFeedbackEnabled !== me.newValues.IsVoiceFeedbackEnabled)) {
                items.push({
                    xtype: 'panel',
                    width: '50%',
                    border: false,
                    header: {
                        style: 'background: none !important; box-shadow: none !important;',
                        title: 'Voice Command Feedback',
                    },
                    items: [{
                        xtype: 'displayfield',
                        value: (me.oldValues.IsVoiceFeedbackEnabled ? 'Enabled' : 'Disabled') + '   \u25BA   ' + (me.newValues.IsVoiceFeedbackEnabled ? 'Enabled' : 'Disabled'),
                        padding: '0 0 0 40',
                    }],
                });
            }

            var subitems = [];
            for (var i = 0; i < me.newValues.VoiceCommands.length; i++) {
                var cmd = me.newValues.VoiceCommands[i];
                for (var j = 0; j < me.oldValues.VoiceCommands.length; j++) {
                    var oldCmd = me.oldValues.VoiceCommands[j];

                    if (cmd.Key === oldCmd.Key) {
                        var oldPhrases = JSON.parse(JSON.stringify(oldCmd.Phrases));
                        var newPhrases = JSON.parse(JSON.stringify(cmd.Phrases));
                        var chgPhrases = [];

                        for (var k = newPhrases.length - 1; k >= 0 ; k--) {
                            var found = false;
                            for (var l = oldPhrases.length - 1; l >= 0; l--) {
                                if (newPhrases[k].Phrase === oldPhrases[l].Phrase) {
                                    if (newPhrases[k].Sensitivity !== oldPhrases[l].Sensitivity) {
                                        chgPhrases.push({ phrase: newPhrases[k].Phrase, oldS: oldPhrases[l].Sensitivity, newS: newPhrases[k].Sensitivity });
                                    }
                                    oldPhrases.splice(l, 1);
                                    found = true;
                                }
                            }
                            if (found) {
                                newPhrases.splice(k, 1);
                            }
                        }

                        var subheader = {
                            xtype: 'panel',
                            width: '100%',
                            border: false,
                            padding: '0 0 0 40',
                            bodyStyle: 'border: none !important;',
                            header: {
                                style: 'background: none !important; box-shadow: none !important;',
                                title: cmd.Command,
                            },
                            items: [],
                        };

                        if (oldPhrases.length > 0 || newPhrases.length > 0 || chgPhrases.length > 0) {
                            subitems.push(subheader);
                        }

                        for (var m = 0; m < newPhrases.length; m++) {
                            subheader.items.push({
                                xtype: 'displayfield',
                                value: newPhrases[m].Phrase + ' (' + newPhrases[m].Sensitivity + ')' + '   \u25BA   ' + 'Added',
                                padding: '0 0 0 40',
                            });
                        }

                        for (var m = 0; m < oldPhrases.length; m++) {
                            subheader.items.push({
                                xtype: 'displayfield',
                                value: oldPhrases[m].Phrase + ' (' + oldPhrases[m].Sensitivity + ')' + '   \u25BA   ' + 'Removed',
                                padding: '0 0 0 40',
                            });
                        }

                        for (var m = 0; m < chgPhrases.length; m++) {
                            subheader.items.push({
                                xtype: 'displayfield',
                                value: chgPhrases[m].phrase + ' (' + chgPhrases[m].oldS + ')' + '   \u25BA   ' + '(' + chgPhrases[m].newS + ')',
                                padding: '0 0 0 40',
                            });
                        }
                    }
                }
            }

            if (subitems.length > 0) {
                items.push({
                    xtype: 'panel',
                    width: '50%',
                    border: false,
                    header: {
                        style: 'background: none !important; box-shadow: none !important;',
                        title: 'Command Phrases',
                    },
                    items: subitems,
                });
            }

            if (items.length === 0) {
                items.push({
                    xtype: 'panel',
                    width: '100%',
                    minHeight: 200,
                    header: false,
                    border: false,
                    layout: {
                        type: 'vbox',
                        align: 'center',
                        pack: 'center',
                    },
                    items: [{
                        xtype: 'displayfield',
                        value: 'No changes have been made.',
                    }]
                });
            }

            return items;
        }
        catch (ex) {
            console.log(ex);
        }
    }
});