﻿Ext.define('MedwebCloud.view.SessionPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.sessionpanel',
    requires: ['MedwebCloud.view.VideoPanel', 'MedwebCloud.view.ImagesPanel', 'MedwebCloud.view.ChatPanel', 'MedwebCloud.view.VitalsPanel', 'MedwebCloud.view.InvitationPanel'],
    record: null,
    defaultSection: 'devices',
    updateTitle: function () {
        this.down('#sessionBbar').down('.tbtext').setText(this.getTitle());
    },
    getTitle: function () {
        var title = MedwebCloud.getMsg('medweb.view.sessionpanel.title', this.record.get('SessionCode'));
        if (this.record.get('Pinned')) {
            title = this.record.get('Name') + ' (' + this.record.get('SessionCode') + ')';
        }
        return title;
    },
    initComponent: function () {
        var me = this;
        var record = this.record;
        var btnLeave = MedwebCloud.getMsg('medweb.view.sessionpanel.button.leave.label');
        var btnImage = MedwebCloud.getMsg('medweb.view.sessionpanel.button.images.label');
        var btnRemoteCameras = MedwebCloud.getMsg('medweb.view.sessionpanel.button.remotecameras.label');
        var btnDevice = MedwebCloud.getMsg('medweb.view.sessionpanel.button.devices.label');
        var titleVitals = MedwebCloud.getMsg('medweb.view.sessionpanel.section.vitals.title');
        var btnInvite = MedwebCloud.getMsg('medweb.view.sessionpanel.button.invite.label');
        var btnScreenShare = MedwebCloud.getMsg('medweb.view.sessionpanel.button.screenshare.label');

        var sessionTitle = this.getTitle();

        // defaultSection

        var activateSection = function (config) {
            var button = me.down('#sessionBbar').down('button[itemId=' + config.itemId + ']');
            var panel = me.down('#' + config.itemId + 'Panel');
            if (button && panel) {
                var all = me.down('#sessionBbar').down('button');
                for (var i = 0; i < all.length; i++) {
                    all[i].toggle(false);
                }
                me.down('#sessionPanels').getLayout().setActiveItem(panel);
                button.toggle(true);
            }
        };

        var tbarItems = [{
            text: btnLeave,
            iconCls: 'leaveIcon',
            handler: function () {
                me.fireEvent('leave');
            }
        }];
        var creator = record.get('User')
        var profile = MedwebCloud.userProfile;

        if (creator.Username === profile.Username) {
            tbarItems.push('->');
            tbarItems.push({
                text: record.get('Pinned') ? 'Unpin' : 'Pin',
                iconCls: record.get('Pinned') ? 'unpinIcon' : 'pinIcon',
                pressed: record.get('Pinned') ? true : false,
                handler: function (btn) {
                    if (record.get('Pinned')) {
                        btn.setIconCls('pinIcon');
                        btn.setText('Pin');
                        record.set('Pinned', false);
                        record.save();
                        btn.toggle(false);

                        me.updateTitle();
                    } else {
                        var f = new Ext.FormPanel({
                            labelWidth: 100,
                            region: 'north',
                            labelAlign: 'left',
                            frame: true,
                            border: false,
                            labelStyle: 'font-weight:bold;',
                            padding: '10 10 10 5',
                            items: [{
                                xtype: 'textfield',
                                fieldLabel: 'Session Name',
                                name: 'name',
                                vtype: 'text'
                            }, {
                                xtype: 'checkbox',
                                fieldLabel: 'Public Session?',
                                inputValue: true,
                                name: 'public'
                            }],
                            buttons: [{
                                text: 'OK',
                                minWidth: 75,
                                handler: function () {
                                    var vals = f.getValues();
                                    if (!vals.name || Ext.String.trim(vals.name).length == 0) {
                                        vals.name = MedwebCloud.userProfile.DisplayName;
                                    }
                                    btn.setIconCls('unpinIcon');
                                    btn.setText('Unpin');
                                    record.set('Name', vals.name);
                                    record.set('Pinned', true);
                                    record.set('Public', !!vals['public']);
                                    record.set('CompanyId', MedwebCloud.userProfile.CompanyId)
                                    record.save();

                                    btn.toggle(true);
                                    btn.blur();
                                    me.updateTitle();
                                    pinDlg.close()
                                }
                            }, {
                                text: 'Cancel',
                                minWidth: 75,
                                handler: function () {
                                    pinDlg.close()
                                }
                            }]
                        });
                        var pinDlg = new Ext.Window({
                            closable: true,
                            closeAction: 'destroy',
                            modal: true,
                            title: 'Pin Session',
                            layout: 'fit',
                            items: f
                        });
                        pinDlg.show();
                    }
                }
            });
        }

        Ext.apply(me, {
            itemId: 'sessionPanel',
            layout: 'border',
            tbar: {
                items: tbarItems
            },
            defaults: {
                border: false
            },
            items: [{
                region: 'west',
                layout: 'border',
                stateful: true,
                stateId: 'session-west',
                width: '50%',
                split: true,
                items: [{
                    region: 'center',
                    border: false,
                    flex: 5,
                    bodyStyle: 'background-color:#000;color:#ccc;',
                    xtype: 'MedwebCloud.videopanel',
                    client: this.client,
                    draw: true,
                    trackPulse: true,
                    labelRemote: true,
                    uniqueId: this.record.get('Id'),
                    listeners: {
                        'up': function (state) {
                            try {
                                var notification = soundManager.createSound({
                                    url: '/sounds/notification.mp3'
                                });
                                notification.play();
                            } catch (e) { }
                        },
                        'photo': function (base64data) {
                            console.log("Received photo event");
                            me.down('#imagesPanel').addPanel(base64data, true, 'RemoteSnapshot');
                        },
                    }
                }, {
                    region: 'south',
                    flex: 2,
                    split: true,
                    border: false,
                    layout: {
                        type: 'vbox',
                        align: 'stretch'
                    },
                    items: [{
                        flex: 1,
                        split: true,
                        //title: 'Chat',
                        xtype: 'MedwebCloud.chatpanel',
                        client: this.client,
                        record: this.record
                    }, {
                        height: 120,
                        title: titleVitals,
                        collapsible: true,
                        xtype: 'MedwebCloud.vitalspanel',
                        client: this.client,
                        record: this.record
                    }]
                }]
            }, {
                //flex: 2,
                region: 'center',
                xtype: 'panel',
                layout: 'card',
                itemId: 'sessionPanels',
                //tabPosition: 'bottom',
                border: false,
                bbar: {
                    itemId: 'sessionBbar',
                    items: [{
                        xtype: 'button',
                        scale: 'large',
                        text: btnDevice,
                        itemId: 'devices',
                        icon: '/images/stethoscope.png',
                        enableToggle: true,
                        toggleGroup: 'sessionPanels',
                        pressed: true,
                        handler: activateSection
                    }, {
                        xtype: 'button',
                        scale: 'large',
                        text: btnImage,
                        itemId: 'images',
                        icon: '/images/photo_landscape2.png',
                        enableToggle: true,
                        toggleGroup: 'sessionPanels',
                        handler: activateSection
                    }, {
                        xtype: 'button',
                        scale: 'large',
                        text: btnRemoteCameras,
                        itemId: 'remoteCameras',
                        icon: '/images/dome_camera.png',
                        enableToggle: true,
                        toggleGroup: 'sessionPanels',
                        handler: activateSection
                    }, {
                        xtype: 'button',
                        scale: 'large',
                        text: btnScreenShare,
                        itemId: 'screenShare',
                        icon: '/images/monitor32.png',
                        enableToggle: true,
                        toggleGroup: 'sessionPanels',
                        handler: activateSection
                    }, '->', {
                        xtype: 'tbtext',
                        text: sessionTitle
                    }, '->', {
                        xtype: 'button',
                        scale: 'large',
                        text: btnInvite,
                        icon: '/images/mail_add.png',
                        handler: this.showInvitationGrid.bind(this)
                    }]
                },
                items: [{
                    //title: btnDevice,
                    //icon: '/images/monitor.png',
                    xtype: 'MedwebCloud.videopanel',
                    client: this.client,
                    itemId: 'devicesPanel',
                    subchannel: 'devices',
                    uniqueId: this.record.get('Id'),
                    //pendingText: '<span style="display:block;width:100%;text-align:center">No Feed Currently Available</span>',
                    autoJoin: true,
                    zoom: true,
                    border: false,
                    permittedPeerIds: [],
                    canSnapshot: false,
                    options: {
                        audio: false,
                        video: false
                    }
                }, {
                    //title: btnImage,
                    //icon: '/images/picture.png',
                    xtype: 'MedwebCloud.imagespanel',
                    itemId: 'imagesPanel',
                    client: this.client,
                    record: this.record,
                    border: false,
                    listeners: {
                        imageReceived: function (count) {
                            var button = me.down('#images');
                            button.setText(MedwebCloud.getMsg('medweb.view.sessionpanel.section.images.count.title', count));
                            button.toggle(true);
                            me.down('#sessionPanels').getLayout().setActiveItem(1);
                        },
                        focusRequested: function (imgId) {
                            me.handleFocusRequest(0 /*'Image'*/, imgId);
                        },
                        unfocusRequested: function (imgId) {
                            me.handleUnfocusRequest(0 /*'Image'*/, imgId);
                        }

                    }
                }, {
                    xtype: 'medwebcloud.remotecameraspanel',
                    itemId: 'remoteCamerasPanel',
                    client: this.client,
                    record: this.record,
                    border: false
                }, {
                    xtype: 'medwebcloud.screensharepanel',
                    itemId: 'screenSharePanel',
                    client: this.client,
                    record: this.record,
                    border: false
                }]
            }]
        });
        //alert(this.getChannel());
        var client = MedwebCloud.websyncClient;
        client.subscribe({
            channel: this.getChannel(),
            onReceive: function (e) {
                // for now, nada
            }
        });

        client.subscribe({
            channel: me.getChannel() + '/activity',
            onReceive: function (e) {
                if (!e.getWasSentByMe()) {
                    var obj = JSON.parse(e.getDataJson());
                    console.log("Received activity object ", obj);
                    switch (obj.Type) {
                        case 0://'Image':
                            if (obj.Action == 2) { focus
                                me.down('#imagesPanel').showImage(obj.Data.ImageId);
                                Ext.Msg.toast({ title: 'Focus Requested', body: 'Focus was requested by another user.' });
                            } else if (obj.Action === 3) {
                                Ext.Msg.toast({ title: 'Focus Release', body: 'Another user released focus' });
                            }
                            break;

                        default:
                            console.log("Could not handle unknown Type: ", obj.Type);
                            break;
                    }
                }
            }
        });

        this.on('afterrender', function () {
            activateSection({
                itemId: me.defaultSection || 'devices'
            })
        });

        me.callParent(arguments);
    },
    handleFocusRequest: function (focusType, payload) {
        var client = MedwebCloud.websyncClient;
        client.publish({
            channel: this.getChannel() + '/activity',
            data: {
                Type: focusType,
                Action: 2,//'Focus',
                Data: payload,
            }
        })
    },
    handleUnfocusRequest: function (focusType, payload) {
        var client = MedwebCloud.websyncClient;
        client.publish({
            channel: this.getChannel() + '/activity',
            data: {
                Type: focusType,
                Action: 3,//'Focus',
                Data: payload,
            }, onSuccess: function () {
                Ext.Msg.toast({ title: 'Focus Released', body: 'You have released focus' });
            }
        })
    },
    onRender: function () {
        this.callParent(arguments);

        var me = this;
        window.setTimeout(function () {
            if (window.File && window.FileReader && window.FileList && window.Blob) {
                var dropZone = $(me.getLayout().getRenderTarget().dom);
                dropZone.on('dragover', Ext.bind(me.handleDragOver, me));
                dropZone.on('drop', Ext.bind(me.handleFileSelect, me));
            } else {
                console.log('The File APIs are not fully supported in this browser.');
            }
        }, 1);
    },
    handleDragOver: function (evt) {
        evt.stopPropagation();
        evt.preventDefault();
        evt.originalEvent.dataTransfer.dropEffect = 'copy'; // Explicitly show this is a copy.
    },
    handleFileSelect: function (evt) {
        var files = evt.originalEvent.dataTransfer.files; // FileList object
        var me = this;

        // Loop through the FileList and render image files as thumbnails.
        for (var i = 0, f; f = files[i]; i++) {
            // Only process image files.
            if (!f.type.match('image.*')) {
                continue;
            }

            var reader = new FileReader();

            // Closure to capture the file information.
            reader.onload = (function (theFile) {
                return function (e) {
                    var prefix = 'data:image/jpeg;base64,';
                    var data = e.target.result;
                    if (data.indexOf(prefix) == 0) {
                        data = data.replace(prefix, '');
                    }
                    var imagesPanel = me.down('#imagesPanel');
                    imagesPanel.addPanel(data, true, theFile.name);
                };
            })(f);

            // Read in the image file as a data URL.
            reader.readAsDataURL(f);
        }
    },
    onDestroy: function () {
        var client = MedwebCloud.websyncClient;
        client.unsubscribe({
            channel: this.getChannel()
        });
        this.callParent(arguments);
    },
    getChannel: function () {
        return '/collage/' + this.record.get('Id');
    },
    showInvitationGrid: function () {
        var me = this;
        var title = MedwebCloud.getMsg('medweb.contact.create_and_invite.window.title')
        var btnPos = MedwebCloud.getMsg('medweb.view.sessionpanel.action.invite.button.postive.label');
        var btnNeg = MedwebCloud.getMsg('medweb.view.sessionpanel.action.invite.button.negative.label');

        var w = new Ext.Window({
            center: true,
            height: 600,
            width: 600,
            border: false,
            modal: true,
            layout: 'fit',
            title: title,
            items: [{
                itemId: 'invitationpanel',
                xtype: 'MedwebCloud.invitationpanel',
                record: me.record,
                listeners: {
                    invite: function () {
                        w.close();
                    },
                    //TODO: on select, activate button; code here isn't working...
                    select: function () {
                        w.down('#invite').enable();
                    }
                }
            }],
            buttons: [{
                text: btnNeg,
                itemId: 'new',
                handler: function () {
                    w.close();
                }
            }, {
                text: btnPos,
                itemId: 'invite',
                //disabled: true,
                handler: function () {
                    w.down('#invitationpanel').inviteSelected();
                }
            }]
        });
        w.show();
    }
});