﻿Ext.define('MedwebCloud.view.EditorGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'medwebcloud.editorgrid',
    require: ['Ext.ux.grid.FiltersFeature'],

    addEnabled: false,
    removeEnabled: false,
    autoSave: false,
    beforeAdd: null,

    storeType: null,
    itemName: 'Record',

    columns: [{
        dataIndex: 'Id',
        text: 'Id',
        flex: 1,
    }],

    initComponent: function () {
        if (!this.storeType) {
            throw new "InvalidArgumentException";
        }

        var me = this;

        var toolbar = { items: [] };
        if (me.addEnabled) {
            toolbar.items.push({
                text: 'Add ' + me.itemName,
                itemId: 'addButton',
                iconCls: 'addIcon',
                handler: function (e) {
                    me.addItem();
                }
            });
        }
        if (me.removeEnabled) {
            toolbar.items.push({
                text: 'Remove ' + me.itemName,
                itemId: 'removeButton',
                iconCls: 'deleteIcon',
                disabled: true,
                handler: function (e) {
                    me.removeItem();
                }
            });
        }

        if (!me.autoSave) {
            //Right-align Save button
            toolbar.items.push({
                xtype: 'tbfill'
            });
            toolbar.items.push({
                text: 'Cancel',
                itemId: 'cancelButton',
                iconCls: 'endIcon',
                disabled: true,
                handler: function (e) {
                    me.cancelChanges();
                }
            });
            toolbar.items.push({
                text: 'Save Changes',
                itemId: 'saveButton',
                iconCls: 'saveIcon',
                disabled: true,
                handler: function (e) {
                    me.saveChanges();
                }
            });
        }

        var doSave = function () {
            if (me.autoSave) {
                me.saveChanges();
            } else {
                me.setAllowSave(this.needsSync());
            }
        };

        var itemStore = Ext.create(me.storeType, {
            autoLoad: true,
            remoteSort: true,
            remoteFilter: true,
            filters: me.filters,
            expand: me.expander,
            listeners: {
                load: function () {
                    me.setAllowSave(false);
                    this.on('datachanged', doSave, me);
                }
            }
        });

        Ext.apply(me, {
            store: itemStore,
            columns: me.columns,
            tbar: toolbar,
            features: me.filterer == null ? [] : [me.filterer],
            dockedItems: [{
                xtype: 'pagingtoolbar',
                store: itemStore,   // same store GridPanel is using
                dock: 'bottom',
                displayInfo: true
            }],
            listeners: {
                itemdblclick: function (view, record, item, index, e, eOpts) {
                    me.fireEvent('requestEdit', record);
                },
                selectionchange: function () {
                    me.setAllowRemove(me.getSelectionModel().hasSelection());
                }
            }
        });

        this.callParent(arguments);
    },
    addItem: function (e) {
        var me = this;

        var newItem = me.store.model.create();
        newItem.store = me.store;

        try {
            if (me.beforeAdd) {
                me.beforeAdd(newItem);
            }
            me.fireEvent('requestCreate', newItem);
        }
        catch (e) {
            me.fireEvent('createFailed', e);
        }
    },
    removeItem: function (e) {
        var me = this;

        if (me.getSelectionModel().hasSelection()) {
            var selected = me.getSelectionModel().getSelection()[0];
            selected.set('IsDeleted', true);
            selected.save();
            //me.store.remove(selected);
            //me.store.fireEvent('datachanged');
        }
    },
    saveChanges: function (e) {
        var me = this;

        me.setAllowSave(false);

        me.store.sync({
            success: function (batch) {
                me.fireEvent('saveSuccess', batch);
            },
            failure: function (batch) {
                me.fireEvent('saveFailure', batch);
            },
            callback: function (batch) {
                me.store.load({
                    callback: function () {
                        me.setAllowSave(this.needsSync());
                    }
                });
                me.fireEvent('saveComplete', batch);
            },
        });
    },
    cancelChanges: function (e) {
        var me = this;

        me.store.rejectChanges();
        me.setAllowSave(this.needsSync()); //should always be false
    },
    setAllowSave: function (allowSave) {
        if (!this.autoSave) {
            this.down('#saveButton').setDisabled(!allowSave);
            this.down('#cancelButton').setDisabled(!allowSave);
        }
    },
    setAllowRemove: function (allowRemove) {
        if (this.removeEnabled) {
            this.down('#removeButton').setDisabled(!allowRemove);
        }
    },
});