﻿Ext.define('MedwebCloud.view.EditorWindow', {
    extend: 'Ext.window.Window',
    xtype: 'medwebcloud.editorwindow',

    title: "Add/Edit a Record",
    record: null,
    formType: null,

    initComponent: function () {
        if (!this.formType || !this.record) {
            throw "InvalidArgumentException";
        }

        var me = this;
        if (!me.toolbar) {
            me.toolbar = {
                items: [new Ext.button.Button({
                    text: window.MedwebCloud.getMsg('medweb.view.shared.editors.buttons.save'),
                    handler: function () {
                        //Note that we can't just use editor.getValues() here because
                        // it doesn't return unchecked checkboxes (as of ExtJS 4.2.2)
                        var editor = this.up('.window').down('#editorForm');
                        var values = {};
                        var items = (!!editor.getFormItems) ? editor.getFormItems() : editor.items;

                        items.each(function (f) {
                            if (f.getName && f.getValue) { values[f.getName()] = f.getValue(); }
                        });
                        me.fireEvent('editComplete', values);
                        this.up('.window').close();
                    }
                }), new Ext.button.Button({
                    text: window.MedwebCloud.getMsg('medweb.view.shared.editors.buttons.cancel'),
                    handler: function () {
                        this.up('.window').close();
                    }
                })]
            };
        }

        me.toolbar.dock = 'bottom';
        me.toolbar.layout = { pack: 'center' };
        me.toolbar.ui = 'footer';
        me.toolbar = new Ext.toolbar.Toolbar(me.toolbar);

        Ext.apply(me, {
            title: me.title,
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            autoShow: me.autoShow || false,
            modal: true,
            dockedItems: [me.toolbar],
            shrinkWrapDock: true,
            closable: false,
            record: me.record,
            items: {
                xtype: me.formType,
                itemId: 'editorForm',
                model: me.record.store.model,
                record: me.record,
                customArgs: me.customArgs,
                bodyStyle: 'padding: 10px;',
                saveButton: me.toolbar.items.items[0],
            },
        });

        this.callParent(arguments);

        this.load(me.record);
    },
    load: function (record) {
        if (record) {
            this.down('#editorForm').getForm().loadRecord(record);
        }
    },
    forceComplete: function (success) {
        this.fireEvent('forceComplete', success);
    }
});