﻿Ext.define('MedwebCloud.view.GridWindowEditor', {
    extend: 'Ext.panel.Panel',
    xtype: 'medwebcloud.gridwindoweditor',

    formType: null,
    addForm: null,
    editForm: null,

    addArgs: null,
    editArgs: null,
    gridArgs: null,

    storeType: null,
    itemName: 'Record',
    beforeAdd: null,
    customArgs: null,

    addEnabled: true,
    editEnabled: true,
    removeEnabled: true,
    autoSave: false,

    columns: null,
    expander: null,
    filters: null,

    filterer: null,

    initComponent: function () {
        var me = this;

        if (!this.storeType) {
            throw new "InvalidArgumentException";
        }

        if (me.addEnabled && (me.addForm == null && me.formType == null) ||
            me.editEnabled && (me.editForm == null && me.formType == null)) {
            throw new "InvalidArgumentException";
        }

        var gridRegion = {
            xtype: 'medwebcloud.editorgrid',
            addEnabled: me.addEnabled,
            removeEnabled: me.removeEnabled,
            autoSave: me.autoSave,
            storeType: me.storeType,
            itemName: me.itemName,
            filters: me.filters,
            filterer: me.filterer,
            expander: me.expander,
            region: 'center',
            layout: 'fit',
            itemId: 'editorGrid',
            border: false,
            listeners: {}, //Listeners defined here DO NOT FIRE, see below
            beforeAdd: me.beforeAdd,
            customArgs: me.gridArgs,
        };

        if (me.columns) {
            gridRegion.columns = me.columns;
        }

        me.msgBoxProps = {
            buttons: Ext.MessageBox.OK,
            width: 300,
            progress: false,
            draggable: true,
            closable: true
        };

        Ext.apply(this, {
            items: [gridRegion],
        });
        this.callParent(arguments);

        //Listeners defined in the above 'constructor' DO NOT FIRE.
        // Seem to require the listeners to be defined here, as these work.

        me.grid = me.down('#editorGrid');

        me.grid.on('requestEdit', function (record) {
            if (me.editEnabled) {
                me.openEditorWindow(record);
            }
        });
        me.grid.on('requestCreate', function (record) {
            if (me.addEnabled) {
                me.openCreateWindow(record);
            }
        });

        me.grid.on('saveSuccess', function () {
            me.reportSaveSuccess();
        });
        me.grid.on('saveFailure', function () {
            me.reportSaveFailure();
        });
    },
    openEditorWindow: function (record) {
        var me = this;

        me.editorWindow = Ext.create('MedwebCloud.view.EditorWindow', {
            formType: me.editForm || me.formType,
            autoShow: true,
            autoSave: me.autoSave,
            title: "Editing " + me.itemName,
            record: record,
            customArgs: me.editArgs,
            listeners: {
                editComplete: function (values) {
                    me.updateRowValues(record, values);
                }
            }
        });
    },
    updateRowValues: function (record, values) {
        record.set(values);
        record.store.fireEvent('datachanged');
    },
    openCreateWindow: function (record) {
        var me = this;

        me.editorWindow = Ext.create('MedwebCloud.view.EditorWindow', {
            formType: me.addForm || me.formType,
            autoShow: true,
            autoSave: me.autoSave,
            title: "Creating " + me.itemName,
            record: record,
            customArgs: me.addArgs,
            listeners: {
                editComplete: function (values) {
                    me.insertRow(record, values);
                }
            }
        });
    },
    insertRow: function (record, values) {
        record.set(values);
        record.store.insert(0, record);
        //record.store.fireEvent('datachanged');
    },
    reportSaveSuccess: function () {
        var me = this;

        /*Ext.MessageBox.show(Ext.merge(me.msgBoxProps, {
            title: "Save Success",
            msg: "Changes were saved successfully.",
            icon: Ext.MessageBox.INFO,
        }));*/
    },
    reportSaveFailure: function () {
        var me = this;

        Ext.MessageBox.show(Ext.merge(me.msgBoxProps, {
            title: "Save Failed",
            msg: "An error occurred when trying to save your changes.",
            icon: Ext.MessageBox.ERROR,
        }));
    },
});