﻿Ext.define('MedwebCloud.view.ItemForm', {
    xtype: 'medwebcloud.itemform',
    extend: 'Ext.form.Panel',
    bodyPadding: 5,
    initComponent: function () {
        Ext.apply(this, {
            tbar: {
                items: [{
                    iconCls: 'saveIcon',
                    text: 'Save',
                    handler: Ext.bind(this.save, this)
                }, {
                    itemId: 'status',
                    xtype: 'tbtext'
                }, '->', {
                    iconCls: 'deleteIcon',
                    text: 'Delete',
                    handler: Ext.bind(this._delete, this)
                }]
            }
        });
        this.callParent(arguments);

        this.load();
    },
    _delete: function () {
        var me = this;
        Ext.Msg.confirm('Are you sure?', 'Are you sure you want to delete this item?', function (btn) {
            if (btn == 'yes') {
                me.setStatus('Deleting...', false);

                var record = me.getRecord();
                record.set('IsDeleted', true);

                record.save({
                    success: function (record) {
                        me.fireEvent('delete', record);
                    }
                });
            }
        });
    },
    save: function () {
        if (this.isValid()) {
            this.setStatus('Saving...', false);
            var me = this;

            var record = this.getRecord();
            record.set(this.getValues());

            if (this.beforeSave) {
                this.beforeSave(record);
            }

            record.save({
                success: function (record) {
                    me.setTitle(record.get(me.nameField));
                    me.recordId = record.getId();

                    me.loadRecord(record);
                    me.setStatus('', false);

                    me.fireEvent('save', record);

                    me.setStatus('<em>Saved.</em>', true);
                }
            });
        } else {
            this.setStatus('<em>You have some invalid entries; please review and try again.</em>', true);
        }
    },
    load: function () {
        if (!this.model) {
            console.error('The type ' + this.xtype + ' requires a model to be specified in the config.');
            return;
        }

        var me = this,
            theModelClass = Ext.ClassManager.get(this.model);

        if (this.recordId) {
            theModelClass.load(this.recordId, {
                // TODO: add failure to all model load calls
                success: function (record) {
                    me.loadRecord(record);
                    me.setTitle(record.get(me.nameField));

                    me.fireEvent('load', record);
                }
            });
        }
    },
    setStatus: function (status, timed) {
        var statusField = this.down('toolbar[dock=top] #status');
        statusField.setText(status);
        if (timed) {
            window.clearTimeout(this._statusTimeout);
            this._statusTimeout = window.setTimeout(function () {
                statusField.setText('');
            }, 3000);
        }
    }
});