﻿Ext.define('MedwebCloud.view.ItemTab', {
    extend: 'Ext.Panel',
    xtype: 'medwebcloud.itemtab',

    initComponent: function () {
        var me = this;
        var panelConfig = Ext.applyIf(this.panelConfig, {
            title: '',
            idField: 'Id',
            nameField: 'Name',
            descriptionField: 'Description',
        });
        var gridConfig = this.gridConfig || {};

        var tbarItems = gridConfig.tbarItems || [{
            xtype: 'button',
            text: 'Add ' + panelConfig.title,
            iconCls: 'addIcon',
            handler: function () {
                me.addPanel(0, 'New ' + panelConfig.title, 'New ' + panelConfig.title);
            }
        }];

        if (this.hideTbar) {
            tbarItems = null;
        } else {
            tbarItems.unshift();
        }

        var centerRegion = {
            xtype: 'tabpanel',
            ref: 'centerContent',
            region: 'center',
            cls: 'border border-left',
            enableTabScroll: true,
            border: false
        };
        var westRegion = {
            xtype: this.gridConfig.xtype,
            title: this.gridConfig.title,
            itemId: this.gridConfig.itemId,
            region: 'west',
            width: 300,
            tbar: tbarItems,
            cls: 'border border-right',
            border: false,
            split: true,
            collapsible: true,
            floatable: false,
            width: this.gridConfig.width || 300,
            ref: 'grid',
            stateId: gridConfig.stateId,
            listeners: {
                itemclick: function (grid, record, item, index) {
                    me.addPanel(record.get(panelConfig.idField), record.get(panelConfig.nameField), record.get(panelConfig.descriptionField), record);
                }
            }
        }
        function doReload() {
            me.down('gridpanel').getStore().reload();
        }
        Ext.apply(this, {
            gridConfig: gridConfig,
            panelConfig: panelConfig,
            layout: 'border',
            border: false,
            items: [westRegion, centerRegion],

            listeners: {
                added: doReload,
                activate: doReload
            }
        });

        me.callParent(arguments);

        // make sure the panel stays centered
        /*Ext.on('resize', function () {
            panel.center();
            if (me.isVisible()) {
                panel.center();
            }
        });*/
    },

    removePanel: function (recordId) {
        var me = this;
        var tabPanel = me.down('tabpanel');
        var found = me.findTab(recordId);
        if (found) {
            tabPanel.remove(found);
        }
    },

    addPanel: function (recordId, name, description, record) {
        var me = this;
        var grid = me.down('gridpanel');
        var tabPanel = me.down('tabpanel');
        var tab = me.findTab(recordId);

        if (tab) {
            tabPanel.setActiveTab(tab);
        } else {
            var args = {
                panelConfig: me.panelConfig,
                recordId: recordId,
                record: record
            };
            me.fireEvent('beforeadd', args);

            if (args.cancel) {
                return;
            }

            var panelConf = Ext.apply({
                xtype: args.panelConfig.xtype,
                border: false,
                autoScroll: true,
                recordId: recordId,
                title: name,
                model: this.model,
                closable: true,
                tabConfig: {
                    tooltip: description
                },
                formDefaults: {
                    width: 500
                },

                listeners: Ext.apply({
                    load: function (record) {
                        tab.setTitle(record.get([args.panelConfig.nameField]));
                    },
                    save: function (record) {
                        grid.getStore().reload();
                        tab.setTitle(record.get([args.panelConfig.nameField]));
                    },
                    duplicate: function (record) {
                        me.addPanel(record.getId()),
                        record.get([args.panelConfig.nameField]),
                        record.get([args.panelConfig.descriptionField]);
                        grid.getStore().reload();
                    },
                    'delete': function (record) {
                        me.removePanel(record.getId());
                        grid.getStore().reload();
                    }
                }, args.panelConfig.listeners || {})
            }, {});

            function loadTab(tabPanel, panelConf, record) {
                //                var theTab = Ext.create(panelConf)
                var tab = tabPanel.add(tabPanel.items.length, panelConf);
                var form = tab;

                if (!form.loadRecord) {
                    console.warn('The editor ' + form.xtype + ' has no loadRecord function defined, so the loaded record cannot be passed to the editor', form);
                } else {
                    form.loadRecord(record);
                }
                //tab.fireEvent('recordloaded', tab, record);

                tabPanel.setActiveTab(tab);
                if (record.getId() == 0) {
                    var deleteButton = tabPanel.getActiveTab().down('#deleteButton');
                    if (deleteButton) {
                        deleteButton.setVisible(false);
                    }
                }
                me.fireEvent('add', args);
                return tab;
            }

            if (!record) {
                record = Ext.create(me.model);
                record.set(me.panelConfig.idField, 0);

                if (me.autoSaveNew) {
                    record.save({
                        callback: function (rec) {
                            grid.getStore().reload();
                            panelConf.recordId = record.get('id');
                            tab = loadTab(tabPanel, panelConf, record);
                        }
                    });
                }
                else {
                    tab = loadTab(tabPanel, panelConf, record);
                }
            }
            else {
                tab = loadTab(tabPanel, panelConf, record);
            }
        }
    },

    // return tab with given recordId
    findTab: function (recordId) {
        if (recordId == undefined) {
            return undefined;
        }
        var me = this;
        var tabPanel = me.down('tabpanel');
        var subtabs = tabPanel.items.items;
        for (var i = 0; i < subtabs.length; i++) {
            if (subtabs[i].recordId == recordId) {
                return subtabs[i];
            }
        }
        return undefined;
    }
});