﻿Ext.define('MedwebCloud.view.PanelEditor', {
    extend: 'MedwebCloud.view.BaseEditorWrapper',
    xtype: 'medwebcloud.paneleditor',

    gridTitle: 'Records',
    columns: null,

    initComponent: function () {
        var me = this;
        
        me.store = Ext.create(me.storeType, Ext.apply({
            autoLoad: true,
            remoteSort: true,
            remoteFilter: true,
            filters: me.filters,
            expand: me.expands,
            sorters: me.sorters,
        }, me.storeOptions || {}));

        var gridProps = {
            store: me.store,
            region: 'west',
            collapsible: true,
            split: true,
            title: me.gridTitle,
            titleAlign: 'center',
            layout: 'fit',
            width: me.gridWidth || 300,
            border: true,
            overflowY: 'auto',
            viewConfig: {
                getRowClass: function (rec, rowIdx, params, store) {
                    if (!!rec && !!rec.get) {
                        return rec.get('IsDeleted') ? 'deleted-record' : '';
                    }
                }
            },
            plugins: [Ext.create('Ext.ux.grid.plugin.HeaderFilters')],
            listeners: {
                itemclick: function (view, record, item, index, e, eOpts) {
                    if (!record.get("IsDeleted")) {
                        me.editItem(record);
                    }
                },
                selectionchange: function (selModel) {
                    var recs = selModel.getSelection();
                    if (recs.length == 1) {
                        var recIsDeleted = recs[0].get('IsDeleted');
                        me.setAllowRemove(!recIsDeleted);
                        me.setAllowRestore(recIsDeleted);
                    } else {
                        me.setAllowRemove(false);
                        me.setAllowRestore(false);
                    }
                }
            },
        };

        if (me.columns)
        {
            gridProps.columns = me.columns;
        }
        else
        {
            gridProps.columns = [{
                dataIndex: 'Id',
                text: 'Id',
                flex: 1,
            }];
        }

        var panelProps = {
            store: me.store,
            region: 'center',
            customArgs: me.customArgs,
        };

        gridProps.tbar = me.initToolbar();

        me.grid = Ext.create('Ext.grid.Panel', gridProps);
        me.panel = Ext.create('Ext.tab.Panel', panelProps);
        
        Ext.apply(this, {
            layout: 'border',
            split: true,
            items: [me.grid, me.panel],
        });

        this.callParent(arguments);
        
        me.idToTabMap = {};
    },
    innerAddCheck: function () {
        var me = this;

        if (me.idToTabMap[0]) {
            Ext.MessageBox.alert(me.getMsg('titles.tempexists'), me.getMsg('messages.tempexists', me.itemName));
            return false;
        }
    },
    addItem: function () {
        this.callParent(arguments);
    },
    startCreate: function (record) {
        var me = this;

        var createTabProps = {
            xtype: me.addForm || me.formType,
            title: (typeof me.addTitle === 'function') ? me.addTitle(record) : me.addTitle,
            saveHandler: function (form, record) {
                var values = me.getFormValues(form);
                if (values) {
                    record.set(values);
                    record.store.insert(0, record);
                }
            },
            onSaveSuccess: function (form, record) {
                form.setTitle(me.titleFunc(record));
                var idx = form.up().items.indexOf(form);
                form.close();
                delete me.idToTabMap[0];
                me.startEdit(record, idx);
            },
        };

        me.activatePanel(record, createTabProps);
    },
    editItem: function (record) {
        this.callParent([record]);
    },
    startEdit: function (record, pos) {
        var me = this;        

        var editTabProps = {
            xtype: me.editForm || me.formType,
            title: (typeof me.editTitle === 'function') ? me.editTitle(record) : me.editTitle,
            pos: pos,
            saveHandler: function (form, record) {
                var values = me.getFormValues(form);
                if (values) {
                    record.set(values);
                }
            },
            onSaveSuccess: function (form, record) {
                form.setTitle(me.titleFunc(record));
            },
        };

        me.activatePanel(record, editTabProps);
    },
    removeItem: function () {
        this.callParent(arguments);
    },
    startRemove: function () {
        var me = this;

        var selModel = me.grid.getSelectionModel();
        if (selModel.hasSelection()) {
            me.openDeleteWindow(selModel.getSelection());
        }
    },
    openDeleteWindow: function (records) {
        var me = this;

        Ext.MessageBox.show({
            title: me.getMsg('titles.confirmdelete'),
            msg: me.getMsg('messages.confirmdelete'),
            modal: true,
            buttons: Ext.Msg.YESNO,
            buttonText: {
                yes: me.getMsg('buttons.deleteyes'),
                no: me.getMsg('buttons.deleteno'),
            },
            icon: Ext.Msg.QUESTION,
            fn: function (btnId) {
                if (btnId === 'yes') {
                    for (var i = 0; i < records.length; i++) {
                        records[i].set('IsDeleted', true);
                        if (me.showingDeleted) {
                            me.store.save();
                        } else {
                            me.store.remove(records[i]);
                        }
                    }
                    me.onOperationComplete(function () {
                        var gridSel = me.grid.getSelectionModel();
                        if (gridSel) {
                            gridSel.clearSelections();
                        }
                        for (var i = 0; i < records.length; i++) {
                            me.idToTabMap[records[i].get("Id")].close();
                        }
                    });
                }
            }
        });

    },

    startRestore: function () {
        var me = this;

        var selModel = me.grid.getSelectionModel();
        if (selModel.hasSelection()) {
            me.openRestoreWindow(selModel.getSelection());
        }
    },
    openRestoreWindow: function (records) {
        var me = this;

        Ext.MessageBox.show({
            title: me.getMsg('titles.confirmrestore'),
            msg: me.getMsg('messages.confirmrestore'),
            modal: true,
            buttons: Ext.Msg.YESNO,
            buttonText: {
                yes: me.getMsg('buttons.restoreyes'),
                no: me.getMsg('buttons.restoreno'),
            },
            icon: Ext.Msg.QUESTION,
            fn: function (btnId) {
                if (btnId === 'yes') {
                    for (var i = 0; i < records.length; i++) {
                        records[i].set('IsDeleted', false);
                        me.store.save();
                    }
                    me.onOperationComplete(function () {
                        var gridSel = me.grid.getSelectionModel();
                        if (gridSel) {
                            gridSel.clearSelections();
                        }
                    });
                }
            }
        });
    },


    activatePanel: function (record, tabProps) {
        var me = this;

        if (me.idToTabMap[record.get("Id")]) {
            var tab = me.idToTabMap[record.get("Id")];
            me.panel.setActiveTab(tab);
            return tab;
        }
        else {

            var tabConfig = Ext.apply({
                border: false,
                autoScroll: true,
                closable: true,
                record: record,
                bodyStyle: 'padding: 10px;',
                listeners: {
                    close: function (tab) {
                        delete me.idToTabMap[tab.record.get("Id")];
                    },
                },
                tbar: {
                    items: [{
                        text: me.getMsg('buttons.savechanges'),
                        iconCls: 'saveIcon',
                        disabled: true,
                        formBind: true,
                        handler: function (e) {
                            var form = this.up('form');
                            tabProps.saveHandler(form, record);
                            me.onOperationComplete(function () { tabProps.onSaveSuccess(form, record); });
                        }
                    }, {
                        xtype: 'tbfill'
                    }, {
                        text: me.getMsg('buttons.undochanges'),
                        iconCls: 'endIcon',
                        handler: function (e) {
                            this.up('form').getForm().loadRecord(record);
                        }
                    }]
                },
            }, tabProps);

            var newTab = me.panel.add(tabConfig.pos || me.panel.items.length, tabConfig);

            if (record) {
                newTab.getForm().loadRecord(record);
            }

            me.idToTabMap[record.get("Id")] = newTab;

            me.panel.setActiveTab(newTab);

            return newTab;
        }
    },
    onOperationComplete: function (onSuccess) {
        this.callParent(arguments);
    },
    getFormValues: function (form) {
        return this.callParent(arguments);
    },
    setAllowRemove: function (allowRemove) {
        if (this.removeEnabled) {
            this.down('#removeButton').setDisabled(!allowRemove);
        }
    },
    setAllowRestore: function (allowRestore) {
        if (this.restoreEnabled) {
            this.down('#restoreButton').setDisabled(!allowRestore);
        }
    },
    initToolbar: function () {
        return this.callParent(arguments);
    },
});