﻿Ext.define('MedwebCloud.view.WizardForm', {
    xtype: 'medwebcloud.wizardform',
    extend: 'Ext.Panel',

    pages: [],
    finalizeData: function (dataObj) { return dataObj; },
    defaultData: {},
    backOutFunction: null,
    finalize: function (record) { return {}; },

    initComponent: function () {
        var me = this;

        me.originalData = me.deepCopyDataObj(me.defaultData);
        me.allDataObj = me.defaultData;
        console.log("Wizard data: ", me.allDataObj);

        var bothItems = me.pagesToItems(me.pages);

        Ext.apply(this, {
            layout: 'border',
            items: [{
                region: 'west',
                xtype: 'panel',
                collapsible: false,
                border: false,
                split: true,
                width: 250,
                padding: '10 0 10 10',
                cls: 'wiz-back',

                items: [{
                    itemId: 'wizard-tree',
                    header: {
                        title: 'Progress',
                        titleAlign: 'center',
                        cls: 'wiz-inner-header',
                    },
                    border: false,

                    xtype: 'treepanel',
                    rootVisible: false,
                    store: Ext.create('Ext.data.TreeStore', {
                        root: {
                            expanded: true,
                            children: bothItems.tree,
                        }
                    }),
                    listeners: {
                        //select: me.genOnTreeSelection(),
                        afterrender: function (tree, opts) {
                            tree.getSelectionModel().select(tree.getStore().getById(bothItems.tree[0].id));
                            tree.addManagedListener(tree, 'select', me.genOnTreeSelection());
                        },
                    },
                    cls: 'wizard-tree x-tree-noicon',
                }],
            }, {
                region: 'center',

                xtype: 'panel',
                itemId: 'wizard-cards',
                border: false,
                cls: 'wizard-form-container',

                layout: 'card',

                padding: '10 10 10 0',

                items: bothItems.forms,
            }],
        });

        this.callParent(arguments);

        me.navTree = me.down('#wizard-tree');
        me.formContainer = me.down('#wizard-cards');

        me.formContainer.items.each(function (itm) {
            itm.on('valuesChanged', function (newValues) {
                for (var prop in newValues) {
                    if (me.allDataObj.hasOwnProperty(prop)) {
                        me.allDataObj[prop] = newValues[prop];
                    }
                }

                me.raiseDataChanged(newValues);
            });
        });
    },
    pagesToItems: function (pages) {
        var me = this;

        var bothItems = { tree: [], forms: [] };

        for (var i = 0; i < pages.length; i++) {
            var page = pages[i];
            var id = page.title.replace(/\s/g, '_').replace(/\W/g, '').toLowerCase();

            bothItems.tree.push({ 'id': id, 'text': page.title, 'leaf': true });

            var buttons = [
            '->', {
                text: 'Done',
                handler: me.genEndWizard(),
            }];

            if (bothItems.forms.length == 0 && me.backOutFunction && typeof (me.backOutFunction) === 'function') {
                buttons.unshift({
                    text: 'Restart',
                    itemId: 'prevBtn',
                    scale: 'large',
                    cls: 'wiz-button-font',
                    iconCls: 'wiz-nav-startarrow contain-image',
                    handler: me.backOutFunction,
                });
            }
            else if (bothItems.forms.length > 0) {
                buttons.unshift({
                    text: 'Previous',
                    itemId: 'prevBtn',
                    scale: 'large',
                    cls: 'wiz-button-font',
                    iconCls: 'wiz-nav-backarrow contain-image',
                    handler: me.genPrevPageFn(),
                });

                bothItems.forms[bothItems.forms.length - 1].buttons.splice(-1, 1, {
                    text: 'Next',
                    itemId: 'nextBtn',
                    scale: 'large',
                    cls: 'wiz-button-font',
                    iconCls: 'wiz-nav-nextarrow contain-image',
                    iconAlign: 'right',
                    handler: me.genNextPageFn(),
                });
            }

            bothItems.forms.push({
                'xtype': 'medwebcloud.lookingglasssetupwizard.' + page.form.toLowerCase(),
                'header': false,
                /*{
                   'title': page.title,
                   'titleAlign': 'center',
                   'cls': 'wiz-inner-header',
               },*/
                'itemId': id,
                'buttons': buttons,
                'border': false,
                'context': me.context,
                'cls': 'wizard-inner-form',
                getRecord: function () {
                    return me.allDataObj;
                },
                listeners: {
                    activate: function () {
                        var items = (this.getFormItems && typeof (this.getFormItems) === 'function') ? this.getFormItems() : this.items;

                        if (items) {
                            items.each(function (f) {
                                if (me.allDataObj[f.getItemId()] && f.setValue && f.setValue === 'function') {
                                    f.setValue(me.allDataObj[f.getItemId()]);
                                }
                            });
                        }
                    }
                }
            });
        }
        var fbs = [];

        if (bothItems.forms.length) {
            fbs.push({
                text: 'Previous',
                itemId: 'prevBtn',
                scale: 'large',
                cls: 'wiz-button-font',
                iconCls: 'wiz-nav-backarrow contain-image',
                handler: me.genPrevPageFn(),
            });
        }
        else {
            fbs.push({
                text: 'Restart',
                itemId: 'prevBtn',
                scale: 'large',
                cls: 'wiz-button-font',
                iconCls: 'wiz-nav-startarrow contain-image',
                handler: me.backOutFunction,
            });
        }

        fbs.push('->');

        if (me.finalButtons) {
            fbs = fbs.concat(me.finalButtons);
        }

        bothItems.forms.push({
            xtype: 'medwebcloud.lookingglasssetupwizard.wizcomplete',
            id: 'final',
            originalData: me.finalize(me.deepCopyDataObj(me.originalData)),
            newData: me.finalize(me.deepCopyDataObj(me.allDataObj)),
            getChangedData: function () {
                return me.finalize(me.deepCopyDataObj(me.allDataObj));
            },
            'context': me.context,
            'buttons': fbs,
            'border': false,
            'cls': 'wizard-inner-form',
            'header': false,
        });

        bothItems.tree.push({ 'id': 'final', 'text': 'Configuration Complete', 'leaf': true });

        return bothItems;
    },
    raiseDataChanged: function (newData) {
        var me = this;

        me.fireEvent('dataChanged', newData);
    },
    genPrevPageFn: function () {
        var me = this;

        return function () {
            var btn = this;

            var form = btn.up('form');
            var id = form.getItemId();

            console.log('Back from step: ' + id);

            me.navTree.selectPath(me.navTree.getSelectionModel().getSelection()[0].previousSibling.getPath());
        };
    },
    genNextPageFn: function () {
        var me = this;

        return function () {
            var btn = this;

            var form = btn.up('form');
            var id = form.getItemId();

            if (form.getForm().isValid()) {
                //var formData = form.getValues();

                //if (formData) {
                //    me.recursiveUpdate(formData, me.allDataObj);
                //}

                //console.log('Completing step: ' + id, formData);

                me.navTree.selectPath(me.navTree.getSelectionModel().getSelection()[0].nextSibling.getPath());
            }
        };
    },
    genSkipPageFn: function () {
        var me = this;

        return function () {
            var btn = this;

            var form = btn.up('form');
            var id = form.getItemId();

            console.log('Skipping step: ' + id);

            me.navTree.selectPath(me.navTree.getSelectionModel().getSelection()[0].nextSibling.getPath());
        };
    },
    recursiveUpdate: function (inData, outData) {
        var me = this;
        console.log(inData, outData);
        for (var key in inData) {
            if (inData.hasOwnProperty(key)) {
                var dataObj = inData[key];

                if (dataObj.hasOwnProperty('wizprop_overwrite') && dataObj.hasOwnProperty('wizprop_data')) {
                    if (dataObj.wizprop_overwrite) {
                        outData[key] = dataObj['wizprop_data']
                    }
                    else {
                        me.recursiveUpdate(dataObj['wizprop_data'], outData[key]);
                    }
                }
                else {
                    outData[key] = dataObj;
                }
            }
        }

        //Ext.Object.merge(me.allDataObj, formData);
    },
    genEndWizard: function () {
        var me = this;

        return function () {
            var finalCard = me.down('#final');
            finalCard.originalData = me.finalize(me.originalData);
            finalCard.newData = me.finalize(me.deepCopyDataObj(me.allDataObj));
            finalCard.initComponent();
            me.down('#wizard-cards').getLayout().setActiveItem('final');
        };
    },
    genOnTreeSelection: function () {
        var me = this;

        return function (tree, record, index, opts) {
            var id = record.id;

            var wizCards = me.down('#wizard-cards');
            wizCards.getLayout().getActiveItem().saveChanges();

            wizCards.getLayout().setActiveItem(id);
        };
    },
    getData: function (key) {
        if (this.allDataObj[key]) {
            return this.deepCopyDataObj(this.allDataObj[key]);
        }

        return null;
    },
    extractData: function () {
        return this.finalizeData(this.deepCopyDataObj(this.allDataObj));
    },
    deepCopyDataObj: function (obj) {
        //Note: This method of deep copying removes functions.
        return JSON.parse(JSON.stringify(obj));
    }
});