﻿Ext.define('MedwebCloud.view.UserEditor', {
    extend: 'Ext.form.Panel',
    xtype: 'MedwebCloud.usereditor',
    requires: [],
    recordId: 0,
    hidePasswordFields: false,
    initComponent: function () {
        var me = this;

        var lblDisplay = MedwebCloud.getMsg('medweb.user.displayname.label');
        var lblPassConf = MedwebCloud.getMsg('medweb.user.password.confirm.label');
        var lblUserName = MedwebCloud.getMsg('medweb.user.email.label');
        var lblPass = MedwebCloud.getMsg('medweb.user.password.label');
        var lblTrack = MedwebCloud.getMsg('medweb.user.tracking.label');
        var lblTZ = MedwebCloud.getMsg('medweb.user.preferredtimezone.label');
        var lblLang = MedwebCloud.getMsg('medweb.user.language.label');
        var lblPhone = MedwebCloud.getMsg('medweb.user.phone.label');
        var lblDoctor = MedwebCloud.getMsg('medweb.user.isdoctor.label');
        var lblDepartment = MedwebCloud.getMsg('medweb.user.department.label');

        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();

        var zones = Ext.create('Ext.data.Store', {
            fields: ['Id', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: ''
                }
            },
            data: MedwebCloud.UIConfig.Timezones
        });

        var langs = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: MedwebCloud.UIConfig.SupportedCultures
        });

        var items = []

        Ext.apply(me, {
            defaults: {
                anchor: '100%',
                labelWidth: 130
            },
            items: [{
                xtype: 'textfield',
                fieldLabel: lblUserName,
                allowBlank: false,
                disabled: me.recordId > 0,
                name: 'Username'
            }, {
                xtype: 'textfield',
                fieldLabel: lblDisplay,
                allowBlank: false,
                name: 'DisplayName'
            }, {
                xtype: 'textfield',
                fieldLabel: lblPass,
                inputType: 'password',
                itemId: 'Password',
                name: 'Password'
            }, {
                xtype: 'textfield',
                fieldLabel: lblPassConf,
                inputType: 'password',
                vtype: 'password',
                initialPassField: 'Password',
                name: 'ConfirmPassword'
            }, {
                xtype: 'textfield',
                fieldLabel: lblPhone,
                itemId: 'Phone',
                name: 'Phone'
            }, {
                xtype: 'checkbox',
                fieldLabel: lblDoctor,
                name: 'IsDoctor',
                readOnly: !(MedwebCloud.userProfile.IsAdmin || MedwebCloud.userProfile.IsSuperAdmin)
            }, {
                xtype: 'combo',
                fieldLabel: lblDepartment,
                name: 'DepartmentId',
                queryMode: 'remote',
                store: me.departments,
                displayField: 'Name',
                valueField: 'Id',
                autoSelect: false,
                forceSelection: false,
                allowBlank: true,
                readOnly: !(MedwebCloud.userProfile.IsAdmin || MedwebCloud.userProfile.IsSuperAdmin)
            }, {
                xtype: 'checkbox',
                fieldLabel: lblTrack,
                name: 'TrackLocation'
            }, {
                xtype: 'combo',
                fieldLabel: lblLang,
                name: 'Language',
                queryMode: 'local',
                store: langs,
                displayField: 'DisplayName',
                valueField: 'Name',
                autoSelect: true,
                forceSelection: true,
                //TODO: if this is going to be used *ANYWHERE* else, move it into a "help" option on all fields
                afterLabelTextTpl: ' <img src="/images/help.png" class="info_image" data-qtip="Requires logging out and back in again"></img>'
            }, {
                xtype: 'combo',
                fieldLabel: lblTZ,
                name: 'PreferredTimezone',
                queryMode: 'local',
                store: zones,
                displayField: 'DisplayName',
                valueField: 'Id',
                autoSelect: true,
                forceSelection: true
            }]
        });

        me.callParent(arguments);
        if (me.recordId) {
            me.setLoading(true);
            MedwebCloud.model.User.load(me.recordId, {
                success: function (model) {
                    console.log(model);
                    me.setLoading(false);
                    me.loadRecord(model);
                }
            });
        }
    }
});