﻿Ext.define('MedwebCloud.view.UserForm2', {
    xtype: 'medwebcloud.userform2',
    extend: 'Ext.form.Panel',

    hidePasswordFields: false,
    initComponent: function () {
        var me = this;

        var lblDisplay = MedwebCloud.getMsg('medweb.user.displayname.label');
        var lblPassConf = MedwebCloud.getMsg('medweb.user.password.confirm.label');
        var lblUserName = MedwebCloud.getMsg('medweb.user.email.label');
        var lblPass = MedwebCloud.getMsg('medweb.user.password.label');
        var lblTrack = MedwebCloud.getMsg('medweb.user.tracking.label');
        var lblTZ = MedwebCloud.getMsg('medweb.user.preferredtimezone.label');
        var lblLang = MedwebCloud.getMsg('medweb.user.language.label');
        var lblDoctor = MedwebCloud.getMsg('medweb.user.isdoctor.label');
        var lblDepartment = MedwebCloud.getMsg('medweb.user.department.label');

        me.departments = Ext.create('MedwebCloud.store.Departments');
        me.departments.load();

        var zones = Ext.create('Ext.data.Store', {
            fields: ['Id', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: ''
                }
            },
            data: MedwebCloud.UIConfig.Timezones
        });

        var langs = Ext.create('Ext.data.Store', {
            fields: ['Name', 'DisplayName'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json'
                }
            },
            data: MedwebCloud.UIConfig.SupportedCultures
        });
        var items = [{
            xtype: 'textfield',
            fieldLabel: lblDisplay,
            allowBlank: false,
            name: 'Autofill_Prevention',
            hidden: true
        }, {
            xtype: 'textfield',
            fieldLabel: lblPass,
            inputType: 'password',
            name: 'Autofill_Prevention_Password',
            hidden: true
        }, {
            xtype: 'textfield',
            fieldLabel: lblUserName,
            allowBlank: false,
            disabled: !me.record.phantom,
            name: 'Username'
        }, {
            xtype: 'textfield',
            fieldLabel: lblDisplay,
            allowBlank: false,
            name: 'DisplayName'
        }, {
            xtype: 'textfield',
            fieldLabel: lblPass,
            inputType: 'password',
            itemId: 'Password',
            name: 'Password'
        }, {
            xtype: 'textfield',
            fieldLabel: lblPassConf,
            inputType: 'password',
            vtype: 'password',
            initialPassField: 'Password',
            name: 'ConfirmPassword'
        }, {
            xtype: 'checkbox',
            fieldLabel: lblDoctor,
            name: 'IsDoctor',
            inputValue: true
        }, {
            xtype: 'combo',
            fieldLabel: lblDepartment,
            name: 'DepartmentId',
            queryMode: 'remote',
            store: me.departments,
            displayField: 'Name',
            valueField: 'Id',
            autoSelect: false,
            forceSelection: false,
            allowBlank: true
        }, {
            xtype: 'checkbox',
            inputValue: true,
            fieldLabel: lblTrack,
            name: 'TrackLocation'
        }, {
            xtype: 'combo',
            fieldLabel: lblLang,
            name: 'Language',
            queryMode: 'local',
            store: langs,
            displayField: 'DisplayName',
            valueField: 'Name',
            autoSelect: true,
            forceSelection: true,
            //TODO: if this is going to be used *ANYWHERE* else, move it into a "help" option on all fields
            afterLabelTextTpl: ' <img src="../../../images/help.png" class="info_image" data-qtip="Requires logging out and back in again"></img>'
        }, {
            xtype: 'combo',
            fieldLabel: lblTZ,
            name: 'PreferredTimezone',
            queryMode: 'local',
            store: zones,
            displayField: 'DisplayName',
            valueField: 'Id',
            autoSelect: true,
            forceSelection: true
        }];

        if (MedwebCloud.userProfile.IsAdmin || MedwebCloud.userProfile.IsSuperAdmin) {
            items.push({
                xtype: 'checkbox',
                inputValue: true,
                fieldLabel: 'Login Disabled',
                name: 'Temporary'
            });
            items.push({
                xtype: 'checkbox',
                inputValue: true,
                fieldLabel: 'Is Admin',
                name: 'IsAdmin'
            });
            items.push({
                xtype: 'checkbox',
                inputValue: true,
                fieldLabel: 'Is Glasses Admin',
                name: 'IsGlassesAdmin'
            });
        }
        if (MedwebCloud.userProfile.IsSuperAdmin) {
            items.push({
                xtype: 'checkbox',
                inputValue: true,
                fieldLabel: 'Is Super Admin',
                name: 'IsSuperAdmin'
            })
        }

        var deviceList = {
            title: 'Owned Devices',
            itemName: 'Device',
            xtype: 'medwebcloud.windoweditor',
            formType: 'medwebcloud.deviceform2',
            storeType: 'MedwebCloud.store.Devices',
            addEnabled: false,
            width: 430,
            height: 360,

            filters: [{
                property: 'CompanyId',
                operator: '=',
                value: me.record.get("CompanyId")
            }, {
                property: 'UserId',
                operator: '=',
                value: me.record.get("Id")
            }],

            columns: [{
                dataIndex: 'Name',
                text: 'Name',
                flex: 2,
            }, {
                dataIndex: 'DeviceDescription',
                text: 'Description',
                flex: 2,
            }],

            beforeAdd: function (item) {
                item.set("CompanyId", me.record.get("CompanyId"));
                item.set("UserId", me.record.get("Id"));
            },
        };

        //var devs = ;

        Ext.apply(me, {
            width: 750,
            height: 380,
            defaults: {
                anchor: '100%',
                labelWidth: 130,
                border: false,
            },
            layout: 'column',
            border: false,
            items: [{
                title: '',
                columnWidth: 0.4,
                items: items
            }, {
                title: '',
                columnWidth: 0.6,
                items: [deviceList]
            }]
        });

        me.callParent(arguments);
        me.getFormItems = function () {
            return this.items.get(0).items;
        };

        //Work around autocomplete being unable to be disabled in FF
        setTimeout(me.reset, 1);
    }
});