﻿Ext.define('MedwebCloud.view.VideoPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.videopanel',
    uniqueId: 0,
    waitingText: MedwebCloud.getMsg('medweb.view.videopanel.waiting.message'),
    altWaitingText: MedwebCloud.getMsg('medweb.view.videopanel.waiting.message'),
    pendingText: '',
    basechannel: 'collage',
    subchannel: 'videoconference',
    autoJoin: true,
    permittedPeerIds: false,
    labelRemote: false,
    labelLocal: true,
    isActive: false,
    autoLeave: false,
    client: null,
    options: null,
    streams: null,
    conference: null,
    mutedVideo: false,
    mutedSpeakers: false,
    mutedMic: false,
    userList: null,
    restoreToContainerSelector: null,
    zoomLevel: 1,
    zoom: false,
    textOnButtons: true,
    draw: false,
    trackPulse: false,
    adHoc: false,
    showLocal: true,
    showRemote: true,
    canSnapshot: true,
    initComponent: function () {
        var me = this;
        var record = this.record;
        this._containerId = Ext.id();
        me.streams = [];
        me.userList = [];
        var client = this.client;
        var textOnButtons = this.textOnButtons;

        var btnExpand = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.expand.label') : '';
        var btnCollapse = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.collapse.label') : '';

        var btnMuteVideo = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.mutevideo.label') : '';
        var btnUnmuteVideo = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.unmutevideo.label') : '';
        var btnMuteMic = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.mutemic.label') : '';
        var btnUnmuteMic = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.unmutemic.label') : '';
        var btnMuteSpeaker = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.mutespeaker.label') : '';
        var btnUnmuteSpeaker = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.unmutespeaker.label') : '';

        var btnZoomIn = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.zoomin.label') : '';
        var btnZoomOut = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.zoomout.label') : '';
        var btnZoomReset = textOnButtons ? MedwebCloud.getMsg('medweb.view.videopanel.button.zoomreset.label') : '';

        var zoomButtons = this.zoom ? [{
            text: btnZoomOut,
            iconCls: 'zoom-out',
            handler: function () {
                if (me.conference && me.localMedia) {
                    me.zoomLevel = me.zoomLevel - 0.1;
                    me.zoomTo(me.zoomLevel);
                }
            }
        }, {
            text: btnZoomReset,
            iconCls: 'zoom-reset',
            handler: function () {
                if (me.conference && me.localMedia) {
                    me.zoomLevel = 1;
                    me.zoomTo(me.zoomLevel);
                }
            }
        }, {
            text: btnZoomIn,
            iconCls: 'zoom-in',
            handler: function () {
                if (me.conference && me.localMedia) {
                    me.zoomLevel = me.zoomLevel + 0.1;
                    me.zoomTo(me.zoomLevel);
                }
            }
        }] : null;

        var actionButtons = me.subchannel == 'videoconference' ? [{
            xtype: 'tbtext',
            itemId: 'status'
        }, '->', {
            text: btnExpand,
            iconCls: 'video-expand',
            handler: function () {
                if (me.fullscreenWindow) {
                    me.hideFullScreen();
                    this.setIconCls('video-expand');
                    this.setText(btnExpand);
                } else {
                    me.showFullScreen();
                    this.setIconCls('video-collapse');
                    this.setText(btnCollapse);
                }
            }
        }, {
            text: btnMuteVideo,
            iconCls: 'video-mute',
            //hidden: true,
            handler: function () {
                if (me.conference && me.localMedia) {
                    if (me.mutedVideo) {
                        me.mutedVideo = false;
                        this.setIconCls('video-mute');
                        this.setText(btnMuteVideo);

                        client.publish({ channel: me.getChannel(), data: 'UNMUTEVIDEO' });

                        me.localMedia.unmuteVideo();
                    } else {
                        // mute local so we stop sending
                        me.mutedVideo = true;
                        this.setIconCls('video-unmute');
                        this.setText(btnUnmuteVideo);

                        client.publish({ channel: me.getChannel(), data: 'MUTEVIDEO' });

                        me.localMedia.muteVideo();
                    }
                }
            }
        }, {
            text: btnMuteMic,
            iconCls: 'mic-mute',
            handler: function () {
                if (me.conference && me.localMedia) {
                    if (me.mutedMic) {
                        me.mutedMic = false;
                        this.setIconCls('mic-mute');
                        this.setText(btnMuteMic);

                        client.publish({ channel: me.getChannel(), data: 'UNMUTEAUDIO' });

                        me.localMedia.unmuteAudio();
                    } else {
                        // mute local so we stop sending
                        me.mutedMic = true;
                        this.setIconCls('mic-unmute');
                        this.setText(btnUnmuteMic);

                        client.publish({ channel: me.getChannel(), data: 'MUTEAUDIO' });

                        me.localMedia.muteAudio();
                    }
                }
            }
        }, {
            text: btnMuteSpeaker,
            iconCls: 'speakers-mute',
            handler: function () {
                if (me.conference && me.localMedia) {
                    if (me.mutedSpeakers) {
                        // unmute remote so we start hearing feeds
                        var links = me.conference.getLinks();
                        for (var i = 0; i < links.length; i++) {
                            links[i].unmuteRemoteAudio();
                        }
                        me.mutedSpeakers = false;
                        this.setIconCls('speakers-mute');
                        this.setText(btnMuteSpeaker);
                    } else {
                        // mute remote so we stop hearing feeds
                        var links = me.conference.getLinks();
                        for (var i = 0; i < links.length; i++) {
                            links[i].muteRemoteAudio();
                        }
                        me.mutedSpeakers = true;
                        this.setIconCls('speakers-unmute');
                        this.setText(btnUnmuteSpeaker);
                    }
                }
            }
        }] : [{
            text: btnExpand,
            iconCls: 'video-expand',
            handler: function () {
                if (me.fullscreenWindow) {
                    me.hideFullScreen();
                    this.setIconCls('video-expand');
                    this.setText(btnExpand);
                } else {
                    me.showFullScreen();
                    this.setIconCls('video-collapse');
                    this.setText(btnCollapse);
                }
            }
        }];

        if (this.canSnapshot) {
            console.log("Adding snapshot button");
            actionButtons = actionButtons.concat([{
                text: 'Snapshot',
                iconCls: 'cameraIcon',
                handler: function () {
                    console.log("Snapshot!");

                    if (me.layoutManager) {
                        var ctrls = me.layoutManager.getRemoteVideoControls();
                        if (ctrls && ctrls.length) {
                            if (ctrls.length === 1) {
                                var snapCtrl = ctrls[0];
                                me.takePicOf(snapCtrl);
                            } else {
                                me.hilightOn = true;
                            }

                            return;
                        }
                    }
                    Ext.Msg.toast({ title: 'Snapshot Error', body: 'No remote video streams to Snapshot' });
                }
            }]);
        }

        if (this.actionButtons) {
            actionButtons = actionButtons.concat(this.actionButtons);
        }

        Ext.apply(me, {
            tbar: zoomButtons,
            bbar: actionButtons,
            html: '<em style="display:block;width:100%;text-align:center;padding-top:5px;">' + this.waitingText + '</em>',
            options: Ext.apply({
                audio: true,
                video: true
            }, me.options || {})
        });

        me.callParent(arguments);
    },
    takePicOf: function (video) {
        console.log("Taking picture of remote video control: ", video);

        if (video && video.tagName && video.tagName === 'DIV') {
            console.log("Getting video child");
            video = video.getElementsByTagName('video')[0];
        }

        console.log(video);

        this.hilightOn = false;

        var scale = 1;
        var canvas = document.createElement("canvas");
        canvas.width = video.videoWidth * scale;
        canvas.height = video.videoHeight * scale;
        canvas.getContext('2d')
              .drawImage(video, 0, 0, canvas.width, canvas.height);

        var imgsrc = canvas.toDataURL();

        this.fireEvent('photo', imgsrc);
        //console.log(imgsrc);
    },
    updateAllCanvasDimensions: function () {
        var Links = this.conference.getLinks();
        for (var lnkIdx = 0; lnkIdx < Links.length; lnkIdx++) {
            this.updateCanvasDimensions(Links[lnkIdx].getPeerId());
        }
    },
    updateCanvasDimensions: function (peerId) {
        if (!this.storedCanvasSize) return;

        var storedCanvasSize = this.storedCanvasSize[peerId];

        var jqVideo = $(this.conference.getLink(peerId).getRemoteVideoControl());
        var jqCanvas = jqVideo.find('canvas').eq(0);
        var jqViewport = jqVideo.find('.viewport').eq(0);

        if (!jqVideo.length || !jqCanvas.length || !jqViewport.length || !storedCanvasSize) {
            console.log("Updating non-existant canvas object.");
            return;
        }

        var viewportElems = jqViewport.add(jqCanvas);

        //CSS borders throw off width/height calculations; bracket calcs with off/on.
        jqViewport.css('border', 'none');

        var viewableWidth = Math.min(storedCanvasSize.Width * storedCanvasSize.ScaleX, storedCanvasSize.ScreenWidth);
        var viewableHeight = Math.min(storedCanvasSize.Height * storedCanvasSize.ScaleY, storedCanvasSize.ScreenHeight);

        var canvasWidthRatio = /*viewableWidth*/storedCanvasSize.ScreenWidth / (storedCanvasSize.Width * storedCanvasSize.ScaleX);
        var canvasHeightRatio = /*viewableHeight*/storedCanvasSize.ScreenHeight / (storedCanvasSize.Height * storedCanvasSize.ScaleY);

        var pixelWidth = jqVideo.width() * canvasWidthRatio;
        var pixelHeight = jqVideo.height() * canvasHeightRatio;

        //viewportElems.width(canvasWidthRatio + '%').height(canvasHeightRatio + '%');

        var newX = (jqVideo.width() - pixelWidth) / 2.0;
        var newY = (jqVideo.height() - pixelHeight) / 2.0;

        var elemToDevRatioX = jqVideo.width() / storedCanvasSize.Width;
        var elemToDevRatioY = jqVideo.height() / storedCanvasSize.Height;

        newX -= (elemToDevRatioX * storedCanvasSize.OffsetX) / storedCanvasSize.ScaleX;
        newY -= (elemToDevRatioY * storedCanvasSize.OffsetY) / storedCanvasSize.ScaleY;

        viewportElems.css({
            'position': 'absolute',
            'top': newY,
            'left': newX,
            'width': pixelWidth,
            'height': pixelHeight,
            'background-color': 'transparent'
        });

        //These shouldn't be necessary, yet they are.  Canvas doesn't seem to be affected by .width()?
        jqCanvas.attr('width', jqCanvas.width())
                .attr('height', jqCanvas.height())
                .css('z-index', 1);

        jqViewport.css('border', '1px dashed #ccc');
    },
    zoomTo: function (size) {
        var videos = $(this.getLayout().getRenderTarget().dom).find('video');
        videos.each(function () {
            var $this = $(this);
            if (!$this._panzoom) {
                $this.panzoom();
                $this._panzoom = true;
            }
            $this.panzoom("zoom", size, { silent: true });
            if (size == 1) {
                $this.css("transform", 'none');
            }
        });
    },
    play: function () {
        this.isActive = true;
        var videos = $(this.getLayout().getRenderTarget().dom).find('video');
        for (var i = 0; i < videos.length; i++) {
            videos[i].play();
            if (this.zoom) {
                $(videos[i]).panzoom();
            }
        }
    },
    syncCanvas: function () {
        if (this.draw) {
            this.updateAllCanvasDimensions();
        }
    },
    onResize: function () {
        this.callParent(arguments);
        if (this.layoutManager) {
            this.layoutManager.doLayout();
            this.syncCanvas();
            this.play();
        }
    },
    afterLayout: function () {
        this.callParent(arguments);
        if (this.layoutManager) {
            this.layoutManager.doLayout();
            this.syncCanvas();
            this.play();
        }
    },
    onRender: function () {
        this.callParent(arguments);
        var me = this;

        window.setTimeout(function () {
            if (me.autoJoin) {
                if (me.autoWait) {
                    me.waitJoin();
                }

                me.joinConference(function () {
                    me.layoutManager.doLayout();
                    me.play();
                });
            }
        }, 1);
    },
    onDestroy: function () {
        this.leave();
        this.callParent(arguments);
    },
    getChannel: function () {
        var c = '/' + this.basechannel + '/' + this.uniqueId + '/' + this.subchannel;
        console.log(c);
        return c;
    },
    getSubChannel: function (sub) {
        var c = '/' + this.basechannel + '/' + this.uniqueId + '/' + sub;
        console.log(c);
        return c;
    },
    getSubSubChannel: function (sub) {
        var c = '/' + this.basechannel + '/' + this.uniqueId + '/' + this.subchannel + '/' + sub;
        console.log(c);
        return c;
    },
    getOfferAnswerTag: function () {
        return this.subchannel + ':offer/answer';
    },
    getCandidateTag: function () {
        return this.subchannel + ':candidate';
    },
    leave: function () {
        this.signalConferenceState("end");
        if (this.localMedia) {
            this.localMedia.stop();
        }
        fm.icelink.webrtc.screenshare.deactivate();
        if (this.conference) {
            this.client.leaveConference({
                conferenceChannel: this.getChannel()
            });
            this.layoutManager.unsetLocalVideoControl();
        }
        this.pulseTimeout = null;
        this.conference = null;
        this.localMedia = null;

        this.fireEvent('stop');
    },
    getAltWaitingText: function () {
        return '<em style="display:block;width:100%;text-align:center">' + (this.altWaitingText ? this.altWaitingText : this.waitingText) + '</em>';
    },
    signalConferenceState: function (action) {
        var me = this;
        var channel = me.getChannel();
        if (me.subchannel != 'videoconference' || channel.indexOf('/internalchat/') < 0) {
            return; //don't bother
        }

        var data = {
            NotificationType: "internalchatvideo",
            InternalChatVideo: {
                Action: action
            }
        }

        me.client.service({
            channel: channel,
            data: data,
            onSuccess: function (e) {
            },
            onFailure: function (e) {
            }
        });
    },
    waitJoin: function () {
        var me = this;
        me.client.subscribe({
            channel: me.getSubChannel("meta"),
            onReceive: function (e) {
                var arr = e.getData()
                  , pubClient = e.getPublishingClient()
                  , instanceId = pubClient.getBoundRecordValue('mwc.instanceId')
                  , permittedPeerIds
                  , start = arr[0] === "start";
                console.log(arr, instanceId);
                if (start) {
                    if (me.permittedPeerIds.indexOf(instanceId) < 0) {
                        me.permittedPeerIds.push(instanceId);
                    }
                } else {
                    var pos = me.permittedPeerIds.indexOf();
                    if (pos >= 0) {
                        me.permittedPeerIds.splice(pos, 1);
                    }
                }
                me.checkPermitted();
                console.log(me.permittedPeerIds);
            }
        });
    },
    initSketchCanvas: function (peerId) {
        var me = this;
        var remoteVideoControl = me.conference.getLink(peerId).getRemoteVideoControl();
        var canvas = document.createElement('canvas');
        remoteVideoControl.appendChild(canvas);

        var div = document.createElement('div');
        div.className = 'viewport';
        remoteVideoControl.appendChild(div);

        me.updateAllCanvasDimensions();

        $(canvas)
            .sketch({ defaultColor: "#ff0" })
            .on('start', function () {
                try {
                    me.conference.sendReliableString(me.sketchDCI, "{'Action':'Start'}", peerId);
                }
                catch (e) {
                    console.log("EXCEPT: " + e);
                }
            })
            .on('stop', function () {
                try {
                    me.conference.sendReliableString(me.sketchDCI, "{'Action':'Stop'}", peerId);
                }
                catch (e) {
                    console.log("EXCEPT: " + e);
                }
            })
            .on('point', function (e, pt) {
                try {
                    me.conference.sendReliableString(me.sketchDCI, "{'Action':'Point', 'Point':{'X':" + Math.round(pt.X) + ",'Y':" + Math.round(pt.Y) + ",'W':" + pt.W + ",'H':" + pt.H + "}}", peerId);
                }
                catch (e) {
                    console.log("EXCEPT: " + e);
                }
            });
    },
    ifPeerIdPermitted: function (peerId) {
        var me = this,
            conference = me.conference,
            layoutManager = me.layoutManager;

        if (!conference) {
            return false;
        }

        if (me.isPeerIdPermitted(peerId)) { //Add peer
            var link = conference.getLink(peerId);
            if (!!link) {
                var peerState = link.getPeerState(),
                    remoteVideoControl = link.getRemoteVideoControl();
                if (layoutManager.getRemoteVideoControl(peerId) == null) {// only add a remote video control if its not already added
                    if (me.labelRemote) {
                        var div = me.getDisplayNameDiv(peerState['displayName'].getValue());
                        remoteVideoControl.appendChild(div);
                    }
                    try {
                        me.uniqueDeviceId = peerState['DeviceUniqueId'].getValue();
                        console.log("Device ID: ", me.uniqueDeviceId);
                    } catch (e) { }

                    if (!peerState['novideo'] && me.showRemote) {
                        layoutManager.addRemoteVideoControl(peerId, remoteVideoControl);
                        var rvc = layoutManager.getRemoteVideoControl(peerId);
                        console.log("RVC: ", rvc);
                        console.log(Ext.get(rvc));
                        console.log(rvc.getElementsByTagName('video')[0]);
                        var video = rvc.getElementsByTagName('video')[0];
                        rvc.addEventListener('mouseover', function (e) { if (me.hilightOn) rvc.style.border = "solid 4px blueviolet"; });
                        rvc.addEventListener('mouseout', function (e) { if (me.hilightOn) rvc.style.border = "none"; });
                        rvc.addEventListener('click', function (e) { if (me.hilightOn) { rvc.style.border = "none"; me.takePicOf(video); } });
                        //Ext.get(rvc).getEl().addListener('mouseover', function () { console.log('mouseover ' + peerId) });
                        if (me.draw) {
                            //me.initSketchCanvas(peerId);
                        }
                    }
                }
            }
        } else { //remove it if its in the layout manager;
            layoutManager.removeRemoteVideoControl(peerId);
        }
    },
    isPeerIdPermitted: function (peerId) {
        var me = this;
        return !me.permittedPeerIds || me.permittedPeerIds.indexOf(peerId) >= 0;
    },
    checkPermitted: function () {
        var me = this,
            conference = me.conference,
            currentLayoutManager = me.layoutManager;
        if (!!conference && !!currentLayoutManager) {
            var peerIds = conference.getPeerIds(),
                visiblePeers = currentLayoutManager.getPeerIds(),
                peerCount = peerIds.length;

            for (var i = 0; i < peerCount; i++) {
                var peerId = peerIds[i];
                me.ifPeerIdPermitted(peerId);
            }
        }
    },
    joinConference: function (successCallback, asCreator) {
        var metaJson = null;

        if (arguments.length >= 2) {
            metaJson = JSON.stringify({
                Creator: !!asCreator
            });
        }

        this.update(this.getAltWaitingText());

        var me = this;
        if (Ext.isArray(me.permittedPeerIds)) {
            me.waitJoin();
        }
        var target = me.body.dom.firstChild;
        if (target.firstChild) {
            target = target.firstChild;
        }
        if (target && target.style) {
            target.style.verticalAlign = 'middle';
        }

        fm.log.setProvider(new fm.consoleLogProvider(fm.logLevel.Info));

        if (this.screen) {
            fm.icelink.webrtc.screenshare.activate();
        } else {
            fm.icelink.webrtc.screenshare.deactivate();
        }
        fm.icelink.webrtc.userMedia.getMedia({
            defaultVideoPreviewScale: me.subchannel == 'videoconference' ? fm.icelink.webrtc.layoutScale.Cover : fm.icelink.webrtc.layoutScale.Contain,
            defaultVideoScale: me.subchannel == 'videoconference' ? fm.icelink.webrtc.layoutScale.Cover : fm.icelink.webrtc.layoutScale.Contain,
            audio: this.options.audio,
            video: this.options.video,
            videoWidth: 320,
            videoHeight: 240,
            onFailure: function (e) {
                console.log("GetMedia Error: ", e);
                Ext.Msg.alert(
                    MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.video.media.error.title'),
                    MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.video.media.error.message', e.getException().message));
                me.signalConferenceState("cancel");
            },
            onSuccess: function (e) {
                me.update(me.pendingText);
                me.fireEvent('start');

                var container = me.getLayout().getRenderTarget().dom;
                container.style.display = 'block';
                if (container.parentNode) {
                    container.parentNode.style.display = 'block';
                }

                var layoutManager = me.getLayoutManager(container);
                me.layoutManager = layoutManager;

                var localMedia = me.localMedia = e.getLocalStream();
                var localVideoControl = me.localVideoControl = e.getLocalVideoControl();

                if (localVideoControl != null && me.showLocal) {
                    layoutManager.setLocalVideoControl(localVideoControl);
                    if (me.labelLocal) {
                        var div = me.getDisplayNameDiv('Me', true);
                        localVideoControl.appendChild(div);
                    }

                    if (me.screen) {
                        // no flipping screens
                        localVideoControl.style.transform = 'scale(1,1)';
                    }
                }
                var audioStream = new fm.icelink.webrtc.audioStream(localMedia);
                var videoStream = new fm.icelink.webrtc.videoStream(localMedia);

                function getPeerName(args) {
                    if (args.getPeerState() && args.getPeerState().displayName && args.getPeerState().displayName.getValue) {
                        return args.getPeerState().displayName.getValue();
                    }
                    return '';
                }

                videoStream.addOnLinkInit(function (e) {
                    fm.log.warn("videoStream LinkInit: " + getPeerName(e));
                });
                videoStream.addOnLinkUp(function (e) {
                    var peerId = e.getPeerId();
                    fm.log.warn("videoStream LinkUp: " + getPeerName(e) + ":" + peerId);

                    console.log("Peer state:", e.getPeerState());

                    me.ifPeerIdPermitted(peerId);

                    // weird bug in FF calculation of width; this fixes it
                    var count = 0;
                    var forceResizeInterval = window.setInterval(function () {
                        me.doLayout();
                        if (count > 50) {
                            window.clearInterval(forceResizeInterval);
                        }
                        count++;
                    }, 100);

                    me.fireEvent('peerarrived');
                });
                videoStream.addOnLinkDown(function (e) {
                    fm.log.warn("videoStream LinkDown: " + getPeerName(e) + " exception: " + e.getException());
                    var peerid = e.getPeerId();

                    if (layoutManager.getRemoteVideoControl(peerid) != null) {
                        layoutManager.removeRemoteVideoControl(e.getPeerId());
                        me.doLayout();
                    }
                    if (conference.getLinks().length == 0 && me.autoLeave) {
                        //me.leaveConference();
                        //me.fireEvent('leave', { auto: true });
                    }

                    if (conference.getLinks().length == 0) {
                        me.fireEvent('empty');
                    }
                });

                var client = me.client;
                var streams = me.streams = [];
                streams.push(audioStream);
                streams.push(videoStream);

                if (me.draw) {
                    me.sketchDCI = new fm.icelink.webrtc.reliableDataChannel('Sketch');
                    me.pulseDCI = new fm.icelink.webrtc.reliableDataChannel('Pulse');
                    me.pulseDCI.setOnReceive(function (e) {
                        try {
                            var obj = JSON.parse(e.getDataString())[0];
                            if (obj.face[2] > 2 && obj.face[3] > 2) {
                                var link = e.getLink();
                                var jqVid = $(link.getRemoteVideoControl());

                                if (!me.img) {
                                    if ($('video').length > 0) {
                                        me.img = $('<div class="speculum_focusgrid" style="bottom: 0px;">' +
                                                        '<div class="speculum_datagrid">' +
                                                            '<div class="speculum_grid" style="margin-bottom: 0.3em;">' +
                                                                '<div class="speculum_3column" style="padding-right: 10%;">' +
                                                                    '<div class="speculum_dataval" id="bpmval">66</div>' +
                                                                    '<div class="speculum_datalabel">BPM</div>' +
                                                                '</div>' +
                                                                '<div class="speculum_3column" style="padding-right: 10%;">' +
                                                                    '<div class="speculum_dataval" id="o2val">96</div>' +
                                                                    '<div class="speculum_datalabel">OXY</div>' +
                                                                '</div>' +
                                                                '<div class="speculum_3column">' +
                                                                    '<div class="speculum_dataval" id="tmpval">102<span style="font-size: 0.5em;">.6</span></div>' +
                                                                    '<div class="speculum_datalabel">TMP</div>' +
                                                                '</div>' +
                                                            '</div>' +
                                                            '<div style="margin-bottom: 0.3em;">EDI: +4789</div>' +
                                                            '<div style="margin-bottom: 0.3em;">' +
                                                                '<div style="display: inline-block;">O+</div>' +
                                                                '<div class="speculum_allergy" style="display: inline-block; margin-left: 3em;">PCN</div>' +
                                                            '</div>' +
                                                        '</div>' +
                                                    '</div>')
                                                  .appendTo(jqVid)
                                                  .css({
                                                      'position': 'absolute',
                                                      'z-index': '10',
                                                      'display': 'none',
                                                  });
                                    }
                                }

                                if (me.img) {
                                    if (me.img.is(':hidden')) {
                                        var tmp = me.pulseTimeout = function () {
                                            setTimeout(function () {
                                                if (!me) {
                                                    return;
                                                }

                                                var tmp = me.pulseTimeout;

                                                if (tmp == null || Date.now() - me.pulseAlive > 5000) {
                                                    if (me.img) {
                                                        me.img.hide();
                                                    }
                                                    me.pulseTimeout = null;
                                                    return;
                                                }

                                                if (tmp != null) {
                                                    tmp();
                                                }
                                            },
                                            1000);
                                        };

                                        tmp();
                                    }

                                    var jqVideo = jqVid.find('video').eq(0);

                                    var inXYRatio = (obj.img_dim[0] * 1.0) / (obj.img_dim[1] * 1.0);
                                    var outXYRatio = (1.0 * jqVideo.width()) / (1.0 * jqVideo.height());

                                    var unboundedWidth;
                                    var unboundedHeight;
                                    var xOffset = 0;
                                    var yOffset = 0;

                                    //X is bigger
                                    if (outXYRatio > inXYRatio) {
                                        unboundedWidth = jqVideo.width();
                                        unboundedHeight = jqVideo.width() / (inXYRatio * 1.0);
                                        yOffset = 0.5 * (unboundedHeight - jqVideo.height());
                                    }
                                    else {
                                        unboundedHeight = jqVideo.height();
                                        unboundedWidth = jqVideo.height() * inXYRatio;
                                        xOffset = 0.5 * (unboundedWidth - jqVideo.width());
                                    }

                                    var scaleX = (1.0 * jqVideo.width()) / (obj.img_dim[0] * 1.0);
                                    var scaleY = (1.0 * jqVideo.height()) / (obj.img_dim[1] * 1.0);

                                    var trueScaleX = (1.0 * unboundedWidth) / (obj.img_dim[0] * 1.0);
                                    var trueScaleY = (1.0 * unboundedHeight) / (obj.img_dim[1] * 1.0);

                                    var scalePct = Math.min(trueScaleX, trueScaleY);

                                    me.img.css({
                                        'transform': 'scale(' + scalePct * 0.3 + ')',
                                        'left': trueScaleX * (obj.face[0] + obj.face[2] * 0.5) - xOffset - 105,
                                        'top': trueScaleY * (obj.face[1] + obj.face[3] * 0.5) - yOffset - 90,
                                    });

                                    $('#bpmval').text('' + obj.bpm);

                                    me.img.show();
                                    me.pulseAlive = Date.now();
                                }
                                else {
                                    me.pulseTimeout = null;
                                    me.img.hide();
                                }
                            }
                        }
                        catch (e) {
                            console.log(e);
                        }
                    });

                    streams.push(new fm.icelink.webrtc.reliableDataStream([me.pulseDCI, me.sketchDCI], false));
                }

                var conference = me.conference = new fm.icelink.conference(MedwebCloud.serverAddressIceLink, MedwebCloud.serverPortIceLink, streams);
                conference.setRelayUsername('test');
                conference.setRelayPassword('pa55w0rd!');
                conference.addOnLinkInit(function (e) {
                    me.setLoading('Please wait while the video stream is established...');
                    //fm.log.info('VideoPanel.js: Link to peer initializing...\n' + e.getPeerId() + "\n" + JSON.stringify(e.getPeerState(), null, '\t'));
                    console.log('VideoPanel.js: Link to peer initializing...\n' + e.getPeerId() + '\n', e.getPeerState());
                });
                conference.addOnLinkUp(function (e) {
                    me.setLoading(false);
                    //fm.log.info('VideoPanel.js: Link to peer is UP.\n' + e.getPeerId() + "\n" + +JSON.stringify(e.getPeerState(), null, '\t'));
                    console.log('VideoPanel.js: Link to peer is UP.\n' + e.getPeerId());
                    me.fireEvent('up', e.getPeerState());
                });
                conference.addOnLinkDown(function (e) {
                    me.setLoading(false);
                    me.fireEvent('down', e.getPeerState());
                    fm.log.info('VideoPanel.js: Link to peer is DOWN.\n' + e.getPeerId() + '\n' + e.getException().message);
                });

                var joinArgs = new fm.icelink.websync.joinConferenceArgs({
                    conferenceChannel: me.getChannel(),
                    conference: conference,
                    onFailure: function (e) {
                        try {
                            Ext.Msg.hide();
                        } catch (e) { }
                        console.log("JoinConference Error:", e);
                        alert(MedwebCloud.getMsg('medweb.view.usermanagementpanel.action.video.join.error.message', e.getConferenceChannel(), e.getException().message));
                    },
                    onSuccess: function (e) {
                        try {
                            Ext.Msg.hide();
                        } catch (e) { }
                    }
                });
                if (metaJson != null) {
                    joinArgs.setMetaJson(metaJson);
                }

                client.startBatch();
                {
                    if (me.adHoc) {
                        client.bind(new fm.websync.bindArgs([new fm.websync.record("novideo", "true", false)]));
                    }

                    if (me.trackPulse) {
                        client.bind(new fm.websync.bindArgs([new fm.websync.record("spec-pulse", "true", false)]));
                    }

                    //client.bind(new fm.websync.bindArgs([new fm.websync.record("MWID", '"' + client.getInstanceId() + '"', false)]));

                    client.joinConference(joinArgs);
                }
                client.endBatch();

                me.client.subscribe({
                    channel: me.uniqueDeviceId ? '/lookingglass/' + me.uniqueDeviceId + '/videoconference/location' : me.getSubSubChannel('location'),
                    onReceive: function (e) {
                        if (e && e.getData()) {
                            me.fireEvent('locationChanged', e.getData());
                        }
                        else {
                            console.log("Received invalid location data: ", e);
                        }
                    },
                    onSuccess: function (e) {
                        console.log('Subscribed to location channel');
                    }
                });

                if (me.draw) {
                    me.client.subscribe({
                        channel: me.uniqueDeviceId ? '/lookingglass/' + me.uniqueDeviceId + '/videoconference/sketchcanvas' : me.getSubSubChannel('sketchcanvas'),
                        onReceive: function (e) {
                            if (!me.conference) return;

                            var data = e.getData();
                            var capability = e.getPublishingClient().getBoundRecordValue('medweb.capability');
                            var pubPeerId = capability['instance'];

                            console.log("Received sketchcanvas data from " + pubPeerId + ": ", e);

                            if (data !== '["requestsize"]') {
                                if (!pubPeerId || !me.conference.getLink(pubPeerId) || !me.conference.getLink(pubPeerId).getRemoteVideoControl()) {
                                    console.log("No such video stream found for sketchcanvas.");
                                    return;
                                }

                                me.storedCanvasSize = me.storedCanvasSize || {};
                                if (!me.storedCanvasSize[pubPeerId]) me.initSketchCanvas(pubPeerId);
                                me.storedCanvasSize[pubPeerId] = data;
                                me.updateCanvasDimensions(pubPeerId);
                            }
                        },
                        onSuccess: function (e) {
                            var pubChannel = me.uniqueDeviceId ? '/lookingglass/' + me.uniqueDeviceId + '/videoconference/sketchcanvas' : me.getSubSubChannel('sketchcanvas');
                            console.log("Publishing to [" + pubChannel + "]");
                            me.client.publish({
                                channel: pubChannel,
                                data: '["requestsize"]'
                            });
                        }
                    });
                }
            }
        });
    },
    getLayoutManager: function (container) {
        var layoutManager = new fm.icelink.webrtc.layoutManager(container);
        return layoutManager;
    },
    conferenceIsActive: function () {
        return this.conference != null;
    },
    getDisplayNameDiv: function (name, local) {
        var div = document.createElement('div');
        div.innerHTML = name;

        div.style.color = '#ccc';
        div.style.position = 'absolute';
        div.style.top = '5px';
        div.style.left = '5px';
        div.style.zIndex = '1';
        div.style.textShadow = '2px 1px #000';

        if (local && !screen) {
            div.style.transform = 'scale(-1,1)';
            console.log('transform');
            window.temp = div;
        }

        return div;
    },

    hideFullScreen: function () {
        this.fullscreenWindow.close();
        this.fullscreenWindow = null;
    },
    showFullScreen: function () {
        var oldContainer = this.ownerCt;

        var me = this;
        var title = MedwebCloud.getMsg('medweb.view.videopanel.fullscreen.title');
        this.fullscreenWindow = new Ext.window.Window({
            title: title,
            maximized: true,
            closable: false,
            layout: 'fit',
            items: [this],
            listeners: {
                close: function () {
                    var parent = oldContainer;
                    if (me.restoreToContainerSelector) {
                        // make sure to include the current element if we can
                        if (oldContainer.up()) {
                            parent = oldContainer.up().down(me.restoreToContainerSelector);
                        } else {
                            parent = oldContainer.down(me.restoreToContainerSelector);
                        }
                        if (parent == null) {
                            parent = oldContainer.up(me.restoreToContainerSelector);
                        }
                    }

                    parent.add(me);
                    parent.doLayout();
                    var layout = parent.getLayout();
                    if (layout != null && layout.setActiveItem != null) {
                        layout.setActiveItem(me);
                    }
                }
            }
        });
        this.fullscreenWindow.show();
        me.doLayout();
    },
    setUniqueId: function (id) {
        this.uniqueId = id;
    }
});