﻿Ext.define('MedwebCloud.view.VitalsPanel', {
    extend: 'Ext.panel.Panel',
    xtype: 'MedwebCloud.vitalspanel',
    record: null,
    client: null,
    subchannel: 'vitals',
    initComponent: function () {
        Ext.apply(this, {
            border: false,
            // 2 vertical
            layout: {
                type: 'vbox',
                align: 'stretch'
            },
            defaults: {
                border: true,
                flex: 1,
                xtype: 'container',
                layout: {
                    type: 'hbox',
                    align: 'stretch'
                },
                defaults: {
                    flex: 1,
                    cls: 'vitalsBox'
                }
            },
            items: [{
                // 3 horizontal
                items: [{
                    itemId: 'pulse',
                    tpl: '<div class="value">{Value}</div><span class="label"><i class="fa fa-heart fa-lg"></i></span>'
                }, {
                    itemId: 'respiration',
                    tpl: '<div class="value">{Value}</div><span class="label">R</span>'
                }, {
                    itemId: 'systolicBloodPressure',
                    tpl: '<div class="value">{Value}</div><span class="label">SYS</span>'
                }]
            }, {
                items: [{
                    itemId: 'o2',
                    tpl: '<div class="value">{Value}</div><span class="label">O<sub>2</sub></span>'
                }, {
                    itemId: 'temperature',
                    tpl: '<div class="value">{Value}&deg;</div><span class="label">T</span>'
                }, {
                    itemId: 'diastolicBloodPressure',
                    tpl: '<div class="value">{Value}</div><span class="label">DIA</span>'
                }]
            }]
        });
        this.callParent(arguments);
        this.join();
    },
    onRender: function () {
        this.callParent(arguments);
        var me = this;

        // fill in all the vitals with blanks
        window.setTimeout(function () {
            me.clearVitals();
        }, 1);
    },
    clearVitals: function () {
        var items = this.queryBy(function (p) { return p.cls == 'vitalsBox' })
        for (var i = 0; i < items.length; i++) {
            var item = items[i];
            item.update({ Value: ' - ' });
        }
    },
    onDestroy: function () {
        this.leave();
        this.callParent(arguments);
    },
    getChannel: function () {
        return '/collage/' + this.record.get('Id') + '/' + this.subchannel;
    },
    join: function () {
        var client = this.client;
        var me = this;
        client.subscribe({
            channel: this.getChannel(),
            onFailure: function (e) {
                Ext.Msg.alert(
                    MedwebCloud.getMsg('medweb.view.vitals.action.subscribe.failure.message'),
                    MedwebCloud.getMsg('medweb.view.vitals.action.subscribe.failure.title', e.getChannel(), e.getException().message));
            },
            onReceive: function (e) {
                // reset the timer when we get data, and then set it again for 5s, so 5s of no data = reset
                window.clearTimeout(me._vitalsTimeout);
                me._vitalsTimeout = window.setTimeout(Ext.bind(me.clearVitals, me), 5000);

                // vitals data, log it for now
                var data = e.getData();

                // backwards compatibility - handle either the old or new data formats (array vs. single item)
                if (data.length) {
                    for (var i = 0; i < data.length; i++) {
                        me.updateVitals(data[i]);
                    }
                } else {
                    me.updateVitals(data);
                }
            }
        });
    },
    updateVitals: function (data) {
        var type = data.VitalsType;
        var itemId = '';
        switch (type) {
            case 1: // pulse
                itemId = 'pulse';
                break;
            case 2: // temp
                itemId = 'temperature';
                break;
            case 3: // resp
                itemId = 'respiration';
                break;
            case 4: // o2
                itemId = 'o2';
                break;
            case 5: // systolic
                itemId = 'systolicBloodPressure';
                break;
            case 6: // diastolic
                itemId = 'diastolicBloodPressure';
                break;
        }
        var panel = this.down('#' + itemId);
        panel.update(data);
    },
    leave: function () {
        this.client.unsubscribe({
            channel: this.getChannel()
        });
    }
});