﻿Ext.define('MedwebCloud.view.WaitingRoom.ActiveChatsGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'MedwebCloud.wr.activechatsgrid',

    loadMask: true,
    activeChatsStore: null,

    initComponent: function () {
        var me = this;

        MedwebCloud.websyncClient.subscribe({
            channel: '/' + MedwebCloud.UIConfig.CompanyContext.Subdomain + '/waitingroom/activeChats',
            onSuccess: function (e) {
                var chats = e.getExtensionValueJson('activeChats');
                me.activeChatsStore.loadRawData(fm.json.deserialize(chats));
            },
            onFailure: function (e) {
                var msg = MedwebCloud.getMsg('medweb.waitingroom.activechats.subscribe.failure.title');
                var title = MedwebCloud.getMsg('medweb.waitingroom.activechats.subscribe.failure.title');
                Ext.Msg.alert(title, msg);
            },
            onReceive: function (e) {
                var data = e.getData();
                if (!data.chatRequest) return;
                var records = me.parseRawRecords(data.chatRequest);
                if (records.length > 0) {
                    try {
                        var redId = records[0].get('Id');
                        var rec = me.activeChatsStore.findRecord('Id', redId);

                        if (data.action == 'add' && !rec) {
                            me.activeChatsStore.add(records);
                        }
                        else if (data.action == 'remove' && rec) {
                            me.activeChatsStore.remove(rec);
                        }
                    }
                    catch (ex) {
                        console.log("Error executing action '" + data.action + "' for record ID : " + redId);
                    }
                }
            }
        });

        me.activeChatsStore = Ext.create('MedwebCloud.store.ChatRequests', {});

        var columnNameTitle = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.name.title');
        var columnEmailTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.email.title');
        var columnIDTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.id.title');

        var columnElapsedTitle = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.title');

        var columnActionTitle = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.action.title');
        var columnActionTooltip = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.action.start.tooltip');

        Ext.apply(me, {
            store: me.activeChatsStore,
            autoHeight: true,
            columns: [
				{
				    dataIndex: 'DisplayName',
				    text: columnNameTitle,
				    flex: 1
				}, {
				    dataIndex: 'Email',
				    text: columnEmailTitle,
				    width: 80
				}, {
				    dataIndex: 'Identifier',
				    text: columnIDTitle,
				    width: 80
				}, {
				    text: columnElapsedTitle,
				    dataIndex: 'ModifiedOn',
				    width: 75,
				    renderer: function (value, meta, record) {
				        var now = new Date();
				        var milliseconds = now - value;

				        var x = milliseconds / 1000;
				        var seconds = ~~(x % 60); x /= 60;
				        var minutes = ~~(x % 60); x /= 60;
				        var hours = ~~(x % 24); x /= 24;
				        var days = ~~(x);

				        var dayFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.day', days);
				        var hrsFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.hrs', hours);
				        var minFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.min', minutes);
				        var secFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.sec', seconds);

				        return (days >= 1 ? dayFmt + ' ' : '') +
                            (hours >= 1 ? hrsFmt + ' ' : '') +
                            (minutes >= 1 ? minFmt + ' ' : '') +
                            (seconds >= 1 ? secFmt + ' ' : '');
				    }
				}, {
				    xtype: 'actioncolumn',
				    text: columnActionTitle,
				    width: 40,
				    items: [{
				        iconCls: 'startIcon',
				        tooltip: columnActionTooltip,
				        handler: function (grid, rowIndex, colIndex, item, e, record, row) {
				            me.fireEvent('startChat', {
				                chat: record.getChat(),
				                chatRequest: record
				            });
				        }
				    }]
				}
            ]
        });

        me.addListener('afterrender', function () {
            setInterval(function () {
                me.getView().refresh();
            }, 1000);
        });

        me.callParent(arguments);
    },
    parseRawRecords: function (data) {
        var me = this;
        var result = me.activeChatsStore.getProxy().getReader().read(data);

        return result.records;
    }
});