﻿/*
***TODO: I18N in this file***
*/
Ext.define('MedwebCloud.view.WaitingRoom.ChatRequestsGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'MedwebCloud.wr.chatrequestsgrid',
    requires: ['MedwebCloud.model.ChatRequest', 'MedwebCloud.model.Chat'],

    loadMask: true,
    chatRequestsStore: null,
    client: null,
    initComponent: function () {
        var me = this;

        me.client.subscribe({
            channel: '/' + MedwebCloud.UIConfig.CompanyContext.Subdomain + '/waitingroom/chatRequests',
            onSuccess: function (e) {
                var chats = e.getExtensionValueJson('chatRequests');
                me.chatRequestsStore.loadRawData(fm.json.deserialize(chats));
            },
            onFailure: function (e) {
                Ext.Msg.alert('Error', 'Could not subscribe /chatRequests channel.');
            },
            onReceive: function (e) {
                var data = e.getData();
                if (!data.chatRequest) return;
                var records = me.parseRawRecords(data.chatRequest);
                if (records.length > 0) {
                    try {
                        var redId = records[0].get('Id');
                        var rec = me.chatRequestsStore.findRecord('Id', redId);

                        if (data.action == 'add' && !rec) {
                            try {
                                var notification = soundManager.createSound({
                                    url: '/sounds/notification.mp3'
                                });
                                notification.play();
                            } catch (e) { }
                            me.chatRequestsStore.add(records);
                        }
                        else if (data.action == 'remove' && rec) {
                            me.chatRequestsStore.remove(rec);
                        }
                    }
                    catch (ex) {
                        console.log("Error executing action '" + data.action + "' for record ID : " + redId);
                    }
                }
            }
        });

        me.chatRequestsStore = Ext.create('MedwebCloud.store.ChatRequests', {});

        var columnNameTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.name.title');
        var columnEmailTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.email.title');
        var columnIDTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.id.title');
        var columnTotalTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.totaltime.title');
        var columnActionTitle = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.action.title');
        var columnActionTooltip = MedwebCloud.getMsg('medweb.waitingroom.requests.grid.column.action.start.tooltip');

        Ext.apply(me, {
            store: me.chatRequestsStore,
            autoHeight: true,
            //dockedItems: [{
            //    dock: 'bottom',
            //    xtype: 'pagingtoolbar',
            //    store: me.chatRequestsStore,
            //    displayInfo: true
            //}],
            columns: [
				{
				    dataIndex: 'DisplayName',
				    text: columnNameTitle,
				    flex: 1
				}, {
				    dataIndex: 'Email',
				    text: columnEmailTitle,
				    width: 80
				}, {
				    dataIndex: 'Identifier',
				    text: columnIDTitle,
				    width: 80
				}, {
				    dataIndex: 'ModifiedOn',
				    text: columnTotalTitle,
				    width: 75,
				    renderer: function (value, meta, record) {
				        var now = new Date();
				        var milliseconds = now - value;

				        var x = milliseconds / 1000;
				        var seconds = ~~(x % 60); x /= 60;
				        var minutes = ~~(x % 60); x /= 60;
				        var hours = ~~(x % 24); x /= 24;
				        var days = ~~(x);

				        var dayFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.day', days);
				        var hrsFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.hrs', hours);
				        var minFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.min', minutes);
				        var secFmt = MedwebCloud.getMsg('medweb.waitingroom.activechats.grid.column.elapsedtime.format.sec', seconds);

				        return (days >= 1 ? dayFmt + ' ' : '') +
                            (hours >= 1 ? hrsFmt + ' ' : '') +
                            (minutes >= 1 ? minFmt + ' ' : '') +
                            (seconds >= 1 ? secFmt + ' ' : '');
				    }
				}, {
				    xtype: 'actioncolumn',
				    text: columnActionTitle,
				    width: 40,
				    items: [{
				        iconCls: 'startIcon',
				        tooltip: columnActionTooltip,
				        handler: function (grid, rowIndex, colIndex, item, e, record, row) {
				            me.fireEvent('startChat', {
				                chat: record.getChat(),
				                chatRequest: record
				            });
				            grid.getStore().remove(record);
				        }
				    }]
				}
            ]
        });

        /* whoa - this is refreshing *way* more than in should...need to fix it, maybe update on the records directly or something */
        me.addListener('afterrender', function () {
            setInterval(function () {
                me.getView().refresh();
            }, 1000);
        });

        me.callParent(arguments);
    },
    parseRawRecords: function (data) {
        var me = this;
        var result = me.chatRequestsStore.getProxy().getReader().read(data);

        return result.records;
    }
});