﻿/*
***TODO: I18N in this file***
*/
Ext.define('MedwebCloud.view.WaitingRoom.DoctorsGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'MedwebCloud.wr.doctorsgrid',

    loadMask: true,

    initComponent: function () {
        var me = this;

        var labels = {
            displayname: MedwebCloud.getMsg('medweb.view.waitingroom.displayname.label')
        };

        Ext.eventBus.on('user:join', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Active);
        });
        Ext.eventBus.on('user:leave', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Idle);
        });
        Ext.eventBus.on('user:busy', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Busy);
        });
        Ext.eventBus.on('user:offline', function (user) {
            if (isNaN(user)) {
                user = user.getUserId();
            } else {
                user = parseInt(user);
            }

            me.setUserPresence(user, MedwebCloud.Core.Enums.Presence.Offline);
        });

        Ext.apply(me, {
            autoHeight: true,
            sortableColumns: false,
            features: [{
                ftype: 'grouping'
            }],
            store: Ext.create('MedwebCloud.v1.store.Users', {
                groupField: 'Department',
                proxy: {
                    type: 'therest',
                    url: MedwebCloud.api('/v1/doctors')
                }
            }),
            columns: [{
                width: 24,
                dataIndex: 'PresenceSort',
                renderer: function (value, meta, record) {
                    var v = record.get('Presence');
                    var cls = MedwebCloud.getPresenceClassName(v);
                    return '<div class="presence ' + cls + '">&nbsp;</div>';
                }
            }, {
                dataIndex: 'DisplayName',
                text: labels.displayname,
                flex: 1,
                renderer: function (value, meta, record) {
                    if (!value) {
                        return record.get('Username')
                    }
                    return '<b>' + value + '</b><br/>' + record.get('Username');
                }
            }, {
                dataIndex: 'GravatarImage',
                width: 40
            }]
        });

        me.callParent(arguments);

        me.store.load();
    },
    setUserPresence: function (userId, presence) {
        var me = this;
        var userIndex = me.store.find('Id', userId)
        if (userIndex >= 0) {
            var record = me.store.getAt(userIndex);
            record.set('Presence', presence);
            record.commit();
            me.store.sort('PresenceSort', 'ASC');
        }
    }
});