﻿Ext.define('MedwebCloud.view.WaitingRoom.InvitationWindow', {
    extend: 'Ext.Window',
    xtype: 'MedwebCloud.wr.invitationwindow',
    requires: ['MedwebCloud.view.ContactsGrid'],
    selectedRecord: null,
    initComponent: function () {
        var me = this;

        var tabPersonal = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.personal.title');
        var btnAdd = MedwebCloud.getMsg('medweb.view.invitationpanel.personal.button.add.title');
        var tabInstitutional = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.institutional.title');
        var tabGlobal = MedwebCloud.getMsg('medweb.view.invitationpanel.tab.global.title');

        //var tabPersonal = MedwebCloud.getMsg('medweb.view.waitingroom.invitationwindow.tab.personal.title');
        //var btnAdd = MedwebCloud.getMsg('medweb.view.waitingroom.invitationwindow.personal.button.add.title');
        var btnTransfer = MedwebCloud.getMsg('medweb.view.waitingroom.invitationwindow.personal.button.transfer.title');
        //var tabInstitutional = MedwebCloud.getMsg('medweb.view.waitingroom.invitationwindow.tab.institutional.title');
        //var tabGlobal = MedwebCloud.getMsg('medweb.view.waitingroom.invitationwindow.tab.global.title');

        var buttonTransfer = Ext.create('Ext.Button', {
            text: btnTransfer,
            handler: function () {
                if (me.selectedRecord != null) {
                    me.transferSelected();
                }
            }
        });
        var buttonAdd = Ext.create('Ext.Button', {
            text: btnAdd,
            handler: function () {
                MedwebCloud.createContact(function (values) {
                    me.fireEvent('invite', values.Email, values.DisplayName);
                });
            }
        });

        var contacts = Ext.create('Ext.tab.Panel', {
            border: false,
            activeItem: 0,
            items: [{
                title: tabPersonal,
                xtype: 'MedwebCloud.contactsgrid',
                itemId: 'personalGrid',
                canEdit: true,
                contactType: 1
            }, {
                title: tabInstitutional,
                xtype: 'MedwebCloud.contactsgrid',
                contactType: 2
            }],
            listeners: {
                cellclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                    me.selectedRecord = record;
                },
                celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                    me.selectedRecord = record;
                    me.transferSelected();
                }
            }
        });

        Ext.apply(this, {
            layout: 'fit',
            items: [contacts],
            bbar: [
                buttonAdd, "->", buttonTransfer
            ]
        });

        this.callParent(arguments);
    },
    transferSelected: function () {
        var me = this;
        if (!me.selectedRecord) {
            return;
        }
        me.showEmails(me.selectedRecord, function (displayName, email) {
            me.fireEvent('invite', email, displayName);
        });
    },
    showEmails: function (contactRecord, callback) {
        var details = contactRecord.data.ContactDetails;

        if (!details) {
            Ext.Msg.alert(MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.failure.message'));
            return;
        }

        var detailsStore = Ext.create('Ext.data.Store', {
            filters: [{
                property: 'TypeName', value: 'email'
            }],
            fields: ['TypeName', 'Value', 'Level1Name', 'Level2Name'],
            data: {
                'items': details
            },
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        if (detailsStore.getCount() > 1) {
            var btnOk = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.button.positive.label');
            var btnCancel = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.button.negative.label');
            var title = MedwebCloud.getMsg('medweb.view.invitationpanel.action.show.window.title');
            var email = MedwebCloud.getMsg('medweb.user.email.label')
            var w = new Ext.Window({
                modal: true,
                title: title,
                height: 300,
                width: 300,
                layout: 'fit',
                border: false,
                items: [{
                    xtype: 'grid',
                    store: detailsStore,
                    columns: [
                        { text: 'Email', dataIndex: 'Value', flex: 1 }
                    ],
                    listeners: {
                        celldblclick: function (grid, td, cellIndex, record, tr, rowIndex, e, eOpts) {
                            callback(contactRecord.get('DisplayName'), record.get('Value'));
                            w.close();
                        }
                    }
                }],
                buttons: [{
                    text: btnOk,
                    handler: function () {
                        var records = w.down('.grid').getSelectionModel().getSelection();
                        if (!records || records.length == 0) {
                            return;
                        }
                        w.close();
                        callback(contactRecord.get('DisplayName'), records[0].get('Value'));
                    }
                }, {
                    text: btnCancel,
                    handler: function () {
                        w.close();
                    }
                }]
            });
            w.show();
        } else {
            callback(contactRecord.get('DisplayName'), detailsStore.getAt(0).get('Value'));
        }
    }
});