
/*
 * Title: TheRest Client for JavaScript
 * Version: 2.8.8
 * Copyright Frozen Mountain Software 2011+
 */

(function(name, dependencies, definition) {
    if (typeof define === 'function' && define.amd) { // AMD/RequireJS
        define(name, dependencies, definition);
    } else if (typeof exports === 'object') { // Node/CommonJS
        for (var i = 0; i < dependencies.length; i++) {
            require('./' + dependencies[i]);
        }
        module.exports = definition();
    } else {
        this[name] = definition();
    }
}('fm.therest', ['fm'], function() {

if (typeof global !== 'undefined' && !global.window) { global.window = global; global.document = { cookie: '' }; }

if (!window.fm) { throw new Error("fm must be loaded before fm.therest."); }

if (!window.fm.therest) { window.fm.therest = {}; }

fm.therest.getVersion = function() {
  return '2.8.8';
};



/*<span id='cls-fm.therest.restFormat'>&nbsp;</span> */

/**
@class fm.therest.restFormat
 <div>
 The format of a REST request/response.
 </div>

@extends fm.enum
 */
fm.therest.restFormat = {

  /*<span id='prop-fm.therest.restFormat-None'>&nbsp;</span> */

  /**
  	 <div>
  	 No data in the request/response.
  	 </div>
  
  	@field None
  	@type {fm.therest.restFormat}
   */
  None: 1,

  /*<span id='prop-fm.therest.restFormat-Json'>&nbsp;</span> */

  /**
  	 <div>
  	 JSON data in the request/response.
  	 </div>
  
  	@field Json
  	@type {fm.therest.restFormat}
   */
  Json: 2,

  /*<span id='prop-fm.therest.restFormat-Binary'>&nbsp;</span> */

  /**
  	 <div>
  	 Binary data in the request/response.
  	 </div>
  
  	@field Binary
  	@type {fm.therest.restFormat}
   */
  Binary: 3
};



/*<span id='cls-fm.therest.baseEventArgs'>&nbsp;</span> */

/**
@class fm.therest.baseEventArgs
 <div>
 Base arguments for <see cref="fm.therest.baseEventArgs.client">fm.therest.baseEventArgs.client</see>-triggered events.
 </div>

@extends fm.object
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseEventArgs = (function(superClass) {
  extend(baseEventArgs, superClass);

  baseEventArgs.prototype._cancel = false;

  baseEventArgs.prototype._client = null;

  function baseEventArgs() {
    this.setClient = bind(this.setClient, this);
    this.setCancel = bind(this.setCancel, this);
    this.getClient = bind(this.getClient, this);
    this.getCancel = bind(this.getCancel, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseEventArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseEventArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseEventArgs-getCancel'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets whether or not to cancel processing of the request.
  	 If set to <c>true</c>, additional request processing will be cancelled.
  	 Defaults to <c>false</c>.
  	 </div>
  
  	@function getCancel
  	@return {Boolean}
   */

  baseEventArgs.prototype.getCancel = function() {
    return this._cancel;
  };


  /*<span id='method-fm.therest.baseEventArgs-getClient'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the <see cref="fm.therest.baseEventArgs.client">fm.therest.baseEventArgs.client</see> triggering the event.
  	 </div>
  
  	@function getClient
  	@return {fm.therest.client}
   */

  baseEventArgs.prototype.getClient = function() {
    return this._client;
  };


  /*<span id='method-fm.therest.baseEventArgs-setCancel'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets whether or not to cancel processing of the request.
  	 If set to <c>true</c>, additional request processing will be cancelled.
  	 Defaults to <c>false</c>.
  	 </div>
  
  	@function setCancel
  	@param {Boolean} value
  	@return {void}
   */

  baseEventArgs.prototype.setCancel = function() {
    var value;
    value = arguments[0];
    return this._cancel = value;
  };


  /*<span id='method-fm.therest.baseEventArgs-setClient'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the <see cref="fm.therest.baseEventArgs.client">fm.therest.baseEventArgs.client</see> triggering the event.
  	 </div>
  
  	@function setClient
  	@param {fm.therest.client} value
  	@return {void}
   */

  baseEventArgs.prototype.setClient = function() {
    var value;
    value = arguments[0];
    return this._client = value;
  };

  return baseEventArgs;

})(fm.object);



/*<span id='cls-fm.therest.baseAfterEventArgs'>&nbsp;</span> */

/**
@class fm.therest.baseAfterEventArgs
 <div>
 Base arguments for <see cref="fm.therest.client">fm.therest.client</see> events that occur
 after a response is received.
 </div>

@extends fm.therest.baseEventArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseAfterEventArgs = (function(superClass) {
  extend(baseAfterEventArgs, superClass);

  baseAfterEventArgs.prototype._exception = null;

  baseAfterEventArgs.prototype._responseHeaders = null;

  baseAfterEventArgs.prototype._responseJson = null;

  function baseAfterEventArgs() {
    this.setResponse = bind(this.setResponse, this);
    this.getResponse = bind(this.getResponse, this);
    this.setResponseJson = bind(this.setResponseJson, this);
    this.setResponseHeaders = bind(this.setResponseHeaders, this);
    this.setException = bind(this.setException, this);
    this.getResponseJson = bind(this.getResponseJson, this);
    this.getResponseHeaders = bind(this.getResponseHeaders, this);
    this.getException = bind(this.getException, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseAfterEventArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseAfterEventArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseAfterEventArgs-getException'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the exception generated while completing the request, if any.
  	 Will be <c>null</c> if no exception was generated.
  	 </div>
  
  	@function getException
  	@return {Error}
   */

  baseAfterEventArgs.prototype.getException = function() {
    return this._exception;
  };


  /*<span id='method-fm.therest.baseAfterEventArgs-getResponseHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the response headers returned by the server.
  	 </div>
  
  	@function getResponseHeaders
  	@return {fm.nameValueCollection}
   */

  baseAfterEventArgs.prototype.getResponseHeaders = function() {
    return this._responseHeaders;
  };


  /*<span id='method-fm.therest.baseAfterEventArgs-getResponseJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the response returned by the server (JSON format).
  	 </div>
  
  	@function getResponseJson
  	@return {String}
   */

  baseAfterEventArgs.prototype.getResponseJson = function() {
    return this._responseJson;
  };


  /*<span id='method-fm.therest.baseAfterEventArgs-setException'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the exception generated while completing the request, if any.
  	 Will be <c>null</c> if no exception was generated.
  	 </div>
  
  	@function setException
  	@param {Error} value
  	@return {void}
   */

  baseAfterEventArgs.prototype.setException = function() {
    var value;
    value = arguments[0];
    return this._exception = value;
  };


  /*<span id='method-fm.therest.baseAfterEventArgs-setResponseHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the response headers returned by the server.
  	 </div>
  
  	@function setResponseHeaders
  	@param {fm.nameValueCollection} value
  	@return {void}
   */

  baseAfterEventArgs.prototype.setResponseHeaders = function() {
    var value;
    value = arguments[0];
    return this._responseHeaders = value;
  };


  /*<span id='method-fm.therest.baseAfterEventArgs-setResponseJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the response returned by the server (JSON format).
  	 </div>
  
  	@function setResponseJson
  	@param {String} value
  	@return {void}
   */

  baseAfterEventArgs.prototype.setResponseJson = function() {
    var value;
    value = arguments[0];
    return this._responseJson = value;
  };

  baseAfterEventArgs.prototype.getResponse = function() {
    return fm.json.deserialize(this.getResponseJson.apply(this, arguments));
  };

  baseAfterEventArgs.prototype.setResponse = function() {
    var response;
    response = arguments[0];
    arguments[arguments.length - 1] = fm.json.serialize(arguments[arguments.length - 1]);
    return this.setResponseJson.apply(this, arguments);
  };

  return baseAfterEventArgs;

})(fm.therest.baseEventArgs);



/*<span id='cls-fm.therest.baseAfterEventArgsGeneric'>&nbsp;</span> */

/**
@class fm.therest.baseAfterEventArgsGeneric
 <div>
 Generic base arguments for <see cref="fm.therest.client">fm.therest.client</see> events that occur
 after a response is received.
 </div>

@extends fm.therest.baseAfterEventArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseAfterEventArgsGeneric = (function(superClass) {
  extend(baseAfterEventArgsGeneric, superClass);

  baseAfterEventArgsGeneric.prototype._methodArgs = null;

  function baseAfterEventArgsGeneric() {
    this.setMethodArgs = bind(this.setMethodArgs, this);
    this.getMethodArgs = bind(this.getMethodArgs, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseAfterEventArgsGeneric.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseAfterEventArgsGeneric.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseAfterEventArgsGeneric-getMethodArgs'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the original arguments passed into the client method.
  	 </div>
  
  	@function getMethodArgs
  	@return {fm.object}
   */

  baseAfterEventArgsGeneric.prototype.getMethodArgs = function() {
    return this._methodArgs;
  };


  /*<span id='method-fm.therest.baseAfterEventArgsGeneric-setMethodArgs'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the original arguments passed into the client method.
  	 </div>
  
  	@function setMethodArgs
  	@param {fm.object} value
  	@return {void}
   */

  baseAfterEventArgsGeneric.prototype.setMethodArgs = function() {
    var value;
    value = arguments[0];
    return this._methodArgs = value;
  };

  return baseAfterEventArgsGeneric;

})(fm.therest.baseAfterEventArgs);



/*<span id='cls-fm.therest.baseBeforeEventArgs'>&nbsp;</span> */

/**
@class fm.therest.baseBeforeEventArgs
 <div>
 Base arguments for <see cref="fm.therest.client">fm.therest.client</see> events that occur
 before a request is sent.
 </div>

@extends fm.therest.baseEventArgs
 */
var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseBeforeEventArgs = (function(superClass) {
  extend(baseBeforeEventArgs, superClass);

  function baseBeforeEventArgs() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseBeforeEventArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseBeforeEventArgs.__super__.constructor.call(this);
    return instance;
  }

  return baseBeforeEventArgs;

})(fm.therest.baseEventArgs);



/*<span id='cls-fm.therest.baseBeforeEventArgsGeneric'>&nbsp;</span> */

/**
@class fm.therest.baseBeforeEventArgsGeneric
 <div>
 Generic base arguments for <see cref="fm.therest.client">fm.therest.client</see> events that occur
 before a request is sent.
 </div>

@extends fm.therest.baseBeforeEventArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseBeforeEventArgsGeneric = (function(superClass) {
  extend(baseBeforeEventArgsGeneric, superClass);

  baseBeforeEventArgsGeneric.prototype._methodArgs = null;

  function baseBeforeEventArgsGeneric() {
    this.setMethodArgs = bind(this.setMethodArgs, this);
    this.getMethodArgs = bind(this.getMethodArgs, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseBeforeEventArgsGeneric.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseBeforeEventArgsGeneric.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseBeforeEventArgsGeneric-getMethodArgs'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the original arguments passed into the client method.
  	 </div>
  
  	@function getMethodArgs
  	@return {fm.object}
   */

  baseBeforeEventArgsGeneric.prototype.getMethodArgs = function() {
    return this._methodArgs;
  };


  /*<span id='method-fm.therest.baseBeforeEventArgsGeneric-setMethodArgs'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the original arguments passed into the client method.
  	 </div>
  
  	@function setMethodArgs
  	@param {fm.object} value
  	@return {void}
   */

  baseBeforeEventArgsGeneric.prototype.setMethodArgs = function() {
    var value;
    value = arguments[0];
    return this._methodArgs = value;
  };

  return baseBeforeEventArgsGeneric;

})(fm.therest.baseBeforeEventArgs);



/*<span id='cls-fm.therest.extensible'>&nbsp;</span> */

/**
@class fm.therest.extensible
 <div>
 <p>
 Base class that defines methods which support dynamic property getting/setting.
 </p>
 <p>
 Classes which inherit from <see cref="fm.therest.extensible">fm.therest.extensible</see> can store
 dynamic property values for local read/write access. This can aid greatly in the
 development of third-party extensions. Custom information can be stored
 with method arguments in the "before" event and read out again for further
 processing in the "after" event.
 </p>
 </div>

@extends fm.dynamic
 */
var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.extensible = (function(superClass) {
  extend(extensible, superClass);

  function extensible() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = extensible.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = extensible.__super__.constructor.call(this);
    return instance;
  }

  extensible.validatePath = function() {
    var path;
    path = arguments[0];
    return (!fm.global.equals(path, null)) && fm.stringExtensions.startsWith(path, "/");
  };

  return extensible;

})(fm.dynamic);



/*<span id='cls-fm.therest.baseOutputArgs'>&nbsp;</span> */

/**
@class fm.therest.baseOutputArgs
 <div>
 Base output arguments for <see cref="fm.therest.baseOutputArgs.client">fm.therest.baseOutputArgs.client</see> methods.
 </div>

@extends fm.therest.extensible
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseOutputArgs = (function(superClass) {
  extend(baseOutputArgs, superClass);

  baseOutputArgs.prototype._client = null;

  baseOutputArgs.prototype._requestDataBytes = null;

  baseOutputArgs.prototype._requestDataJson = null;

  baseOutputArgs.prototype._requestFormat = null;

  baseOutputArgs.prototype._requestLanguages = null;

  baseOutputArgs.prototype._requestMethod = null;

  baseOutputArgs.prototype._requestPath = null;

  baseOutputArgs.prototype._requestRange = null;

  function baseOutputArgs() {
    this.setRequestData = bind(this.setRequestData, this);
    this.getRequestData = bind(this.getRequestData, this);
    this.setRequestRange = bind(this.setRequestRange, this);
    this.setRequestPath = bind(this.setRequestPath, this);
    this.setRequestMethod = bind(this.setRequestMethod, this);
    this.setRequestLanguages = bind(this.setRequestLanguages, this);
    this.setRequestLanguage = bind(this.setRequestLanguage, this);
    this.setRequestFormat = bind(this.setRequestFormat, this);
    this.setRequestDataJson = bind(this.setRequestDataJson, this);
    this.setRequestDataBytes = bind(this.setRequestDataBytes, this);
    this.setClient = bind(this.setClient, this);
    this.getRequestRange = bind(this.getRequestRange, this);
    this.getRequestPath = bind(this.getRequestPath, this);
    this.getRequestMethod = bind(this.getRequestMethod, this);
    this.getRequestLanguages = bind(this.getRequestLanguages, this);
    this.getRequestLanguage = bind(this.getRequestLanguage, this);
    this.getRequestFormat = bind(this.getRequestFormat, this);
    this.getRequestDataJson = bind(this.getRequestDataJson, this);
    this.getRequestDataBytes = bind(this.getRequestDataBytes, this);
    this.getClient = bind(this.getClient, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseOutputArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseOutputArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseOutputArgs-getClient'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the active <see cref="fm.therest.baseOutputArgs.client">fm.therest.baseOutputArgs.client</see> who made the request.
  	 </div>
  
  	@function getClient
  	@return {fm.therest.client}
   */

  baseOutputArgs.prototype.getClient = function() {
    return this._client;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the binary data sent to the server.
  	 </div>
  
  	@function getRequestDataBytes
  	@return {fm.array}
   */

  baseOutputArgs.prototype.getRequestDataBytes = function() {
    return this._requestDataBytes;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the JSON data sent to the server.
  	 </div>
  
  	@function getRequestDataJson
  	@return {String}
   */

  baseOutputArgs.prototype.getRequestDataJson = function() {
    return this._requestDataJson;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the format of the request sent to the server.
  	 </div>
  
  	@function getRequestFormat
  	@return {fm.therest.restFormat}
   */

  baseOutputArgs.prototype.getRequestFormat = function() {
    return this._requestFormat;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the preferred language sent to the server.
  	 </div>
  
  	@function getRequestLanguage
  	@return {fm.therest.language}
   */

  baseOutputArgs.prototype.getRequestLanguage = function() {
    if ((fm.global.equals(this.getRequestLanguages(), null)) || (fm.global.equals(this.getRequestLanguages().length, 0))) {
      return null;
    }
    return this.getRequestLanguages()[0];
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestLanguages'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the languages sent to the server.
  	 </div>
  
  	@function getRequestLanguages
  	@return {fm.array}
   */

  baseOutputArgs.prototype.getRequestLanguages = function() {
    return this._requestLanguages;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestMethod'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the method type sent to the server.
  	 </div>
  
  	@function getRequestMethod
  	@return {fm.httpMethod}
   */

  baseOutputArgs.prototype.getRequestMethod = function() {
    return this._requestMethod;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestPath'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the path of the request.
  	 </div>
  
  	@function getRequestPath
  	@return {String}
   */

  baseOutputArgs.prototype.getRequestPath = function() {
    return this._requestPath;
  };


  /*<span id='method-fm.therest.baseOutputArgs-getRequestRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the range sent to the server.
  	 </div>
  
  	@function getRequestRange
  	@return {fm.therest.range}
   */

  baseOutputArgs.prototype.getRequestRange = function() {
    return this._requestRange;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setClient'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the active <see cref="fm.therest.baseOutputArgs.client">fm.therest.baseOutputArgs.client</see> who made the request.
  	 </div>
  
  	@function setClient
  	@param {fm.therest.client} value
  	@return {void}
   */

  baseOutputArgs.prototype.setClient = function() {
    var value;
    value = arguments[0];
    return this._client = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the binary data sent to the server.
  	 </div>
  
  	@function setRequestDataBytes
  	@param {fm.array} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestDataBytes = function() {
    var value;
    value = arguments[0];
    return this._requestDataBytes = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the JSON data sent to the server.
  	 </div>
  
  	@function setRequestDataJson
  	@param {String} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestDataJson = function() {
    var value;
    value = arguments[0];
    return this._requestDataJson = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the format of the request sent to the server.
  	 </div>
  
  	@function setRequestFormat
  	@param {fm.therest.restFormat} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestFormat = function() {
    var value;
    value = arguments[0];
    return this._requestFormat = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the preferred language sent to the server.
  	 </div>
  
  	@function setRequestLanguage
  	@param {fm.therest.language} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestLanguage = function() {
    var value;
    value = arguments[0];
    if (fm.global.equals(value, null)) {
      return this.setRequestLanguages(null);
    } else {
      return this.setRequestLanguages([value]);
    }
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestLanguages'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the languages sent to the server.
  	 </div>
  
  	@function setRequestLanguages
  	@param {fm.array} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestLanguages = function() {
    var value;
    value = arguments[0];
    return this._requestLanguages = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestMethod'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the method type sent to the server.
  	 </div>
  
  	@function setRequestMethod
  	@param {fm.httpMethod} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestMethod = function() {
    var value;
    value = arguments[0];
    return this._requestMethod = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestPath'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the path of the request.
  	 </div>
  
  	@function setRequestPath
  	@param {String} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestPath = function() {
    var value;
    value = arguments[0];
    return this._requestPath = value;
  };


  /*<span id='method-fm.therest.baseOutputArgs-setRequestRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the range sent to the server.
  	 </div>
  
  	@function setRequestRange
  	@param {fm.therest.range} value
  	@return {void}
   */

  baseOutputArgs.prototype.setRequestRange = function() {
    var value;
    value = arguments[0];
    return this._requestRange = value;
  };

  baseOutputArgs.prototype.getRequestData = function() {
    return fm.json.deserialize(this.getRequestDataJson.apply(this, arguments));
  };

  baseOutputArgs.prototype.setRequestData = function() {
    var requestData;
    requestData = arguments[0];
    arguments[arguments.length - 1] = fm.json.serialize(arguments[arguments.length - 1]);
    return this.setRequestDataJson.apply(this, arguments);
  };

  return baseOutputArgs;

})(fm.therest.extensible);



/*<span id='cls-fm.therest.sendCompleteArgs'>&nbsp;</span> */

/**
@class fm.therest.sendCompleteArgs
 <div>
 Arguments for <see cref="fm.therest.client">fm.therest.client</see> send complete callbacks.
 </div>

@extends fm.therest.baseOutputArgs
 */
var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.sendCompleteArgs = (function(superClass) {
  extend(sendCompleteArgs, superClass);

  function sendCompleteArgs() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = sendCompleteArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = sendCompleteArgs.__super__.constructor.call(this);
    return instance;
  }

  return sendCompleteArgs;

})(fm.therest.baseOutputArgs);



/*<span id='cls-fm.therest.sendFailureArgs'>&nbsp;</span> */

/**
@class fm.therest.sendFailureArgs
 <div>
 Arguments for <see cref="fm.therest.client">fm.therest.client</see> send failure callbacks.
 </div>

@extends fm.therest.baseOutputArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.sendFailureArgs = (function(superClass) {
  extend(sendFailureArgs, superClass);

  sendFailureArgs.prototype._exception = null;

  sendFailureArgs.prototype._responseStatusCode = 0;

  function sendFailureArgs() {
    this.setResponseStatusCode = bind(this.setResponseStatusCode, this);
    this.setException = bind(this.setException, this);
    this.getResponseStatusCode = bind(this.getResponseStatusCode, this);
    this.getException = bind(this.getException, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = sendFailureArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = sendFailureArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.sendFailureArgs-getException'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the exception generated while completing the request.
  	 </div>
  
  	@function getException
  	@return {Error}
   */

  sendFailureArgs.prototype.getException = function() {
    return this._exception;
  };


  /*<span id='method-fm.therest.sendFailureArgs-getResponseStatusCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the status code received from the server.
  	 </div>
  
  	@function getResponseStatusCode
  	@return {Integer}
   */

  sendFailureArgs.prototype.getResponseStatusCode = function() {
    return this._responseStatusCode;
  };


  /*<span id='method-fm.therest.sendFailureArgs-setException'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the exception generated while completing the request.
  	 </div>
  
  	@function setException
  	@param {Error} value
  	@return {void}
   */

  sendFailureArgs.prototype.setException = function() {
    var value;
    value = arguments[0];
    return this._exception = value;
  };


  /*<span id='method-fm.therest.sendFailureArgs-setResponseStatusCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the status code received from the server.
  	 </div>
  
  	@function setResponseStatusCode
  	@param {Integer} value
  	@return {void}
   */

  sendFailureArgs.prototype.setResponseStatusCode = function() {
    var value;
    value = arguments[0];
    return this._responseStatusCode = value;
  };

  return sendFailureArgs;

})(fm.therest.baseOutputArgs);



/*<span id='cls-fm.therest.baseInputArgs'>&nbsp;</span> */

/**
@class fm.therest.baseInputArgs
 <div>
 Base input arguments for TheRest <see cref="fm.therest.client">fm.therest.client</see> methods.
 </div>

@extends fm.therest.extensible
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.baseInputArgs = (function(superClass) {
  extend(baseInputArgs, superClass);

  baseInputArgs.prototype._requestUrl = null;

  baseInputArgs.prototype._synchronous = null;

  function baseInputArgs() {
    this.setSynchronous = bind(this.setSynchronous, this);
    this.setRequestUrl = bind(this.setRequestUrl, this);
    this.getSynchronous = bind(this.getSynchronous, this);
    this.getRequestUrl = bind(this.getRequestUrl, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = baseInputArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = baseInputArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.baseInputArgs-getRequestUrl'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the absolute URL of the TheRest handler, typically
  	 ending with therest.ashx, to override the client-level setting.
  	 </div>
  
  	@function getRequestUrl
  	@return {String}
   */

  baseInputArgs.prototype.getRequestUrl = function() {
    return this._requestUrl;
  };


  /*<span id='method-fm.therest.baseInputArgs-getSynchronous'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets whether the request should be executed asynchronously.
  	 If <c>true</c>, the request will be executed synchronously.
  	 If <c>false</c>, the request will be executed asynchronously.
  	 If <c>null</c>, the request will be executed synchronously or asynchronously,
  	 depending on the value of <see cref="fm.therest.client.synchronous">fm.therest.client.synchronous</see>.
  	 Defaults to <c>null</c>.
  	 </div>
  
  	@function getSynchronous
  	@return {fm.nullable}
   */

  baseInputArgs.prototype.getSynchronous = function() {
    return this._synchronous;
  };


  /*<span id='method-fm.therest.baseInputArgs-setRequestUrl'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the absolute URL of the TheRest handler, typically
  	 ending with therest.ashx, to override the client-level setting.
  	 </div>
  
  	@function setRequestUrl
  	@param {String} value
  	@return {void}
   */

  baseInputArgs.prototype.setRequestUrl = function() {
    var value;
    value = arguments[0];
    return this._requestUrl = value;
  };


  /*<span id='method-fm.therest.baseInputArgs-setSynchronous'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets whether the request should be executed asynchronously.
  	 If <c>true</c>, the request will be executed synchronously.
  	 If <c>false</c>, the request will be executed asynchronously.
  	 If <c>null</c>, the request will be executed synchronously or asynchronously,
  	 depending on the value of <see cref="fm.therest.client.synchronous">fm.therest.client.synchronous</see>.
  	 Defaults to <c>null</c>.
  	 </div>
  
  	@function setSynchronous
  	@param {fm.nullable} value
  	@return {void}
   */

  baseInputArgs.prototype.setSynchronous = function() {
    var value;
    value = arguments[0];
    return this._synchronous = value;
  };

  return baseInputArgs;

})(fm.therest.extensible);



/*<span id='cls-fm.therest.sendSuccessArgs'>&nbsp;</span> */

/**
@class fm.therest.sendSuccessArgs
 <div>
 Arguments for <see cref="fm.therest.client">fm.therest.client</see> send success callbacks.
 </div>

@extends fm.therest.baseOutputArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.sendSuccessArgs = (function(superClass) {
  extend(sendSuccessArgs, superClass);

  sendSuccessArgs.prototype._responseDataBytes = null;

  sendSuccessArgs.prototype._responseDataJson = null;

  sendSuccessArgs.prototype._responseFormat = null;

  sendSuccessArgs.prototype._responseHeaders = null;

  sendSuccessArgs.prototype._responseLanguage = null;

  sendSuccessArgs.prototype._responseRange = null;

  sendSuccessArgs.prototype._responseStatusCode = 0;

  function sendSuccessArgs() {
    this.setResponseData = bind(this.setResponseData, this);
    this.getResponseData = bind(this.getResponseData, this);
    this.setResponseStatusCode = bind(this.setResponseStatusCode, this);
    this.setResponseRange = bind(this.setResponseRange, this);
    this.setResponseLanguage = bind(this.setResponseLanguage, this);
    this.setResponseHeaders = bind(this.setResponseHeaders, this);
    this.setResponseFormat = bind(this.setResponseFormat, this);
    this.setResponseDataJson = bind(this.setResponseDataJson, this);
    this.setResponseDataBytes = bind(this.setResponseDataBytes, this);
    this.getResponseStatusCode = bind(this.getResponseStatusCode, this);
    this.getResponseRange = bind(this.getResponseRange, this);
    this.getResponseLanguage = bind(this.getResponseLanguage, this);
    this.getResponseHeaders = bind(this.getResponseHeaders, this);
    this.getResponseFormat = bind(this.getResponseFormat, this);
    this.getResponseDataJson = bind(this.getResponseDataJson, this);
    this.getResponseDataBytes = bind(this.getResponseDataBytes, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = sendSuccessArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = sendSuccessArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the binary data received from the server.
  	 </div>
  
  	@function getResponseDataBytes
  	@return {fm.array}
   */

  sendSuccessArgs.prototype.getResponseDataBytes = function() {
    return this._responseDataBytes;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the JSON data received from the server.
  	 </div>
  
  	@function getResponseDataJson
  	@return {String}
   */

  sendSuccessArgs.prototype.getResponseDataJson = function() {
    return this._responseDataJson;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the format of the response received from the server.
  	 </div>
  
  	@function getResponseFormat
  	@return {fm.therest.restFormat}
   */

  sendSuccessArgs.prototype.getResponseFormat = function() {
    return this._responseFormat;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the headers received from the server.
  	 </div>
  
  	@function getResponseHeaders
  	@return {fm.nameValueCollection}
   */

  sendSuccessArgs.prototype.getResponseHeaders = function() {
    return this._responseHeaders;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets language received from the server.
  	 </div>
  
  	@function getResponseLanguage
  	@return {fm.therest.language}
   */

  sendSuccessArgs.prototype.getResponseLanguage = function() {
    return this._responseLanguage;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the range received from the server.
  	 </div>
  
  	@function getResponseRange
  	@return {fm.therest.range}
   */

  sendSuccessArgs.prototype.getResponseRange = function() {
    return this._responseRange;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-getResponseStatusCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the status code received from the server.
  	 </div>
  
  	@function getResponseStatusCode
  	@return {Integer}
   */

  sendSuccessArgs.prototype.getResponseStatusCode = function() {
    return this._responseStatusCode;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the binary data received from the server.
  	 </div>
  
  	@function setResponseDataBytes
  	@param {fm.array} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseDataBytes = function() {
    var value;
    value = arguments[0];
    return this._responseDataBytes = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the JSON data received from the server.
  	 </div>
  
  	@function setResponseDataJson
  	@param {String} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseDataJson = function() {
    var value;
    value = arguments[0];
    return this._responseDataJson = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the format of the response received from the server.
  	 </div>
  
  	@function setResponseFormat
  	@param {fm.therest.restFormat} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseFormat = function() {
    var value;
    value = arguments[0];
    return this._responseFormat = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the headers received from the server.
  	 </div>
  
  	@function setResponseHeaders
  	@param {fm.nameValueCollection} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseHeaders = function() {
    var value;
    value = arguments[0];
    return this._responseHeaders = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets language received from the server.
  	 </div>
  
  	@function setResponseLanguage
  	@param {fm.therest.language} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseLanguage = function() {
    var value;
    value = arguments[0];
    return this._responseLanguage = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the range received from the server.
  	 </div>
  
  	@function setResponseRange
  	@param {fm.therest.range} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseRange = function() {
    var value;
    value = arguments[0];
    return this._responseRange = value;
  };


  /*<span id='method-fm.therest.sendSuccessArgs-setResponseStatusCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the status code received from the server.
  	 </div>
  
  	@function setResponseStatusCode
  	@param {Integer} value
  	@return {void}
   */

  sendSuccessArgs.prototype.setResponseStatusCode = function() {
    var value;
    value = arguments[0];
    return this._responseStatusCode = value;
  };

  sendSuccessArgs.prototype.getResponseData = function() {
    return fm.json.deserialize(this.getResponseDataJson.apply(this, arguments));
  };

  sendSuccessArgs.prototype.setResponseData = function() {
    var responseData;
    responseData = arguments[0];
    arguments[arguments.length - 1] = fm.json.serialize(arguments[arguments.length - 1]);
    return this.setResponseDataJson.apply(this, arguments);
  };

  return sendSuccessArgs;

})(fm.therest.baseOutputArgs);



/*<span id='cls-fm.therest.afterSendArgs'>&nbsp;</span> */

/**
@class fm.therest.afterSendArgs
 <div>
 Arguments for <see cref="fm.therest.client.addOnAfterSend">fm.therest.client.addOnAfterSend</see>.
 </div>

@extends fm.therest.baseAfterEventArgsGeneric
 */
var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.afterSendArgs = (function(superClass) {
  extend(afterSendArgs, superClass);

  function afterSendArgs() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = afterSendArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = afterSendArgs.__super__.constructor.call(this);
    return instance;
  }

  return afterSendArgs;

})(fm.therest.baseAfterEventArgsGeneric);



/*<span id='cls-fm.therest.beforeSendArgs'>&nbsp;</span> */

/**
@class fm.therest.beforeSendArgs
 <div>
 Arguments for <see cref="fm.therest.client.addOnBeforeSend">fm.therest.client.addOnBeforeSend</see>.
 </div>

@extends fm.therest.baseBeforeEventArgsGeneric
 */
var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.beforeSendArgs = (function(superClass) {
  extend(beforeSendArgs, superClass);

  function beforeSendArgs() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = beforeSendArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = beforeSendArgs.__super__.constructor.call(this);
    return instance;
  }

  return beforeSendArgs;

})(fm.therest.baseBeforeEventArgsGeneric);



/*<span id='cls-fm.therest.sendArgs'>&nbsp;</span> */

/**
@class fm.therest.sendArgs
 <div>
 Arguments for <see cref="fm.therest.client">fm.therest.client</see> send requests.
 </div>

@extends fm.therest.baseInputArgs
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.sendArgs = (function(superClass) {
  extend(sendArgs, superClass);

  sendArgs.prototype.__dataJson = null;

  sendArgs.prototype.__headers = null;

  sendArgs.prototype.__path = null;

  sendArgs.prototype._dataBytes = null;

  sendArgs.prototype._format = null;

  sendArgs.prototype._languages = null;

  sendArgs.prototype._method = null;

  sendArgs.prototype._onComplete = null;

  sendArgs.prototype._onFailure = null;

  sendArgs.prototype._onSuccess = null;

  sendArgs.prototype._range = null;


  /*<span id='method-fm.therest.sendArgs-fm.therest.sendArgs'>&nbsp;</span> */


  /**
  	 <div>
  	 Initializes a new instance of the <see cref="fm.therest.sendArgs">fm.therest.sendArgs</see> class.
  	 </div>
  	@function fm.therest.sendArgs
  	@param {fm.httpMethod} method The method to use when sending.
  	@param {String} path The path to use when sending.
  	@return {}
   */

  function sendArgs() {
    this.setData = bind(this.setData, this);
    this.getData = bind(this.getData, this);
    this.setRange = bind(this.setRange, this);
    this.setPath = bind(this.setPath, this);
    this.setOnSuccess = bind(this.setOnSuccess, this);
    this.setOnFailure = bind(this.setOnFailure, this);
    this.setOnComplete = bind(this.setOnComplete, this);
    this.setMethod = bind(this.setMethod, this);
    this.setLanguages = bind(this.setLanguages, this);
    this.setLanguage = bind(this.setLanguage, this);
    this.setHeaders = bind(this.setHeaders, this);
    this.setFormat = bind(this.setFormat, this);
    this.setDataJson = bind(this.setDataJson, this);
    this.setDataBytes = bind(this.setDataBytes, this);
    this.getRange = bind(this.getRange, this);
    this.getPath = bind(this.getPath, this);
    this.getOnSuccess = bind(this.getOnSuccess, this);
    this.getOnFailure = bind(this.getOnFailure, this);
    this.getOnComplete = bind(this.getOnComplete, this);
    this.getMethod = bind(this.getMethod, this);
    this.getLanguages = bind(this.getLanguages, this);
    this.getLanguage = bind(this.getLanguage, this);
    this.getHeaders = bind(this.getHeaders, this);
    this.getFormat = bind(this.getFormat, this);
    this.getDataJson = bind(this.getDataJson, this);
    this.getDataBytes = bind(this.getDataBytes, this);
    var instance, method, path;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = sendArgs.__super__.constructor.call(this);
      this.setFormat(fm.therest.restFormat.None);
      this.__headers = new fm.nameValueCollection();
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    if (arguments.length === 2) {
      method = arguments[0];
      path = arguments[1];
      sendArgs.call(this);
      this.setMethod(method);
      this.setPath(path);
      return instance;
    }
    if (arguments.length === 0) {
      instance = sendArgs.__super__.constructor.call(this);
      this.setFormat(fm.therest.restFormat.None);
      this.__headers = new fm.nameValueCollection();
      return instance;
    }
  }


  /*<span id='method-fm.therest.sendArgs-getDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the data to send to the server, in binary format.
  	 </div>
  
  	@function getDataBytes
  	@return {fm.array}
   */

  sendArgs.prototype.getDataBytes = function() {
    return this._dataBytes;
  };


  /*<span id='method-fm.therest.sendArgs-getDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the data to send to the server, in JSON format.
  	 Must be valid JSON.
  	 </div>
  
  	@function getDataJson
  	@return {String}
   */

  sendArgs.prototype.getDataJson = function() {
    return this.__dataJson;
  };


  /*<span id='method-fm.therest.sendArgs-getFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the format of the request.
  	 </div>
  
  	@function getFormat
  	@return {fm.therest.restFormat}
   */

  sendArgs.prototype.getFormat = function() {
    return this._format;
  };


  /*<span id='method-fm.therest.sendArgs-getHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets additional headers to include in the request.
  	 </div>
  
  	@function getHeaders
  	@return {fm.nameValueCollection}
   */

  sendArgs.prototype.getHeaders = function() {
    return this.__headers;
  };


  /*<span id='method-fm.therest.sendArgs-getLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the preferred language to send with the query.
  	 </div>
  
  	@function getLanguage
  	@return {fm.therest.language}
   */

  sendArgs.prototype.getLanguage = function() {
    if ((fm.global.equals(this.getLanguages(), null)) || (fm.global.equals(this.getLanguages().length, 0))) {
      return null;
    }
    return this.getLanguages()[0];
  };


  /*<span id='method-fm.therest.sendArgs-getLanguages'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the languages to send with the query.
  	 </div>
  
  	@function getLanguages
  	@return {fm.array}
   */

  sendArgs.prototype.getLanguages = function() {
    return this._languages;
  };


  /*<span id='method-fm.therest.sendArgs-getMethod'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the method to use when sending.
  	 </div>
  
  	@function getMethod
  	@return {fm.httpMethod}
   */

  sendArgs.prototype.getMethod = function() {
    return this._method;
  };


  /*<span id='method-fm.therest.sendArgs-getOnComplete'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the callback to invoke after <see cref="fm.therest.sendArgs.onSuccess">fm.therest.sendArgs.onSuccess</see> or <see cref="fm.therest.sendArgs.onFailure">fm.therest.sendArgs.onFailure</see>.
  	 </div>
  
  	@function getOnComplete
  	@return {fm.singleAction}
   */

  sendArgs.prototype.getOnComplete = function() {
    return this._onComplete;
  };


  /*<span id='method-fm.therest.sendArgs-getOnFailure'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the callback to invoke if the request fails.
  	 </div>
  
  	@function getOnFailure
  	@return {fm.singleAction}
   */

  sendArgs.prototype.getOnFailure = function() {
    return this._onFailure;
  };


  /*<span id='method-fm.therest.sendArgs-getOnSuccess'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the callback to invoke if the request succeeds.
  	 </div>
  
  	@function getOnSuccess
  	@return {fm.singleAction}
   */

  sendArgs.prototype.getOnSuccess = function() {
    return this._onSuccess;
  };


  /*<span id='method-fm.therest.sendArgs-getPath'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the path to use when sending.
  	 </div>
  
  	@function getPath
  	@return {String}
   */

  sendArgs.prototype.getPath = function() {
    return this.__path;
  };


  /*<span id='method-fm.therest.sendArgs-getRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the range to send with the query.
  	 </div>
  
  	@function getRange
  	@return {fm.therest.range}
   */

  sendArgs.prototype.getRange = function() {
    return this._range;
  };


  /*<span id='method-fm.therest.sendArgs-setDataBytes'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the data to send to the server, in binary format.
  	 </div>
  
  	@function setDataBytes
  	@param {fm.array} value
  	@return {void}
   */

  sendArgs.prototype.setDataBytes = function() {
    var value;
    value = arguments[0];
    return this._dataBytes = value;
  };


  /*<span id='method-fm.therest.sendArgs-setDataJson'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the data to send to the server, in JSON format.
  	 Must be valid JSON.
  	 </div>
  
  	@function setDataJson
  	@param {String} value
  	@return {void}
   */

  sendArgs.prototype.setDataJson = function() {
    var value;
    value = arguments[0];
    if (!((fm.global.equals(value, null)) || fm.serializer.isValidJson(value))) {
      throw new Error("The value is not valid JSON.");
    }
    return this.__dataJson = value;
  };


  /*<span id='method-fm.therest.sendArgs-setFormat'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the format of the request.
  	 </div>
  
  	@function setFormat
  	@param {fm.therest.restFormat} value
  	@return {void}
   */

  sendArgs.prototype.setFormat = function() {
    var value;
    value = arguments[0];
    return this._format = value;
  };


  /*<span id='method-fm.therest.sendArgs-setHeaders'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets additional headers to include in the request.
  	 </div>
  
  	@function setHeaders
  	@param {fm.nameValueCollection} value
  	@return {void}
   */

  sendArgs.prototype.setHeaders = function() {
    var value;
    value = arguments[0];
    if (fm.global.equals(value, null)) {
      return this.__headers = new fm.nameValueCollection();
    } else {
      return this.__headers = value;
    }
  };


  /*<span id='method-fm.therest.sendArgs-setLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the preferred language to send with the query.
  	 </div>
  
  	@function setLanguage
  	@param {fm.therest.language} value
  	@return {void}
   */

  sendArgs.prototype.setLanguage = function() {
    var value;
    value = arguments[0];
    if (fm.global.equals(value, null)) {
      return this.setLanguages(null);
    } else {
      return this.setLanguages([value]);
    }
  };


  /*<span id='method-fm.therest.sendArgs-setLanguages'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the languages to send with the query.
  	 </div>
  
  	@function setLanguages
  	@param {fm.array} value
  	@return {void}
   */

  sendArgs.prototype.setLanguages = function() {
    var value;
    value = arguments[0];
    return this._languages = value;
  };


  /*<span id='method-fm.therest.sendArgs-setMethod'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the method to use when sending.
  	 </div>
  
  	@function setMethod
  	@param {fm.httpMethod} value
  	@return {void}
   */

  sendArgs.prototype.setMethod = function() {
    var value;
    value = arguments[0];
    return this._method = value;
  };


  /*<span id='method-fm.therest.sendArgs-setOnComplete'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the callback to invoke after <see cref="fm.therest.sendArgs.onSuccess">fm.therest.sendArgs.onSuccess</see> or <see cref="fm.therest.sendArgs.onFailure">fm.therest.sendArgs.onFailure</see>.
  	 </div>
  
  	@function setOnComplete
  	@param {fm.singleAction} value
  	@return {void}
   */

  sendArgs.prototype.setOnComplete = function() {
    var value;
    value = arguments[0];
    return this._onComplete = value;
  };


  /*<span id='method-fm.therest.sendArgs-setOnFailure'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the callback to invoke if the request fails.
  	 </div>
  
  	@function setOnFailure
  	@param {fm.singleAction} value
  	@return {void}
   */

  sendArgs.prototype.setOnFailure = function() {
    var value;
    value = arguments[0];
    return this._onFailure = value;
  };


  /*<span id='method-fm.therest.sendArgs-setOnSuccess'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the callback to invoke if the request succeeds.
  	 </div>
  
  	@function setOnSuccess
  	@param {fm.singleAction} value
  	@return {void}
   */

  sendArgs.prototype.setOnSuccess = function() {
    var value;
    value = arguments[0];
    return this._onSuccess = value;
  };


  /*<span id='method-fm.therest.sendArgs-setPath'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the path to use when sending.
  	 </div>
  
  	@function setPath
  	@param {String} value
  	@return {void}
   */

  sendArgs.prototype.setPath = function() {
    var value;
    value = arguments[0];
    if (!fm.therest.extensible.validatePath(value)) {
      throw new Error("Invalid path.");
    }
    return this.__path = value;
  };


  /*<span id='method-fm.therest.sendArgs-setRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the range to send with the query.
  	 </div>
  
  	@function setRange
  	@param {fm.therest.range} value
  	@return {void}
   */

  sendArgs.prototype.setRange = function() {
    var value;
    value = arguments[0];
    return this._range = value;
  };

  sendArgs.prototype.getData = function() {
    return fm.json.deserialize(this.getDataJson.apply(this, arguments));
  };

  sendArgs.prototype.setData = function() {
    var data;
    data = arguments[0];
    arguments[arguments.length - 1] = fm.json.serialize(arguments[arguments.length - 1]);
    return this.setDataJson.apply(this, arguments);
  };

  return sendArgs;

})(fm.therest.baseInputArgs);



/*<span id='cls-fm.therest.unhandledExceptionArgs'>&nbsp;</span> */

/**
@class fm.therest.unhandledExceptionArgs
 <div>
 Arguments for an unhandled exception.
 </div>

@extends fm.object
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.unhandledExceptionArgs = (function(superClass) {
  extend(unhandledExceptionArgs, superClass);

  unhandledExceptionArgs.prototype.__exception = null;

  unhandledExceptionArgs.prototype._handled = false;

  function unhandledExceptionArgs() {
    this.setHandled = bind(this.setHandled, this);
    this.getHandled = bind(this.getHandled, this);
    this.getException = bind(this.getException, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = unhandledExceptionArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = unhandledExceptionArgs.__super__.constructor.call(this);
    return instance;
  }


  /*<span id='method-fm.therest.unhandledExceptionArgs-getException'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the unhandled exception.
  	 </div>
  
  	@function getException
  	@return {Error}
   */

  unhandledExceptionArgs.prototype.getException = function() {
    return this.__exception;
  };


  /*<span id='method-fm.therest.unhandledExceptionArgs-getHandled'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets whether the exception has been
  	 appropriately handled. If set to <c>true</c>,
  	 then the exception will not be thrown.
  	 </div>
  
  	@function getHandled
  	@return {Boolean}
   */

  unhandledExceptionArgs.prototype.getHandled = function() {
    return this._handled;
  };


  /*<span id='method-fm.therest.unhandledExceptionArgs-setHandled'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets whether the exception has been
  	 appropriately handled. If set to <c>true</c>,
  	 then the exception will not be thrown.
  	 </div>
  
  	@function setHandled
  	@param {Boolean} value
  	@return {void}
   */

  unhandledExceptionArgs.prototype.setHandled = function() {
    var value;
    value = arguments[0];
    return this._handled = value;
  };

  return unhandledExceptionArgs;

})(fm.object);


var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.clientResponseArgs = (function(superClass) {
  extend(clientResponseArgs, superClass);

  clientResponseArgs.prototype._bytes = null;

  clientResponseArgs.prototype._exception = null;

  clientResponseArgs.prototype._format = null;

  clientResponseArgs.prototype._headers = null;

  clientResponseArgs.prototype._json = null;

  clientResponseArgs.prototype._language = null;

  clientResponseArgs.prototype._range = null;

  clientResponseArgs.prototype._statusCode = 0;

  function clientResponseArgs() {
    this.setStatusCode = bind(this.setStatusCode, this);
    this.setRange = bind(this.setRange, this);
    this.setLanguage = bind(this.setLanguage, this);
    this.setJson = bind(this.setJson, this);
    this.setHeaders = bind(this.setHeaders, this);
    this.setFormat = bind(this.setFormat, this);
    this.setException = bind(this.setException, this);
    this.setBytes = bind(this.setBytes, this);
    this.getStatusCode = bind(this.getStatusCode, this);
    this.getRange = bind(this.getRange, this);
    this.getLanguage = bind(this.getLanguage, this);
    this.getJson = bind(this.getJson, this);
    this.getHeaders = bind(this.getHeaders, this);
    this.getFormat = bind(this.getFormat, this);
    this.getException = bind(this.getException, this);
    this.getBytes = bind(this.getBytes, this);
    var bytes, dynamicProperties, exception, format, headers, instance, json, language, range, statusCode;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = clientResponseArgs.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    if (arguments.length === 3) {
      exception = arguments[0];
      statusCode = arguments[1];
      dynamicProperties = arguments[2];
      instance = clientResponseArgs.__super__.constructor.call(this);
      this.setException(exception);
      this.setStatusCode(statusCode);
      this.setDynamicProperties(dynamicProperties);
      return instance;
    }
    if (arguments.length === 8) {
      json = arguments[0];
      bytes = arguments[1];
      statusCode = arguments[2];
      range = arguments[3];
      language = arguments[4];
      format = arguments[5];
      headers = arguments[6];
      dynamicProperties = arguments[7];
      instance = clientResponseArgs.__super__.constructor.call(this);
      this.setJson(json);
      this.setBytes(bytes);
      this.setStatusCode(statusCode);
      this.setRange(range);
      this.setLanguage(language);
      this.setFormat(format);
      this.setHeaders(headers);
      this.setDynamicProperties(dynamicProperties);
      return instance;
    }
  }

  clientResponseArgs.prototype.getBytes = function() {
    return this._bytes;
  };

  clientResponseArgs.prototype.getException = function() {
    return this._exception;
  };

  clientResponseArgs.prototype.getFormat = function() {
    return this._format;
  };

  clientResponseArgs.prototype.getHeaders = function() {
    return this._headers;
  };

  clientResponseArgs.prototype.getJson = function() {
    return this._json;
  };

  clientResponseArgs.prototype.getLanguage = function() {
    return this._language;
  };

  clientResponseArgs.prototype.getRange = function() {
    return this._range;
  };

  clientResponseArgs.prototype.getStatusCode = function() {
    return this._statusCode;
  };

  clientResponseArgs.prototype.setBytes = function() {
    var value;
    value = arguments[0];
    return this._bytes = value;
  };

  clientResponseArgs.prototype.setException = function() {
    var value;
    value = arguments[0];
    return this._exception = value;
  };

  clientResponseArgs.prototype.setFormat = function() {
    var value;
    value = arguments[0];
    return this._format = value;
  };

  clientResponseArgs.prototype.setHeaders = function() {
    var value;
    value = arguments[0];
    return this._headers = value;
  };

  clientResponseArgs.prototype.setJson = function() {
    var value;
    value = arguments[0];
    return this._json = value;
  };

  clientResponseArgs.prototype.setLanguage = function() {
    var value;
    value = arguments[0];
    return this._language = value;
  };

  clientResponseArgs.prototype.setRange = function() {
    var value;
    value = arguments[0];
    return this._range = value;
  };

  clientResponseArgs.prototype.setStatusCode = function() {
    var value;
    value = arguments[0];
    return this._statusCode = value;
  };

  return clientResponseArgs;

})(fm.dynamic);



/*<span id='cls-fm.therest.client'>&nbsp;</span> */

/**
@class fm.therest.client
 <div>
 <p>
 The TheRest client, used for sending REST queries to a TheRest server.
 </p>
 </div><div>
 <p>
 The TheRest client has 4 primary operations:
 </p>
 <ol>
 <li>
 Post: Typically used for sending data to the server for creation or changing state.
 </li>
 <li>
 Get: Typically used for reading data from the server.
 </li>
 <li>
 Put: Typically used for sending data to the server for modification.
 </li>
 <li>
 Delete: Typically used for sending data to the server for deletion.
 </li>
 </ol>
 <p>
 Each method (and the constructor) take a single "args" object. This object defines
 callback functions, configuration settings, and error handling information. It
 allows the client to default to sensible values, while allowing easy overrides.
 </p>
 </div>

@extends fm.object
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.client = (function(superClass) {
  extend(client, superClass);

  client.prototype.__requestUrl = null;

  client._argsKey = null;


  /*<span id='prop-fm.therest.client-_invalidResponseMessage'>&nbsp;</span> */


  /**
  	 <div>
  	 The default message for an invalid server response.
  	 </div>
  
  	@field _invalidResponseMessage
  	@type {String}
   */

  client._invalidResponseMessage = "Invalid response received from server.";

  client._onAfterSend = null;

  client._onBeforeSend = null;

  client.prototype._onHttpRequestCreated = null;

  client.prototype._onHttpResponseReceived = null;

  client.prototype._onUnhandledException = null;

  client._requestKey = null;

  client.prototype._requestTimeout = 0;

  client.prototype._requestTransfer = null;

  client.prototype._synchronous = null;


  /*<span id='method-fm.therest.client-fm.therest.client'>&nbsp;</span> */


  /**
  	 <div>
  	 Initializes a new instance of the <see cref="fm.therest.client">fm.therest.client</see> class.
  	 </div>
  	@function fm.therest.client
  	@param {String} requestUrl The absolute URL of the REST handler,
  	 typically ending with therest.ashx.
  	@return {}
   */

  function client() {
    this.setSynchronous = bind(this.setSynchronous, this);
    this.setRequestUrl = bind(this.setRequestUrl, this);
    this.setRequestTimeout = bind(this.setRequestTimeout, this);
    this.sendInternalCallback = bind(this.sendInternalCallback, this);
    this.sendInternal = bind(this.sendInternal, this);
    this.send = bind(this.send, this);
    this.removeOnUnhandledException = bind(this.removeOnUnhandledException, this);
    this.removeOnHttpResponseReceived = bind(this.removeOnHttpResponseReceived, this);
    this.removeOnHttpRequestCreated = bind(this.removeOnHttpRequestCreated, this);
    this.raiseUnhandledException = bind(this.raiseUnhandledException, this);
    this.raiseSendSuccess = bind(this.raiseSendSuccess, this);
    this.raiseSendFailure = bind(this.raiseSendFailure, this);
    this.raiseSendComplete = bind(this.raiseSendComplete, this);
    this.raiseEvent = bind(this.raiseEvent, this);
    this.raiseBeforeEvent = bind(this.raiseBeforeEvent, this);
    this.raiseBaseEvent = bind(this.raiseBaseEvent, this);
    this.raiseAfterEvent = bind(this.raiseAfterEvent, this);
    this.raiseAction = bind(this.raiseAction, this);
    this.processRequestUrl = bind(this.processRequestUrl, this);
    this.performSendCallback = bind(this.performSendCallback, this);
    this.performSend = bind(this.performSend, this);
    this.internalOnHttpResponseReceived = bind(this.internalOnHttpResponseReceived, this);
    this.internalOnHttpRequestCreated = bind(this.internalOnHttpRequestCreated, this);
    this.getSynchronous = bind(this.getSynchronous, this);
    this.getRequestUrl = bind(this.getRequestUrl, this);
    this.getRequestTimeout = bind(this.getRequestTimeout, this);
    this.generateToken = bind(this.generateToken, this);
    this.checkSynchronous = bind(this.checkSynchronous, this);
    this.addOnUnhandledException = bind(this.addOnUnhandledException, this);
    this.addOnHttpResponseReceived = bind(this.addOnHttpResponseReceived, this);
    this.addOnHttpRequestCreated = bind(this.addOnHttpRequestCreated, this);
    var instance, requestUrl;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = client.__super__.constructor.call(this);
      this.setRequestTimeout(15000);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    if (arguments.length === 0) {
      instance = client.__super__.constructor.call(this);
      this.setRequestTimeout(15000);
      return instance;
    }
    if (arguments.length === 1) {
      requestUrl = arguments[0];
      client.call(this);
      this.setRequestUrl(requestUrl);
      return instance;
    }
  }


  /*<span id='method-fm.therest.client-addOnAfterSend'>&nbsp;</span> */


  /**
  	 <div>
  	 Adds a handler that is raised after a <see cref="fm.therest.client">fm.therest.client</see> response returns. This event is
  	 designed to support extensions by allowing modifications to be applied
  	 to a response after it is received from the server.
  	 </div>
  	@function addOnAfterSend
  	@param {fm.doubleAction} value
  	@return {void}
   */

  client.addOnAfterSend = function() {
    var value;
    value = arguments[0];
    return fm.therest.client._onAfterSend = fm.delegate.combine(fm.therest.client._onAfterSend, value);
  };


  /*<span id='method-fm.therest.client-addOnBeforeSend'>&nbsp;</span> */


  /**
  	 <div>
  	 Adds a handler that is raised before a <see cref="fm.therest.client">fm.therest.client</see> request begins. This event is
  	 designed to support extensions by allowing modifications to be applied
  	 to a request before it is sent to the server.
  	 </div>
  	@function addOnBeforeSend
  	@param {fm.doubleAction} value
  	@return {void}
   */

  client.addOnBeforeSend = function() {
    var value;
    value = arguments[0];
    return fm.therest.client._onBeforeSend = fm.delegate.combine(fm.therest.client._onBeforeSend, value);
  };


  /*<span id='method-fm.therest.client-removeOnAfterSend'>&nbsp;</span> */


  /**
  	 <div>
  	 Removes a handler that is raised after a <see cref="fm.therest.client">fm.therest.client</see> response returns. This event is
  	 designed to support extensions by allowing modifications to be applied
  	 to a response after it is received from the server.
  	 </div>
  	@function removeOnAfterSend
  	@param {fm.doubleAction} value
  	@return {void}
   */

  client.removeOnAfterSend = function() {
    var value;
    value = arguments[0];
    return fm.therest.client._onAfterSend = fm.delegate.remove(fm.therest.client._onAfterSend, value);
  };


  /*<span id='method-fm.therest.client-removeOnBeforeSend'>&nbsp;</span> */


  /**
  	 <div>
  	 Removes a handler that is raised before a <see cref="fm.therest.client">fm.therest.client</see> request begins. This event is
  	 designed to support extensions by allowing modifications to be applied
  	 to a request before it is sent to the server.
  	 </div>
  	@function removeOnBeforeSend
  	@param {fm.doubleAction} value
  	@return {void}
   */

  client.removeOnBeforeSend = function() {
    var value;
    value = arguments[0];
    return fm.therest.client._onBeforeSend = fm.delegate.remove(fm.therest.client._onBeforeSend, value);
  };


  /*<span id='method-fm.therest.client-addOnHttpRequestCreated'>&nbsp;</span> */


  /**
  	 <div>
  	 Adds a handler that is raised whenever a new HttpWebRequest is created for transmission
  	 of a request over HTTP to the server. This allows for modification of
  	 cookies, headers, etc.
  	 </div>
  
  	@function addOnHttpRequestCreated
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.addOnHttpRequestCreated = function() {
    var value;
    value = arguments[0];
    return this._onHttpRequestCreated = fm.delegate.combine(this._onHttpRequestCreated, value);
  };


  /*<span id='method-fm.therest.client-addOnHttpResponseReceived'>&nbsp;</span> */


  /**
  	 <div>
  	 Adds a handler that is raised whenever a new HTTP response has been received from
  	 the server. This allows reading of cookies, headers, etc.
  	 </div>
  
  	@function addOnHttpResponseReceived
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.addOnHttpResponseReceived = function() {
    var value;
    value = arguments[0];
    return this._onHttpResponseReceived = fm.delegate.combine(this._onHttpResponseReceived, value);
  };


  /*<span id='method-fm.therest.client-addOnUnhandledException'>&nbsp;</span> */


  /**
  	 <div>
  	 Adds a handler that is raised when an exception is thrown in user code and not handled,
  	 typically in a callback or event handler.
  	 </div>
  
  	@function addOnUnhandledException
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.addOnUnhandledException = function() {
    var value;
    value = arguments[0];
    return this._onUnhandledException = fm.delegate.combine(this._onUnhandledException, value);
  };

  client.prototype.checkSynchronous = function() {
    var synchronous;
    synchronous = arguments[0];
    if (synchronous !== null) {
      return synchronous;
    }
    if (this.getSynchronous() !== null) {
      return this.getSynchronous();
    }
    return false;
  };


  /*<span id='method-fm.therest.client-generateToken'>&nbsp;</span> */


  /**
  	 <div>
  	 Generates a new token based on the current date/time.
  	 </div>
  	@function generateToken
  	@return {String} The generated token.
   */

  client.prototype.generateToken = function() {
    var str;
    str = fm.intExtensions.toString(fm.dateTime.getUtcNow().getTicks());
    return fm.stringExtensions.substring(str, str.length - 12, 8);
  };


  /*<span id='method-fm.therest.client-getRequestTimeout'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the number of milliseconds to wait for a standard request to
  	 return a response before it is aborted and another request is attempted.
  	 </div>
  
  	@function getRequestTimeout
  	@return {Integer}
   */

  client.prototype.getRequestTimeout = function() {
    return this._requestTimeout;
  };


  /*<span id='method-fm.therest.client-getRequestUrl'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the absolute URL of the TheRest handler, typically
  	 ending with therest.ashx.
  	 </div>
  
  	@function getRequestUrl
  	@return {String}
   */

  client.prototype.getRequestUrl = function() {
    return this.__requestUrl;
  };


  /*<span id='method-fm.therest.client-getSynchronous'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets whether or not to execute client methods synchronously. This approach is not
  	 recommended for UI threads, as it will block until the request completes.
  	 </div>
  
  	@function getSynchronous
  	@return {fm.nullable}
   */

  client.prototype.getSynchronous = function() {
    return this._synchronous;
  };

  client.prototype.internalOnHttpRequestCreated = function() {
    var e;
    e = arguments[0];
    return this.raiseBaseEvent(this._onHttpRequestCreated, e, "OnHttpRequestCreated");
  };

  client.prototype.internalOnHttpResponseReceived = function() {
    var e;
    e = arguments[0];
    return this.raiseBaseEvent(this._onHttpResponseReceived, e, "OnHttpResponseReceived");
  };

  client.prototype.performSend = function() {
    var args, request, request2, sendArgs;
    sendArgs = arguments[0];
    if (fm.global.equals(sendArgs.getFormat(), fm.therest.restFormat.None)) {
      if (!fm.stringExtensions.isNullOrEmpty(sendArgs.getDataJson())) {
        sendArgs.setFormat(fm.therest.restFormat.Json);
      } else {
        if (!fm.global.equals(sendArgs.getDataBytes(), null)) {
          sendArgs.setFormat(fm.therest.restFormat.Binary);
        }
      }
    }
    args = new fm.therest.beforeSendArgs();
    args.setMethodArgs(sendArgs);
    if (this.raiseBeforeEvent(fm.therest.client._onBeforeSend, args, "OnBeforeSend")) {
      request2 = new fm.therest.clientRequest();
      request2.setRequestJson(sendArgs.getDataJson());
      request2.setRequestBytes(sendArgs.getDataBytes());
      request2.setRequestFormat(sendArgs.getFormat());
      request2.setRequestRange(sendArgs.getRange());
      request2.setRequestLanguages(sendArgs.getLanguages());
      request2.setRequestPath(sendArgs.getPath());
      request2.setRequestMethod(sendArgs.getMethod());
      request2.setRequestHeaders(sendArgs.getHeaders());
      request2.setCallback(this.performSendCallback);
      request = request2;
      request.setDynamicValue(fm.therest.client._argsKey, sendArgs);
      return this.sendInternal(request, sendArgs.getRequestUrl(), this.checkSynchronous(sendArgs.getSynchronous()));
    }
  };

  client.prototype.performSendCallback = function() {
    var args, dynamicValue, responseArgs;
    responseArgs = arguments[0];
    dynamicValue = responseArgs.getDynamicValue(fm.therest.client._argsKey);
    args = new fm.therest.afterSendArgs();
    args.setMethodArgs(dynamicValue);
    if (this.raiseAfterEvent(fm.therest.client._onAfterSend, args, "OnAfterSend", responseArgs)) {
      if (!fm.global.equals(responseArgs.getException(), null)) {
        this.raiseSendFailure(dynamicValue, responseArgs);
        return this.raiseSendComplete(dynamicValue, responseArgs);
      } else {
        this.raiseSendSuccess(dynamicValue, responseArgs);
        return this.raiseSendComplete(dynamicValue, responseArgs);
      }
    }
  };

  client.prototype.processRequestUrl = function() {
    var requestUrl;
    requestUrl = arguments[0];
    requestUrl = fm.httpTransfer.addQueryToUrl(requestUrl, "src", fm.httpWebRequestTransfer.getPlatformCode());
    requestUrl = fm.httpTransfer.addQueryToUrl(requestUrl, "AspxAutoDetectCookieSupport", "1");
    return requestUrl;
  };

  client.prototype.raiseAction = function() {
    var args, callback, error, exception, source;
    callback = arguments[0];
    args = arguments[1];
    source = arguments[2];
    try {
      return callback(args);
    } catch (error) {
      exception = error;
      if (!this.raiseUnhandledException(exception)) {
        return fm.asyncException.asyncThrow(exception, source);
      }
    } finally {

    }
  };

  client.prototype.raiseAfterEvent = function() {
    var args, eventMethod, eventName, responseArgs;
    eventMethod = arguments[0];
    args = arguments[1];
    eventName = arguments[2];
    responseArgs = arguments[3];
    args.setException(responseArgs.getException());
    args.setResponseJson(responseArgs.getJson());
    args.setResponseHeaders(responseArgs.getHeaders());
    this.raiseEvent(eventMethod, args, eventName);
    return !args.getCancel();
  };

  client.prototype.raiseBaseEvent = function() {
    var args, error, eventMethod, eventName, exception;
    eventMethod = arguments[0];
    args = arguments[1];
    eventName = arguments[2];
    if (!fm.global.equals(eventMethod, null)) {
      try {
        return eventMethod(args);
      } catch (error) {
        exception = error;
        if (!this.raiseUnhandledException(exception)) {
          return fm.asyncException.asyncThrow(exception, fm.stringExtensions.format("Client -> {0}", eventName));
        }
      } finally {

      }
    }
  };

  client.prototype.raiseBeforeEvent = function() {
    var args, eventMethod, eventName;
    eventMethod = arguments[0];
    args = arguments[1];
    eventName = arguments[2];
    this.raiseEvent(eventMethod, args, eventName);
    return !args.getCancel();
  };

  client.prototype.raiseEvent = function() {
    var args, error, eventMethod, eventName, exception;
    eventMethod = arguments[0];
    args = arguments[1];
    eventName = arguments[2];
    args.setClient(this);
    if (!fm.global.equals(eventMethod, null)) {
      try {
        return eventMethod(this, args);
      } catch (error) {
        exception = error;
        if (!this.raiseUnhandledException(exception)) {
          return fm.asyncException.asyncThrow(exception, fm.stringExtensions.format("Client -> {0}", eventName));
        }
      } finally {

      }
    }
  };

  client.prototype.raiseSendComplete = function() {
    var args, responseArgs, sendArgs;
    sendArgs = arguments[0];
    responseArgs = arguments[1];
    if (!fm.global.equals(sendArgs.getOnComplete(), null)) {
      args = new fm.therest.sendCompleteArgs();
      args.setRequestDataJson(sendArgs.getDataJson());
      args.setRequestDataBytes(sendArgs.getDataBytes());
      args.setRequestFormat(sendArgs.getFormat());
      args.setRequestRange(sendArgs.getRange());
      args.setRequestLanguages(sendArgs.getLanguages());
      args.setRequestMethod(sendArgs.getMethod());
      args.setRequestPath(sendArgs.getPath());
      args.setClient(this);
      args.setDynamicProperties(sendArgs.getDynamicProperties());
      return this.raiseAction(sendArgs.getOnComplete(), args, "Client -> Send -> OnComplete");
    }
  };

  client.prototype.raiseSendFailure = function() {
    var args, responseArgs, sendArgs;
    sendArgs = arguments[0];
    responseArgs = arguments[1];
    if (!fm.global.equals(sendArgs.getOnFailure(), null)) {
      args = new fm.therest.sendFailureArgs();
      args.setException(responseArgs.getException());
      args.setResponseStatusCode(responseArgs.getStatusCode());
      args.setRequestDataJson(sendArgs.getDataJson());
      args.setRequestDataBytes(sendArgs.getDataBytes());
      args.setRequestFormat(sendArgs.getFormat());
      args.setRequestRange(sendArgs.getRange());
      args.setRequestLanguages(sendArgs.getLanguages());
      args.setRequestMethod(sendArgs.getMethod());
      args.setRequestPath(sendArgs.getPath());
      args.setClient(this);
      args.setDynamicProperties(sendArgs.getDynamicProperties());
      return this.raiseAction(sendArgs.getOnFailure(), args, "Client -> Send -> OnFailure");
    }
  };

  client.prototype.raiseSendSuccess = function() {
    var args, responseArgs, sendArgs;
    sendArgs = arguments[0];
    responseArgs = arguments[1];
    if (!fm.global.equals(sendArgs.getOnSuccess(), null)) {
      args = new fm.therest.sendSuccessArgs();
      args.setResponseDataJson(responseArgs.getJson());
      args.setResponseDataBytes(responseArgs.getBytes());
      args.setResponseFormat(responseArgs.getFormat());
      args.setResponseRange(responseArgs.getRange());
      args.setResponseLanguage(responseArgs.getLanguage());
      args.setResponseHeaders(responseArgs.getHeaders());
      args.setResponseStatusCode(responseArgs.getStatusCode());
      args.setRequestDataJson(sendArgs.getDataJson());
      args.setRequestDataBytes(sendArgs.getDataBytes());
      args.setRequestFormat(sendArgs.getFormat());
      args.setRequestRange(sendArgs.getRange());
      args.setRequestLanguages(sendArgs.getLanguages());
      args.setRequestMethod(sendArgs.getMethod());
      args.setRequestPath(sendArgs.getPath());
      args.setClient(this);
      args.setDynamicProperties(sendArgs.getDynamicProperties());
      return this.raiseAction(sendArgs.getOnSuccess(), args, "Client -> Send -> OnSuccess");
    }
  };

  client.prototype.raiseUnhandledException = function() {
    var args2, error, exception, exception2, onUnhandledException, p;
    exception = arguments[0];
    onUnhandledException = this._onUnhandledException;
    if (!fm.global.equals(onUnhandledException, null)) {
      args2 = new fm.therest.unhandledExceptionArgs();
      args2.__exception = exception;
      p = args2;
      try {
        onUnhandledException(p);
      } catch (error) {
        exception2 = error;
        fm.asyncException.asyncThrow(exception2, "Client -> OnUnhandledException");
      } finally {

      }
      return p.getHandled();
    }
    return false;
  };


  /*<span id='method-fm.therest.client-removeOnHttpRequestCreated'>&nbsp;</span> */


  /**
  	 <div>
  	 Removes a handler that is raised whenever a new HttpWebRequest is created for transmission
  	 of a request over HTTP to the server. This allows for modification of
  	 cookies, headers, etc.
  	 </div>
  
  	@function removeOnHttpRequestCreated
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.removeOnHttpRequestCreated = function() {
    var value;
    value = arguments[0];
    return this._onHttpRequestCreated = fm.delegate.remove(this._onHttpRequestCreated, value);
  };


  /*<span id='method-fm.therest.client-removeOnHttpResponseReceived'>&nbsp;</span> */


  /**
  	 <div>
  	 Removes a handler that is raised whenever a new HTTP response has been received from
  	 the server. This allows reading of cookies, headers, etc.
  	 </div>
  
  	@function removeOnHttpResponseReceived
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.removeOnHttpResponseReceived = function() {
    var value;
    value = arguments[0];
    return this._onHttpResponseReceived = fm.delegate.remove(this._onHttpResponseReceived, value);
  };


  /*<span id='method-fm.therest.client-removeOnUnhandledException'>&nbsp;</span> */


  /**
  	 <div>
  	 Removes a handler that is raised when an exception is thrown in user code and not handled,
  	 typically in a callback or event handler.
  	 </div>
  
  	@function removeOnUnhandledException
  	@param {fm.singleAction} value
  	@return {void}
   */

  client.prototype.removeOnUnhandledException = function() {
    var value;
    value = arguments[0];
    return this._onUnhandledException = fm.delegate.remove(this._onUnhandledException, value);
  };


  /*<span id='method-fm.therest.client-send'>&nbsp;</span> */


  /**
  	 <div>
  	 Sends a request to the server.
  	 </div><div>
  	 <p>
  	 When the request completes successfully, the OnSuccess callback
  	 will be invoked with the server response.
  	 </p>
  	 </div>
  	@function send
  	@param {fm.therest.sendArgs} sendArgs The send arguments.
  	 See fm.therest.sendArgs for details.
  	@return {fm.therest.client} The client.
   */

  client.prototype.send = function() {
    var sendArgs;
    sendArgs = arguments[0];
    if (fm.stringExtensions.isNullOrEmpty(sendArgs.getPath())) {
      throw new Error("path cannot be null.");
    }
    if (((((!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Get)) && (!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Head))) && ((!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Post)) && (!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Put)))) && (!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Patch))) && (!fm.global.equals(sendArgs.getMethod(), fm.httpMethod.Delete))) {
      throw new Error("method cannot be null.");
    }
    this.performSend(sendArgs);
    return this;
  };

  client.prototype.sendInternal = function() {
    var args2, request, requestArgs, str, synchronous, url;
    request = arguments[0];
    url = arguments[1];
    synchronous = arguments[2];
    if (fm.stringExtensions.isNullOrEmpty(url)) {
      url = this.getRequestUrl();
    }
    if (fm.stringExtensions.endsWith(url, "/")) {
      url = fm.stringExtensions.substring(url, 0, url.length - 1);
    }
    str = this.processRequestUrl(url);
    url = fm.stringExtensions.concat(url, request.getRequestPath());
    url = this.processRequestUrl(url);
    args2 = new fm.httpRequestArgs();
    args2.setSender(this);
    args2.setUrl(url);
    args2.setMethod(request.getRequestMethod());
    args2.setHeaders(request.getRequestHeaders());
    args2.setTimeout(this.getRequestTimeout());
    args2.setOnRequestCreated(this.internalOnHttpRequestCreated);
    args2.setOnResponseReceived(this.internalOnHttpResponseReceived);
    requestArgs = args2;
    requestArgs.setDynamicValue("frameUrl", str);
    if (fm.global.equals(request.getRequestFormat(), fm.therest.restFormat.Binary)) {
      requestArgs.getHeaders().set("Content-Type", "application/octet-stream");
      requestArgs.setBinaryContent(request.getRequestBytes());
    } else {
      requestArgs.getHeaders().set("Content-Type", "application/json");
      requestArgs.setTextContent(request.getRequestJson());
    }
    if (!fm.global.equals(request.getRequestRange(), null)) {
      requestArgs.getHeaders().set("Range", request.getRequestRange().serializeRequestRange());
    }
    if (!fm.global.equals(request.getRequestLanguages(), null)) {
      requestArgs.getHeaders().set("Accept-Language", fm.therest.language.serializeAcceptLanguage(request.getRequestLanguages()));
    }
    requestArgs.setDynamicValue(fm.therest.client._requestKey, request);
    this._requestTransfer = fm.httpTransferFactory.getHttpTransfer();
    if (synchronous) {
      return this.sendInternalCallback(this._requestTransfer.send(requestArgs));
    } else {
      return this._requestTransfer.sendAsync(requestArgs, this.sendInternalCallback);
    }
  };

  client.prototype.sendInternalCallback = function() {
    var args, bytes, contentType, dynamicValue, error, exception1, json, language, none, range, str2;
    args = arguments[0];
    dynamicValue = args.getRequestArgs().getDynamicValue(fm.therest.client._requestKey);
    try {
      this._requestTransfer.shutdown();
    } catch (error) {
      exception1 = error;
    } finally {

    }
    contentType = fm.stringExtensions.empty;
    if (!fm.global.equals(args.getHeaders().get("Content-Type"), null)) {
      contentType = args.getHeaders().get("Content-Type");
    }
    if ((fm.global.equals(args.getException(), null)) && ((args.getStatusCode() < 200) || (args.getStatusCode() > 299))) {
      str2 = null;
      if ((fm.global.equals(fm.therest.restFormatUtility.getFormat(contentType), fm.therest.restFormat.Json)) && (!fm.global.equals(args.getTextContent(), null))) {
        str2 = fm.serializer.deserializeString(args.getTextContent());
      }
      if (fm.stringExtensions.isNullOrEmpty(str2)) {
        str2 = "Could not connect to the server.";
        if (args.getStatusCode() > 0) {
          str2 = fm.stringExtensions.concat(str2, fm.stringExtensions.format(" (status code {0})", fm.intExtensions.toString(args.getStatusCode())));
        }
      }
      args.setException(new Error(str2));
    }
    if (!fm.global.equals(args.getException(), null)) {
      return dynamicValue.getCallback()(new fm.therest.clientResponseArgs(args.getException(), args.getStatusCode(), dynamicValue.getDynamicProperties()));
    } else {
      range = fm.therest.range.deserializeResponseRange(args.getHeaders().get("Content-Range"));
      language = fm.therest.language.deserializeContentLanguage(args.getHeaders().get("Content-Language"));
      none = fm.therest.restFormat.None;
      json = null;
      bytes = null;
      if (fm.global.equals(fm.therest.restFormatUtility.getFormat(contentType), fm.therest.restFormat.Binary)) {
        none = fm.therest.restFormat.Binary;
        bytes = args.getBinaryContent();
      } else {
        none = fm.therest.restFormat.Json;
        json = args.getTextContent();
      }
      return dynamicValue.getCallback()(new fm.therest.clientResponseArgs(json, bytes, args.getStatusCode(), range, language, none, args.getHeaders(), dynamicValue.getDynamicProperties()));
    }
  };


  /*<span id='method-fm.therest.client-setRequestTimeout'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the number of milliseconds to wait for a standard request to
  	 return a response before it is aborted and another request is attempted.
  	 </div>
  
  	@function setRequestTimeout
  	@param {Integer} value
  	@return {void}
   */

  client.prototype.setRequestTimeout = function() {
    var value;
    value = arguments[0];
    return this._requestTimeout = value;
  };


  /*<span id='method-fm.therest.client-setRequestUrl'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the absolute URL of the TheRest handler, typically
  	 ending with therest.ashx.
  	 </div>
  
  	@function setRequestUrl
  	@param {String} value
  	@return {void}
   */

  client.prototype.setRequestUrl = function() {
    var value;
    value = arguments[0];
    if (fm.global.equals(value, null)) {
      throw new Error("Request URL cannot be null.");
    }
    return this.__requestUrl = value;
  };


  /*<span id='method-fm.therest.client-setSynchronous'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets whether or not to execute client methods synchronously. This approach is not
  	 recommended for UI threads, as it will block until the request completes.
  	 </div>
  
  	@function setSynchronous
  	@param {fm.nullable} value
  	@return {void}
   */

  client.prototype.setSynchronous = function() {
    var value;
    value = arguments[0];
    return this._synchronous = value;
  };

  client._argsKey = "fm.args";

  client._requestKey = "fm.request";

  return client;

})(fm.object);


var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.clientRequest = (function(superClass) {
  extend(clientRequest, superClass);

  clientRequest.prototype._callback = null;

  clientRequest.prototype._requestBytes = null;

  clientRequest.prototype._requestFormat = null;

  clientRequest.prototype._requestHeaders = null;

  clientRequest.prototype._requestJson = null;

  clientRequest.prototype._requestLanguages = null;

  clientRequest.prototype._requestMethod = null;

  clientRequest.prototype._requestPath = null;

  clientRequest.prototype._requestRange = null;

  function clientRequest() {
    this.setRequest = bind(this.setRequest, this);
    this.getRequest = bind(this.getRequest, this);
    this.setRequestRange = bind(this.setRequestRange, this);
    this.setRequestPath = bind(this.setRequestPath, this);
    this.setRequestMethod = bind(this.setRequestMethod, this);
    this.setRequestLanguages = bind(this.setRequestLanguages, this);
    this.setRequestJson = bind(this.setRequestJson, this);
    this.setRequestHeaders = bind(this.setRequestHeaders, this);
    this.setRequestFormat = bind(this.setRequestFormat, this);
    this.setRequestBytes = bind(this.setRequestBytes, this);
    this.setCallback = bind(this.setCallback, this);
    this.getRequestRange = bind(this.getRequestRange, this);
    this.getRequestPath = bind(this.getRequestPath, this);
    this.getRequestMethod = bind(this.getRequestMethod, this);
    this.getRequestLanguages = bind(this.getRequestLanguages, this);
    this.getRequestJson = bind(this.getRequestJson, this);
    this.getRequestHeaders = bind(this.getRequestHeaders, this);
    this.getRequestFormat = bind(this.getRequestFormat, this);
    this.getRequestBytes = bind(this.getRequestBytes, this);
    this.getCallback = bind(this.getCallback, this);
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = clientRequest.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = clientRequest.__super__.constructor.call(this);
    return instance;
  }

  clientRequest.prototype.getCallback = function() {
    return this._callback;
  };

  clientRequest.prototype.getRequestBytes = function() {
    return this._requestBytes;
  };

  clientRequest.prototype.getRequestFormat = function() {
    return this._requestFormat;
  };

  clientRequest.prototype.getRequestHeaders = function() {
    return this._requestHeaders;
  };

  clientRequest.prototype.getRequestJson = function() {
    return this._requestJson;
  };

  clientRequest.prototype.getRequestLanguages = function() {
    return this._requestLanguages;
  };

  clientRequest.prototype.getRequestMethod = function() {
    return this._requestMethod;
  };

  clientRequest.prototype.getRequestPath = function() {
    return this._requestPath;
  };

  clientRequest.prototype.getRequestRange = function() {
    return this._requestRange;
  };

  clientRequest.prototype.setCallback = function() {
    var value;
    value = arguments[0];
    return this._callback = value;
  };

  clientRequest.prototype.setRequestBytes = function() {
    var value;
    value = arguments[0];
    return this._requestBytes = value;
  };

  clientRequest.prototype.setRequestFormat = function() {
    var value;
    value = arguments[0];
    return this._requestFormat = value;
  };

  clientRequest.prototype.setRequestHeaders = function() {
    var value;
    value = arguments[0];
    return this._requestHeaders = value;
  };

  clientRequest.prototype.setRequestJson = function() {
    var value;
    value = arguments[0];
    return this._requestJson = value;
  };

  clientRequest.prototype.setRequestLanguages = function() {
    var value;
    value = arguments[0];
    return this._requestLanguages = value;
  };

  clientRequest.prototype.setRequestMethod = function() {
    var value;
    value = arguments[0];
    return this._requestMethod = value;
  };

  clientRequest.prototype.setRequestPath = function() {
    var value;
    value = arguments[0];
    return this._requestPath = value;
  };

  clientRequest.prototype.setRequestRange = function() {
    var value;
    value = arguments[0];
    return this._requestRange = value;
  };

  clientRequest.prototype.getRequest = function() {
    return fm.json.deserialize(this.getRequestJson.apply(this, arguments));
  };

  clientRequest.prototype.setRequest = function() {
    var request;
    request = arguments[0];
    arguments[arguments.length - 1] = fm.json.serialize(arguments[arguments.length - 1]);
    return this.setRequestJson.apply(this, arguments);
  };

  return clientRequest;

})(fm.dynamic);



/*<span id='cls-fm.therest.language'>&nbsp;</span> */

/**
@class fm.therest.language
 <div>
 A language sent in the Accept-Language or Content-Language headers.
 </div>

@extends fm.object
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.language = (function(superClass) {
  extend(language, superClass);

  language.prototype.__quality = 0.0;

  language.prototype._code = null;


  /*<span id='method-fm.therest.language-fm.therest.language'>&nbsp;</span> */


  /**
  	 <div>
  	 Initializes a new instance of the <see cref="fm.therest.language">fm.therest.language</see> class.
  	 </div>
  	@function fm.therest.language
  	@param {String} code The two-letter ISO code/tag representing the language.
  	@param {Decimal} quality The language quality.
  	@return {}
   */

  function language() {
    this.setQuality = bind(this.setQuality, this);
    this.setCode = bind(this.setCode, this);
    this.serializeLanguage = bind(this.serializeLanguage, this);
    this.serializeContentLanguage = bind(this.serializeContentLanguage, this);
    this.getQuality = bind(this.getQuality, this);
    this.getCode = bind(this.getCode, this);
    var code, instance, quality;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = language.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    if (arguments.length === 2) {
      code = arguments[0];
      quality = arguments[1];
      instance = language.__super__.constructor.call(this);
      this.setCode(code);
      this.setQuality(quality);
      return instance;
    }
    if (arguments.length === 1) {
      code = arguments[0];
      language.call(this, code, 1);
      return instance;
    }
  }


  /*<span id='method-fm.therest.language-deserializeAcceptLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Deserializes an array of languages from a request HTTP header value.
  	 </div>
  	@function deserializeAcceptLanguage
  	@param {String} acceptLanguageHeader The accept language header.
  	@return {fm.array}
   */

  language.deserializeAcceptLanguage = function() {
    var acceptLanguageHeader, i, languageArray, strArray;
    acceptLanguageHeader = arguments[0];
    if (fm.global.equals(acceptLanguageHeader, null)) {
      return null;
    }
    strArray = fm.stringExtensions.split(acceptLanguageHeader, [',']);
    languageArray = new Array(strArray.length);
    i = 0;
    while (i < strArray.length) {
      try {
        languageArray[i] = fm.therest.language.deserializeLanguage(fm.stringExtensions.trim(strArray[i]));
      } finally {
        i++;
      }
    }
    return languageArray;
  };


  /*<span id='method-fm.therest.language-deserializeContentLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Deserializes a language from a request HTTP header value.
  	 </div>
  	@function deserializeContentLanguage
  	@param {String} contentLanguageHeader The content language header.
  	@return {fm.therest.language}
   */

  language.deserializeContentLanguage = function() {
    var contentLanguageHeader;
    contentLanguageHeader = arguments[0];
    if (fm.global.equals(contentLanguageHeader, null)) {
      return null;
    }
    return new fm.therest.language(contentLanguageHeader);
  };


  /*<span id='method-fm.therest.language-deserializeLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Deserializes a language chunk from an HTTP header value.
  	 </div>
  	@function deserializeLanguage
  	@param {String} languageChunk The language chunk.
  	@return {fm.therest.language} A language.
   */

  language.deserializeLanguage = function() {
    var _var0, _var1, code, floatResult, languageChunk, str2, strArray;
    languageChunk = arguments[0];
    if (fm.stringExtensions.isNullOrEmpty(languageChunk)) {
      return null;
    }
    strArray = fm.stringExtensions.split(languageChunk, [';']);
    code = strArray[0];
    floatResult = 1;
    if (strArray.length > 1) {
      str2 = strArray[1];
      _var0 = new fm.holder(floatResult);
      _var1 = fm.parseAssistant.tryParseFloatValue(str2.substring(2), _var0);
      floatResult = _var0.getValue();
      if (fm.stringExtensions.startsWith(str2, "q=") && !_var1) {
        floatResult = 1;
      }
    }
    return new fm.therest.language(code, floatResult);
  };


  /*<span id='method-fm.therest.language-serializeAcceptLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes an array of languages for use as a request HTTP header value.
  	 </div>
  	@function serializeAcceptLanguage
  	@param {fm.array} acceptLanguage The languages.
  	@return {String}
   */

  language.serializeAcceptLanguage = function() {
    var acceptLanguage, i, strArray;
    acceptLanguage = arguments[0];
    if (fm.global.equals(acceptLanguage, null)) {
      return null;
    }
    strArray = new Array(acceptLanguage.length);
    i = 0;
    while (i < acceptLanguage.length) {
      try {
        strArray[i] = fm.therest.language.serializeLanguage(acceptLanguage[i]);
      } finally {
        i++;
      }
    }
    return fm.stringExtensions.join(", ", strArray);
  };


  /*<span id='method-fm.therest.language-serializeContentLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes a language for use as a response HTTP header value.
  	 </div>
  	@function serializeContentLanguage
  	@param {fm.therest.language} contentLanguage The content language.
  	@return {String}
   */

  language.serializeContentLanguage = function() {
    var contentLanguage;
    contentLanguage = arguments[0];
    if (fm.global.equals(contentLanguage, null)) {
      return null;
    }
    return contentLanguage.getCode();
  };


  /*<span id='method-fm.therest.language-serializeLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes a language for use as an HTTP header value.
  	 </div>
  	@function serializeLanguage
  	@param {fm.therest.language} language The language.
  	@return {String} A language chunk.
   */

  language.serializeLanguage = function() {
    var language;
    language = arguments[0];
    if (fm.global.equals(language.getQuality(), 1)) {
      return language.getCode();
    }
    return fm.stringExtensions.format("{0};q={1}", language.getCode(), language.getQuality().toString());
  };


  /*<span id='method-fm.therest.language-getCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the two-letter ISO code/tag representing the language.
  	 </div>
  
  	@function getCode
  	@return {String}
   */

  language.prototype.getCode = function() {
    return this._code;
  };


  /*<span id='method-fm.therest.language-getQuality'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the language quality, used in Accept-Language
  	 headers to indicate preference over other languages. Acceptable
  	 values must exist in the range from 0 to 1 (inclusive).
  	 </div>
  
  	@function getQuality
  	@return {Decimal}
   */

  language.prototype.getQuality = function() {
    return this.__quality;
  };


  /*<span id='method-fm.therest.language-serializeContentLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes this language for use as a response HTTP header value.
  	 </div>
  	@function serializeContentLanguage
  	@return {String}
   */

  language.prototype.serializeContentLanguage = function() {
    return fm.therest.language.serializeContentLanguage(this);
  };


  /*<span id='method-fm.therest.language-serializeLanguage'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes this language for use as an HTTP header value.
  	 </div>
  	@function serializeLanguage
  	@return {String}
   */

  language.prototype.serializeLanguage = function() {
    return fm.therest.language.serializeLanguage(this);
  };


  /*<span id='method-fm.therest.language-setCode'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the two-letter ISO code/tag representing the language.
  	 </div>
  
  	@function setCode
  	@param {String} value
  	@return {void}
   */

  language.prototype.setCode = function() {
    var value;
    value = arguments[0];
    return this._code = value;
  };


  /*<span id='method-fm.therest.language-setQuality'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the language quality, used in Accept-Language
  	 headers to indicate preference over other languages. Acceptable
  	 values must exist in the range from 0 to 1 (inclusive).
  	 </div>
  
  	@function setQuality
  	@param {Decimal} value
  	@return {void}
   */

  language.prototype.setQuality = function() {
    var value;
    value = arguments[0];
    if ((value < 0) || (value > 1)) {
      throw new Error(fm.stringExtensions.format("quality '{0}' is outside the permissible range of 0 to 1 (inclusive).", value.toString()));
    }
    return this.__quality = value;
  };

  return language;

})(fm.object);



/*<span id='cls-fm.therest.range'>&nbsp;</span> */

/**
@class fm.therest.range
 <div>
 The range of values requested by or returned from a REST query.
 </div>

@extends fm.object
 */
var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.range = (function(superClass) {
  extend(range, superClass);

  range.prototype._length = 0;

  range.prototype._start = 0;

  range._token = null;

  range.prototype._total = 0;


  /*<span id='method-fm.therest.range-fm.therest.range'>&nbsp;</span> */


  /**
  	 <div>
  	 Initializes a new instance of the <see cref="fm.therest.range">fm.therest.range</see> class.
  	 </div>
  	@function fm.therest.range
  	@param {Integer} start An integer value indicating the index of the first item.
  	@param {Integer} length An integer value indicating the number of items.
  	@param {Integer} total An integer value indicating the total number of items in the collection.
  	@return {}
   */

  function range() {
    this.setTotal = bind(this.setTotal, this);
    this.setStart = bind(this.setStart, this);
    this.setLength = bind(this.setLength, this);
    this.serializeResponseRange = bind(this.serializeResponseRange, this);
    this.serializeRequestRange = bind(this.serializeRequestRange, this);
    this.getTotal = bind(this.getTotal, this);
    this.getStart = bind(this.getStart, this);
    this.getLength = bind(this.getLength, this);
    var instance, length, start, total;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = range.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    if (arguments.length === 2) {
      start = arguments[0];
      length = arguments[1];
      instance = range.__super__.constructor.call(this);
      this.setStart(start);
      this.setLength(length);
      return instance;
    }
    if (arguments.length === 3) {
      start = arguments[0];
      length = arguments[1];
      total = arguments[2];
      range.call(this, start, length);
      this.setTotal(total);
      return instance;
    }
  }


  /*<span id='method-fm.therest.range-deserializeRequestRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Deserializes a range from a request HTTP header value.
  	 </div>
  	@function deserializeRequestRange
  	@param {String} requestRangeHeader The request HTTP header value.
  	@return {fm.therest.range}
   */

  range.deserializeRequestRange = function() {
    var _var0, _var1, _var2, _var3, intResult, num2, num4, requestRangeHeader, s, start, str3, strArray, strArray2;
    requestRangeHeader = arguments[0];
    if (fm.stringExtensions.isNullOrEmpty(requestRangeHeader)) {
      return null;
    }
    strArray = fm.stringExtensions.split(requestRangeHeader, ['=']);
    if ((fm.global.equals(strArray.length, 2)) && (fm.global.equals(strArray[0], fm.therest.range.getToken()))) {
      strArray2 = fm.stringExtensions.split(strArray[1], ['-']);
      if (fm.global.equals(strArray2.length, 2)) {
        s = strArray2[0];
        str3 = strArray2[1];
        intResult = 0;
        num2 = 0;
        _var0 = new fm.holder(intResult);
        _var1 = fm.parseAssistant.tryParseIntegerValue(s, _var0);
        intResult = _var0.getValue();
        if (!_var1) {
          intResult = 0;
        }
        _var2 = new fm.holder(num2);
        _var3 = fm.parseAssistant.tryParseIntegerValue(str3, _var2);
        num2 = _var2.getValue();
        if (!_var3) {
          num2 = 0;
        }
        start = intResult;
        num4 = (num2 - intResult) + 1;
        start = (start > 0 ? start : 0);
        return new fm.therest.range(start, (num4 > 0 ? num4 : 0));
      }
    }
    return new fm.therest.range(0, 0, 0);
  };


  /*<span id='method-fm.therest.range-deserializeResponseRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Deserializes a range from a response HTTP header value.
  	 </div>
  	@function deserializeResponseRange
  	@param {String} responseRangeHeader The response HTTP header value.
  	@return {fm.therest.range}
   */

  range.deserializeResponseRange = function() {
    var _var0, _var1, _var2, _var3, _var4, _var5, intResult, num2, num3, num5, responseRangeHeader, s, start, str3, str4, strArray, strArray2, strArray3;
    responseRangeHeader = arguments[0];
    if (fm.stringExtensions.isNullOrEmpty(responseRangeHeader)) {
      return null;
    }
    strArray = fm.stringExtensions.split(responseRangeHeader, ['=']);
    if ((fm.global.equals(strArray.length, 2)) && (fm.global.equals(strArray[0], fm.therest.range.getToken()))) {
      strArray2 = fm.stringExtensions.split(strArray[1], ['/']);
      if (fm.global.equals(strArray2.length, 2)) {
        strArray3 = fm.stringExtensions.split(strArray2[0], ['-']);
        if (fm.global.equals(strArray3.length, 2)) {
          s = strArray3[0];
          str3 = strArray3[1];
          str4 = strArray2[1];
          intResult = 0;
          num2 = 0;
          num3 = 0;
          _var0 = new fm.holder(intResult);
          _var1 = fm.parseAssistant.tryParseIntegerValue(s, _var0);
          intResult = _var0.getValue();
          if (!_var1) {
            intResult = 0;
          }
          _var2 = new fm.holder(num2);
          _var3 = fm.parseAssistant.tryParseIntegerValue(str3, _var2);
          num2 = _var2.getValue();
          if (!_var3) {
            num2 = 0;
          }
          _var4 = new fm.holder(num3);
          _var5 = fm.parseAssistant.tryParseIntegerValue(str4, _var4);
          num3 = _var4.getValue();
          if (!_var5) {
            num3 = 0;
          }
          start = intResult;
          num5 = (num2 - intResult) + 1;
          start = (start > 0 ? start : 0);
          return new fm.therest.range(start, (num5 > 0 ? num5 : 0), num3);
        }
      }
    }
    return new fm.therest.range(0, 0, 0);
  };


  /*<span id='method-fm.therest.range-getToken'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets the token used in the Range header. Defaults to "items".
  	 </div>
  
  	@function getToken
  	@return {String}
   */

  range.getToken = function() {
    return fm.therest.range._token;
  };


  /*<span id='method-fm.therest.range-serializeRequestRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes a range for use as a request HTTP header value.
  	 </div>
  	@function serializeRequestRange
  	@param {fm.therest.range} range The range.
  	@return {String}
   */

  range.serializeRequestRange = function() {
    var num2, range, start;
    range = arguments[0];
    start = range.getStart();
    num2 = (range.getStart() + range.getLength()) - 1;
    start = (start > 0 ? start : 0);
    num2 = (num2 > 0 ? num2 : 0);
    return fm.stringExtensions.format("{0}={1}-{2}", fm.therest.range.getToken(), fm.intExtensions.toString(start), fm.intExtensions.toString(num2));
  };


  /*<span id='method-fm.therest.range-serializeResponseRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes a range for use as a response HTTP header value.
  	 </div>
  	@function serializeResponseRange
  	@param {fm.therest.range} range The range.
  	@return {String}
   */

  range.serializeResponseRange = function() {
    var range;
    range = arguments[0];
    return fm.stringExtensions.format("{0}/{1}", fm.therest.range.serializeRequestRange(range), fm.intExtensions.toString(range.getTotal()));
  };


  /*<span id='method-fm.therest.range-setToken'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets the token used in the Range header. Defaults to "items".
  	 </div>
  
  	@function setToken
  	@param {String} value
  	@return {void}
   */

  range.setToken = function() {
    var value;
    value = arguments[0];
    return range._token = value;
  };


  /*<span id='method-fm.therest.range-getLength'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets an integer value indicating the number of items.
  	 </div>
  
  	@function getLength
  	@return {Integer}
   */

  range.prototype.getLength = function() {
    return this._length;
  };


  /*<span id='method-fm.therest.range-getStart'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets an integer value indicating the index of the first item.
  	 </div>
  
  	@function getStart
  	@return {Integer}
   */

  range.prototype.getStart = function() {
    return this._start;
  };


  /*<span id='method-fm.therest.range-getTotal'>&nbsp;</span> */


  /**
  	 <div>
  	 Gets an integer value indicating the total number of items in the collection.
  	 </div>
  
  	@function getTotal
  	@return {Integer}
   */

  range.prototype.getTotal = function() {
    return this._total;
  };


  /*<span id='method-fm.therest.range-serializeRequestRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes this range for use as a request HTTP header value.
  	 </div>
  	@function serializeRequestRange
  	@return {String}
   */

  range.prototype.serializeRequestRange = function() {
    return fm.therest.range.serializeRequestRange(this);
  };


  /*<span id='method-fm.therest.range-serializeResponseRange'>&nbsp;</span> */


  /**
  	 <div>
  	 Serializes this range for use as a response HTTP header value.
  	 </div>
  	@function serializeResponseRange
  	@return {String}
   */

  range.prototype.serializeResponseRange = function() {
    return fm.therest.range.serializeResponseRange(this);
  };


  /*<span id='method-fm.therest.range-setLength'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets an integer value indicating the number of items.
  	 </div>
  
  	@function setLength
  	@param {Integer} value
  	@return {void}
   */

  range.prototype.setLength = function() {
    var value;
    value = arguments[0];
    return this._length = value;
  };


  /*<span id='method-fm.therest.range-setStart'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets an integer value indicating the index of the first item.
  	 </div>
  
  	@function setStart
  	@param {Integer} value
  	@return {void}
   */

  range.prototype.setStart = function() {
    var value;
    value = arguments[0];
    return this._start = value;
  };


  /*<span id='method-fm.therest.range-setTotal'>&nbsp;</span> */


  /**
  	 <div>
  	 Sets an integer value indicating the total number of items in the collection.
  	 </div>
  
  	@function setTotal
  	@param {Integer} value
  	@return {void}
   */

  range.prototype.setTotal = function() {
    var value;
    value = arguments[0];
    return this._total = value;
  };

  range.setToken("items");

  return range;

})(fm.object);


var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

fm.therest.restFormatUtility = (function(superClass) {
  extend(restFormatUtility, superClass);

  restFormatUtility._binaryContentType = null;

  restFormatUtility._jsonContentType = null;

  function restFormatUtility() {
    var instance;
    if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
      instance = restFormatUtility.__super__.constructor.call(this);
      fm.util.attachProperties(this, arguments[0]);
      return instance;
    }
    instance = restFormatUtility.__super__.constructor.call(this);
    return instance;
  }

  restFormatUtility.getFormat = function() {
    var contentType;
    contentType = arguments[0];
    if ((fm.global.equals(contentType, fm.therest.restFormatUtility._jsonContentType)) || fm.stringExtensions.startsWith(contentType, fm.stringExtensions.concat(fm.therest.restFormatUtility._jsonContentType, ";"))) {
      return fm.therest.restFormat.Json;
    }
    if ((fm.global.equals(contentType, fm.therest.restFormatUtility._binaryContentType)) || fm.stringExtensions.startsWith(contentType, fm.stringExtensions.concat(fm.therest.restFormatUtility._binaryContentType, ";"))) {
      return fm.therest.restFormat.Binary;
    }
    return fm.therest.restFormat.None;
  };

  restFormatUtility._jsonContentType = "application/json";

  restFormatUtility._binaryContentType = "application/octet-stream";

  return restFormatUtility;

})(fm.object);


(function() {
  var client, fn, j, len, methodName, ref;
  client = fm.therest.client;
  ref = ['send'];
  fn = function(methodName) {
    var method;
    method = client.prototype[methodName];
    return client.prototype[methodName] = function() {
      var i, k, language, len1, obj, ref1;
      if (arguments.length === 1 && fm.util.isPlainObject(arguments[0])) {
        obj = arguments[0];
        if (obj.range && fm.util.isPlainObject(obj.range)) {
          obj.range = new fm.therest.range(obj.range);
        }
        if (obj.language && fm.util.isPlainObject(obj.language)) {
          obj.language = new fm.therest.language(obj.language);
        }
        if (obj.languages) {
          ref1 = obj.languages;
          for (i = k = 0, len1 = ref1.length; k < len1; i = ++k) {
            language = ref1[i];
            if (fm.util.isPlainObject(obj.languages[i])) {
              obj.languages[i] = new fm.therest.language(obj.languages[i]);
            }
          }
        }
        if (obj.headers && fm.util.isPlainObject(obj.headers)) {
          obj.headers = new fm.nameValueCollection(obj.headers);
        }
        return method.call(this, new fm.therest[methodName + 'Args'](obj));
      } else {
        return method.apply(this, arguments);
      }
    };
  };
  for (j = 0, len = ref.length; j < len; j++) {
    methodName = ref[j];
    fn(methodName);
  }
  client.prototype.setDisableCORS = function(disableCORS) {
    return this._disableCORS = disableCORS;
  };
  client.prototype.getDisableCORS = function() {
    return this._disableCORS || false;
  };
  client.prototype.setDisablePostMessage = function(disablePostMessage) {
    return this._disablePostMessage = disablePostMessage;
  };
  client.prototype.getDisablePostMessage = function() {
    return this._disablePostMessage || false;
  };
  client.prototype.setDisableJSONP = function(disableJSONP) {
    return this._disableJSONP = disableJSONP;
  };
  client.prototype.getDisableJSONP = function() {
    return this._disableJSONP || false;
  };
  client.prototype.setForceJSONP = function(forceJSONP) {
    return this._forceJSONP = forceJSONP;
  };
  return client.prototype.getForceJSONP = function() {
    return this._forceJSONP || false;
  };
})();


return fm.therest;
}));