﻿var __slice = Array.prototype.slice;
(function ($) {
    var Sketch;
    $.fn.sketch = function () {
        var args, key, sketch;
        key = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
        if (this.length > 1) {
            $.error('Sketch.js can only be called on one element at a time.');
        }
        sketch = this.data('sketch');
        if (typeof key === 'string') {
            if (!sketch) return null;

            if (sketch[key]) {
                if (typeof sketch[key] === 'function') {
                    return sketch[key].apply(sketch, args);
                } else if (args.length === 0) {
                    return sketch[key];
                } else if (args.length === 1) {
                    return sketch[key] = args[0];
                }
            } else {
                return $.error('Sketch.js did not recognize the given command.');
            }
        } else if (sketch && !key) {
            return sketch;
        } else {
            this.data('sketch', new Sketch(this.get(0), key));
            return this;
        }
    };
    Sketch = (function () {
        function Sketch(el, opts) {
            this.el = el;
            this.canvas = $(el);
            this.contrails = true;
            this.context = el.getContext('2d');
            this.options = $.extend({
                toolLinks: true,
                defaultTool: 'marker',
                defaultColor: '#000000',
                defaultSize: 4,
                markerTimeout: 1000,
            }, opts);
            this.painting = false;
            this.color = this.options.defaultColor;
            this.size = this.options.defaultSize;
            this.tool = this.options.defaultTool;
            this.markerTimeout = this.options.markerTimeout;
            this.actions = [];
            this.action = [];
            this.canvas.bind('click mousedown mouseup mousemove mouseleave mouseout touchstart touchmove touchend touchcancel', this.onEvent);
            this.canvas.css({
                'cursor': 'crosshair'
            });
            if (this.options.toolLinks) {
                $('body').delegate("a[href=\"#" + (this.canvas.attr('id')) + "\"]", 'click', function (e) {
                    var $canvas, $this, key, sketch, _i, _len, _ref;
                    $this = $(this);
                    $canvas = $($this.attr('href'));
                    
                    sketch = $canvas.data('sketch');
                    _ref = ['color', 'size', 'tool'];
                    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                        key = _ref[_i];
                        if ($this.attr("data-" + key)) {
                            sketch.set(key, $(this).attr("data-" + key));
                        }
                    }
                    if ($(this).attr('data-download')) {
                        sketch.download($(this).attr('data-download'));
                    }
                    return false;
                });
            }
        }
        Sketch.prototype.download = function (format) {
            var mime;
            format || (format = "png");
            if (format === "jpg") {
                format = "jpeg";
            }
            mime = "image/" + format;
            return window.open(this.el.toDataURL(mime));
        };
        Sketch.prototype.set = function (key, value) {
            this[key] = value;
            return this.canvas.trigger("sketch.change" + key, value);
        };
        Sketch.prototype.startPainting = function () {
            this.painting = true;
            this.canvas.trigger('start');
            return this.action = {
                tool: this.tool,
                color: this.color,
                time: (new Date().getTime()),
                size: parseFloat(this.size),
                events: []
            };
        };
        Sketch.prototype.stopPainting = function () {
            if (!this.painting) {
                return;
            }

            if (this.action) {
                this.actions.push(this.action);
            }

            var closedOver = $.extend(true, {}, this.action);
            this.painting = false;
            this.action = null;

            var me = this;
            if (me.markerTimeout && me.markerTimeout > 0)
            {
                window.setTimeout(function () {
                    if (closedOver.tool == 'marker') {
                        $(me.el).trigger('clear');
                        $.sketch.tools.fader.draw.call(me, closedOver);
                    }
                }, me.markerTimeout);
            }

            $(me.el).trigger('stop');
            return this.redraw();
        };
        Sketch.prototype.onEvent = function (e) {
            if (e.originalEvent && e.originalEvent.targetTouches) {
                e.pageX = e.originalEvent.targetTouches[0].pageX;
                e.pageY = e.originalEvent.targetTouches[0].pageY;
            }
            $.sketch.tools[$(this).data('sketch').tool].onEvent.call($(this).data('sketch'), e);
            e.preventDefault();
            return false;
        };
        Sketch.prototype.clear = function () {
            this.actions.length = 0;
            this.el.getContext('2d').clearRect(0, 0, this.canvas.width(), this.canvas.height());
        };
        Sketch.prototype.redraw = function () {
            var sketch;
            this.el.width = this.canvas.width();
            this.context = this.el.getContext('2d');
            sketch = this;
            $.each(this.actions, function () {
                if (this.tool) {
                    return $.sketch.tools[this.tool].draw.call(sketch, this);
                }
            });
            if (this.painting && this.action) {
                return $.sketch.tools[this.action.tool].draw.call(sketch, this.action);
            }
        };
        Sketch.prototype.addLines = function (lines) {
            
            var remoteAction = {
                tool: this.tool,
                color: this.color,
                time: (new Date().getTime()),
                size: parseFloat(this.size),
                events: []
            };

            for (var i = 0, len = lines.length; i < len; i++) {
                var pt = {
                    X: lines[i].x,
                    Y: lines[i].y,
                    event: 'remote',
                    Type: 'remote',
                    W: this.canvas.width(),
                    H: this.canvas.height(),
                    Time: (new Date().getTime())
                };

                remoteAction.events.push(pt);
            }

            this.actions.push(remoteAction);
            this.redraw();
        };
        return Sketch;
    })();
    $.sketch = {
        tools: {}
    };
    $.sketch.tools.marker = {
        onEvent: function (e) {
            switch (e.type) {
                case 'mousedown':
                case 'touchstart':
                    this.startPainting();
                    break;
                case 'mouseup':
                case 'mouseout':
                case 'mouseleave':
                case 'touchend':
                case 'touchcancel':
                    this.stopPainting();
            }
            if (this.painting) {
                var pt = {
                    X: e.pageX - this.canvas.offset().left,
                    Y: e.pageY - this.canvas.offset().top,
                    event: e.type,
                    Type: e.type,
                    W: this.canvas.width(),
                    H: this.canvas.height(),
                    Time: (new Date().getTime())
                };
                pt.X = pt.X / pt.W;
                pt.Y = pt.Y / pt.H;
                this.action.events.push(pt);

                this.canvas.trigger('point', pt);

                return this.redraw();
            }
        },
        draw: function (action) {
            var translate = function (canvas, pt) {
                return {
                    X: pt.X * canvas.width,
                    Y: pt.Y * canvas.height,
                };
            }

            var event, previous, _i, _len, _ref;
            this.context.lineJoin = "round";
            this.context.lineCap = "round";
            this.context.beginPath();
            var trPt = translate(this.context.canvas, action.events[0]);
            this.context.moveTo(trPt.X, trPt.Y);
            _ref = action.events;
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                event = _ref[_i];
                var tPt = translate(this.context.canvas, event);
                this.context.lineTo(tPt.X, tPt.Y);
                previous = event;
            }
            this.context.strokeStyle = action.color;
            this.context.lineWidth = action.size;


            return this.context.stroke();
        }
    };
    $.sketch.tools.fader = {
        onEvent: function (e) {
            return $.sketch.tools.marker.onEvent.call(this, e);
        },
        draw: function (action) {
            var idx = -1;
            for (var i = 0; i < this.actions.length; i++){
                if(this.actions[i].time == action.time){
                    idx = i;
                    break;
                }
            }
            if (idx > -1) {
                this.actions.splice(idx, 1);
            }
            this.redraw();
            /*var oldcomposite;
            oldcomposite = this.context.globalCompositeOperation;
            this.context.globalCompositeOperation = "copy";
            var me = this;
            window.setTimeout(function () {
                action.color = "rgba(0,0,0,0)"; //TODO: fade
                $.sketch.tools.marker.draw.call(me, action);
            }, 10);

            return this.context.globalCompositeOperation = oldcomposite;*/
        }
    };
    return $.sketch.tools.eraser = {
        onEvent: function (e) {
            return $.sketch.tools.marker.onEvent.call(this, e);
        },
        draw: function (action) {
            var oldcomposite;
            oldcomposite = this.context.globalCompositeOperation;
            this.context.globalCompositeOperation = "copy";
            action.color = "rgba(0,0,0,0)";
            $.sketch.tools.marker.draw.call(this, action);
            return this.context.globalCompositeOperation = oldcomposite;
        }
    };
    
})(jQuery);